@@ -34,6 +34,8 @@ extern struct Hjava_lang_Class*	StringClass;
 extern struct Hjava_lang_Class*	SystemClass;
 extern struct Hjava_lang_Class*	SerialClass;
 extern struct Hjava_lang_Class*	CloneClass;
+extern struct Hjava_lang_Class* PtrClass;
+extern struct Hjava_lang_Class* ClassLoaderClass;
 
 extern struct Hjava_lang_Class*	javaLangVoidClass;
 extern struct Hjava_lang_Class*	javaLangBooleanClass;
@@ -45,4 +47,9 @@ extern struct Hjava_lang_Class*	javaLangLongClass;
 extern struct Hjava_lang_Class*	javaLangFloatClass;
 extern struct Hjava_lang_Class*	javaLangDoubleClass;
 
+extern struct Hjava_lang_Class* javaLangThrowable;
+extern struct Hjava_lang_Class*	javaLangNullPointerException;
+extern struct Hjava_lang_Class*	javaLangArithmeticException;
+extern struct Hjava_lang_Class* javaLangArrayIndexOutOfBoundsException;
+
 #endif
