@@ -28,6 +28,8 @@
 #include "locks.h"
 #include "thread.h"
 #include "jthread.h"
+#include "baseClasses.h"
+#include "soft.h"
 #include "md.h"
 #include "gc.h"
 
@@ -52,6 +54,8 @@ const uint8 insnLen[256] = {
 
 static void mergeFrame(codeinfo*, int, int, frameElement*, Method*);
 static bool verifyBasicBlock(codeinfo*, Method*, int32, errorInfo*);
+static void updateLocals(codeinfo*, int32, frameElement*);
+static bool verifyCatchClause(jexceptionEntry*, Hjava_lang_Class*, errorInfo*);
 
 bool
 verifyMethod(Method* meth, codeinfo **pcodeinfo, errorInfo *einfo)
@@ -61,7 +65,7 @@ verifyMethod(Method* meth, codeinfo **pcodeinfo, errorInfo *einfo)
 	int32 idx;
 	int32 sp;
 	int32 lcl;
-	const char* sig;
+	int count;
 	perPCInfo* bhead;
 	perPCInfo* btail;
 	perPCInfo* bcurr;
@@ -69,16 +73,41 @@ verifyMethod(Method* meth, codeinfo **pcodeinfo, errorInfo *einfo)
 	bool failed;
 	bool wide;
 	codeinfo *codeInfo;
-
-	/* We don't need to do this twice */
-	meth->accflags |= ACC_VERIFIED;
+	localUse* localuse;
 
 DBG(CODEANALYSE,
 	dprintf(__FUNCTION__ " %p: %s.%s\n", THREAD_NATIVE(), 
 		meth->class->name->data, meth->name->data);
     )
-	codeInfo = KMALLOC(sizeof(codeinfo) + (meth->c.bcode.codelen * sizeof(perPCInfo)));
+	codeInfo = KMALLOC(sizeof(codeinfo) + (meth->c.bcode.codelen *
+					       sizeof(perPCInfo)));
 	*pcodeinfo = codeInfo;
+	if (!codeInfo) {
+		postOutOfMemory(einfo);
+		return false;
+	}
+	/* Allocate space for local register info - we add in an extra one
+	 * to avoid mallocing 0 bytes.
+	 */
+	localuse = KCALLOC(sizeof(localUse), meth->localsz+1);
+	if (!localuse) {
+		KFREE(codeInfo);
+		postOutOfMemory(einfo);
+		return false;
+	}
+	codeInfo->localuse = localuse;
+
+	/* We don't need to do this twice */
+	meth->accflags |= ACC_VERIFIED;
+
+	for (lcl = 0; lcl < meth->localsz; lcl++) {
+		localuse = codeInfo->localuse;
+		localuse[lcl].use = 0;
+		localuse[lcl].first = 0x7FFFFFFF;
+		localuse[lcl].last = -1;
+		localuse[lcl].write = -1;
+		localuse[lcl].type = 0;
+	}
 
 DBG(CODEANALYSE,
 	dprintf(__FUNCTION__" %p: codeInfo = %p\n", THREAD_NATIVE(), codeInfo);	
@@ -223,12 +252,23 @@ DBG(CODEANALYSE,
 	sp = meth->localsz + meth->stacksz - 1;
 	if (meth->exception_table != 0) {
 		for (lcl = 0; lcl < meth->exception_table->length; lcl++) {
-			pc = meth->exception_table->entry[lcl].handler_pc;
+			bool succ;
+			jexceptionEntry *entry;
+			
+			entry = &(meth->exception_table->entry[lcl]);
+			
+			/* Verify catch clause exception has valid type. */
+			succ = verifyCatchClause(entry, meth->class, einfo);
+			if (succ == false) {
+				return false;
+			}
+
+			pc = entry->handler_pc;
+			ATTACH_NEW_BASICBLOCK(pc);
 			SET_STARTOFEXCEPTION(pc);
 			SET_STACKPOINTER(pc, sp);
 			SET_NEWFRAME(pc);
 			STACKINIT(0, TOBJ);
-			ATTACH_NEW_BASICBLOCK(pc);
 		}
 	}
 
@@ -247,31 +287,13 @@ DBG(CODEANALYSE,
 		LOCALINIT(0, TOBJ);
 		idx++;
 	}
-	sig = meth->signature->data;
-	assert(sig[0] == '(');
-	sig++;
-	while (sig[0] != ')') {
-		switch (sig[0]) {
+
+	for (count = 0; count < METHOD_NARGS(meth); ++count) {
+		switch (*METHOD_ARG_TYPE(meth, count)) {
+		case 'L':
 		case '[':
 			LOCALINIT(idx, TOBJ);
 			idx += 1;
-			while (sig[0] == '[') {
-				sig++;
-			}
-			if (sig[0] == 'L') {
-				while (sig[0] != ';') {
-					sig++;
-				}
-			}
-			sig++;
-			break;
-		case 'L':
-			LOCALINIT(idx, TOBJ);
-			idx += 1;
-			while (sig[0] != ';') {
-				sig++;
-			}
-			sig++;
 			break;
 		case 'I':
 		case 'Z':
@@ -280,24 +302,20 @@ DBG(CODEANALYSE,
 		case 'C':
 			LOCALINIT(idx, TINT);
 			idx += 1;
-			sig++;
 			break;
 		case 'J':
 			LOCALINIT(idx, TLONG);
 			LOCALINIT(idx+1, TVOID);
 			idx += 2;
-			sig++;
 			break;
 		case 'F':
 			LOCALINIT(idx, TFLOAT);
 			idx += 1;
-			sig++;
 			break;
 		case 'D':
 			LOCALINIT(idx, TDOUBLE);
 			LOCALINIT(idx+1, TVOID);
 			idx += 2;
-			sig++;
 			break;
 		default:
 			assert("Signature character unknown" == 0);
@@ -330,7 +348,7 @@ DBG(CODEANALYSE,
 #if VDBG(1) - 1 == 0
 	for (bcurr = bhead; bcurr != NULL; bcurr = bcurr->nextBB) {
 		if ((bcurr->flags & FLAG_DONEVERIFY) == 0) {
-			VDBG(dprintf("%s.%s%s pc %d bcurr->flags 0x%04x\n", meth->class->name->data, meth->name->data, meth->signature->data, bcurr - codeInfo->perPC, bcurr->flags);)
+			VDBG(dprintf("%s.%s%s pc %d bcurr->flags 0x%04x\n", meth->class->name->data, meth->name->data, METHOD_SIGD(meth), bcurr - codeInfo->perPC, bcurr->flags);)
 		}
 	}
 #endif
@@ -344,31 +362,40 @@ verifyBasicBlock(codeinfo* codeInfo, Method* meth, int32 pc, errorInfo *einfo)
 	int32 tabpc;
 	int32 idx;
 	int32 sp;
-	int32 lcl;
-	int32 lclw;
-	int32 lclww;
 	int32 opc;
 	callInfo call;
 	fieldInfo finfo;
 	Hjava_lang_Class* type;
 	const char* sig;
-	Hjava_lang_Class* class;
 	frameElement* activeFrame;
 	bool wide;
 	bool failed;
+	bool firsttime;
 
 	opc = pc;
 	assert(pc == 0 || IS_STARTOFBASICBLOCK(pc) || IS_STARTOFEXCEPTION(pc));
 	assert(IS_STACKPOINTERSET(pc));
 
-	SET_DONEVERIFY(pc);
+	/* If this block hasn't been verified before then note this */
+	if (!IS_DONEVERIFY(pc)) {
+		firsttime = true;
+	}
+	else {
+		firsttime = false;
+	}
 
 	/* Get stack pointer */
 	sp = STACKPOINTER(pc);
 
 	/* Allocate frame to hold type data */
 	activeFrame = ALLOCFRAME();
-	assert(activeFrame != 0);
+	if (activeFrame == 0) {
+		postOutOfMemory(einfo);
+		return true;
+	}
+
+	SET_DONEVERIFY(pc);
+
 	FRAMELOAD(pc);
 
 	/* Process basic block until we get to the beginning of a new one */
@@ -378,7 +405,10 @@ verifyBasicBlock(codeinfo* codeInfo, Method* meth, int32 pc, errorInfo *einfo)
 
 		if (sp < meth->localsz || sp > meth->localsz + meth->stacksz) {
 			failed = true;
-			VDBG(dprintf("sp out of range: %d <%d> %d\n", meth->localsz, sp, meth->localsz + meth->stacksz);)
+			postExceptionMessage(einfo, JAVA_LANG(VerifyError),
+				"sp out of range: %d <%d> %d\n", 
+				meth->localsz, sp, 
+				meth->localsz + meth->stacksz);
 			break;
 		}
 
@@ -393,14 +423,11 @@ verifyBasicBlock(codeinfo* codeInfo, Method* meth, int32 pc, errorInfo *einfo)
 			}
 		}
 
-		/* Retrieve a potential local slot */
-		lcl = BYTE(pc+1);
-		lclw = WORD(pc+1);
-		lclww = DWORD(pc+1);
-
 IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 
 		switch (INSN(pc)) {
+		int32 lcl;
+
 		case NOP:
 			INCPC(1);
 			break;
@@ -419,20 +446,19 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 		case ICONST_4:
 		case ICONST_5:
 			STKPUSH(1);
-			lcl = INSN(pc) - ICONST_0;
-			STACKOUT_CONST(0, TINT, lcl);
+			STACKOUT_CONST(0, TINT, INSN(pc) - ICONST_0);
 			INCPC(1);
 			break;
 
 		case BIPUSH:
 			STKPUSH(1);
-			STACKOUT_CONST(0, TINT, lcl);
+			STACKOUT_CONST(0, TINT, BYTE(pc+1));
 			INCPC(2);
 			break;
 
 		case SIPUSH:
 			STKPUSH(1);
-			STACKOUT_CONST(0, TINT, lclw);
+			STACKOUT_CONST(0, TINT, WORD(pc+1));
 			INCPC(3);
 			break;
 
@@ -462,7 +488,7 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 
 		case LDC1:
 			STKPUSH(1);
-			lcl = (uint8)lcl;
+			lcl = (uint8)BYTE(pc + 1);
 			CONSTANTTYPE(type, lcl);
 			STACKOUT(0, type);
 			INCPC(2);
@@ -470,14 +496,14 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 
 		case LDC2:
 			STKPUSH(1);
-			CONSTANTTYPE(type, (uint16)lclw);
+			CONSTANTTYPE(type, (uint16)WORD(pc+1));
 			STACKOUT(0, type);
 			INCPC(3);
 			break;
 
 		case LDC2W:
 			STKPUSH(2);
-			CONSTANTTYPE(type, (uint16)lclw);
+			CONSTANTTYPE(type, (uint16)WORD(pc+1));
 			STACKOUT(0, type);
 			STACKOUT(1, TVOID);
 			INCPC(3);
@@ -497,12 +523,12 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			if (wide) {
 				wide = false;
 				STKPUSH(1);
-				STACKOUT_LOCAL(0, TINT, lclw);
+				STACKOUT_LOCAL(0, TINT, WORD(pc+1));
 				INCPC(3);
 			}
 			else {
 				STKPUSH(1);
-				STACKOUT_LOCAL(0, TINT, lcl);
+				STACKOUT_LOCAL(0, TINT, BYTE(pc+1));
 				INCPC(2);
 			}
 			break;
@@ -522,14 +548,14 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			if (wide) {
 				wide = false;
 				STKPUSH(2);
-				STACKOUT_LOCAL(0, TLONG, lclw);
-				STACKOUT_LOCAL(1, TVOID, lclw+1);
+				STACKOUT_LOCAL(0, TLONG, WORD(pc+1));
+				STACKOUT_LOCAL(1, TVOID, WORD(pc+1)+1);
 				INCPC(3);
 			}
 			else {
 				STKPUSH(2);
-				STACKOUT_LOCAL(0, TLONG, lcl);
-				STACKOUT_LOCAL(1, TVOID, lcl+1);
+				STACKOUT_LOCAL(0, TLONG, BYTE(pc+1));
+				STACKOUT_LOCAL(1, TVOID, BYTE(pc+1)+1);
 				INCPC(2);
 			}
 			break;
@@ -548,12 +574,12 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			if (wide) {
 				wide = false;
 				STKPUSH(1);
-				STACKOUT_LOCAL(0, TFLOAT, lclw);
+				STACKOUT_LOCAL(0, TFLOAT, WORD(pc+1));
 				INCPC(3);
 			}
 			else {
 				STKPUSH(1);
-				STACKOUT_LOCAL(0, TFLOAT, lcl);
+				STACKOUT_LOCAL(0, TFLOAT, BYTE(pc+1));
 				INCPC(2);
 			}
 			break;
@@ -571,13 +597,15 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 
 		case DLOAD:
 			if (wide) {
+				lcl = WORD(pc+1);
 				wide = false;
 				STKPUSH(2);
-				STACKOUT_LOCAL(0, TDOUBLE, lclw);
-				STACKOUT_LOCAL(1, TVOID, lclw+1);
+				STACKOUT_LOCAL(0, TDOUBLE, lcl);
+				STACKOUT_LOCAL(1, TVOID, lcl+1);
 				INCPC(3);
 			}
 			else {
+				lcl = BYTE(pc+1);
 				STKPUSH(2);
 				STACKOUT_LOCAL(0, TDOUBLE, lcl);
 				STACKOUT_LOCAL(1, TVOID, lcl+1);
@@ -599,12 +627,12 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			if (wide) {
 				wide = false;
 				STKPUSH(1);
-				STACKOUT_LOCAL(0, TOBJ, lclw);
+				STACKOUT_LOCAL(0, TOBJ, WORD(pc+1));
 				INCPC(3);
 			}
 			else {
 				STKPUSH(1);
-				STACKOUT_LOCAL(0, TOBJ, lcl);
+				STACKOUT_LOCAL(0, TOBJ, BYTE(pc+1));
 				INCPC(2);
 			}
 			break;
@@ -664,12 +692,12 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 
 		case ISTORE:
 			if (wide) {
-				LOCALOUT_STACK(lclw, TINT, 0);
+				LOCALOUT_STACK(WORD(pc+1), TINT, 0);
 				INCPC(1);
 				wide = false;
 			}
 			else {
-				LOCALOUT_STACK(lcl, TINT, 0);
+				LOCALOUT_STACK(BYTE(pc+1), TINT, 0);
 			}
 			STKPOP(1);
 			INCPC(2);
@@ -688,12 +716,14 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 
 		case LSTORE:
 			if (wide) {
-				LOCALOUT_STACK(lclw, TLONG, 0);
-				LOCALOUT_STACK(lclw+1, TVOID, 1);
+				lcl = WORD(pc+1);
+				LOCALOUT_STACK(lcl, TLONG, 0);
+				LOCALOUT_STACK(lcl+1, TVOID, 1);
 				INCPC(1);
 				wide = false;
 			}
 			else {
+				lcl = BYTE(pc+1);
 				LOCALOUT_STACK(lcl, TLONG, 0);
 				LOCALOUT_STACK(lcl+1, TVOID, 1);
 			}
@@ -715,12 +745,12 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 		case FSTORE:
 			STACKIN(0, TFLOAT);
 			if (wide) {
-				LOCALOUT(lclw, TFLOAT);
+				LOCALOUT(WORD(pc+1), TFLOAT);
 				INCPC(1);
 				wide = false;
 			}
 			else {
-				LOCALOUT(lcl, TFLOAT);
+				LOCALOUT(BYTE(pc+1), TFLOAT);
 			}
 			STKPOP(1);
 			INCPC(2);
@@ -743,14 +773,14 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			STACKIN(0, TDOUBLE);
 			STACKIN(1, TVOID);
 			if (wide) {
-				LOCALOUT(lclw, TDOUBLE);
-				LOCALOUT(lclw+1, TVOID);
+				LOCALOUT(WORD(pc+1), TDOUBLE);
+				LOCALOUT(WORD(pc+1)+1, TVOID);
 				INCPC(1);
 				wide = false;
 			}
 			else {
-				LOCALOUT(lcl, TDOUBLE);
-				LOCALOUT(lcl+1, TVOID);
+				LOCALOUT(BYTE(pc+1), TDOUBLE);
+				LOCALOUT(BYTE(pc+1)+1, TVOID);
 			}
 			STKPOP(2);
 			INCPC(2);
@@ -768,12 +798,12 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 
 		case ASTORE:
 			if (wide) {
-				LOCALOUT_STACK(lclw, TOBJ, 0);
+				LOCALOUT_STACK(WORD(pc+1), TOBJ, 0);
 				INCPC(1);
 				wide = false;
 			}
 			else {
-				LOCALOUT_STACK(lcl, TOBJ, 0);
+				LOCALOUT_STACK(BYTE(pc+1), TOBJ, 0);
 			}
 			STKPOP(1);
 			INCPC(2);
@@ -1081,12 +1111,12 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 
 		case IINC:
 			if (wide) {
-				LOCALINOUT(lclw, TINT);
+				LOCALINOUT(WORD(pc+1), TINT);
 				INCPC(2);
 				wide = false;
 			}
 			else {
-				LOCALINOUT(lcl, TINT);
+				LOCALINOUT(BYTE(pc+1), TINT);
 			}
 			INCPC(3);
 			break;
@@ -1229,7 +1259,7 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 		case IFLE:
 			STACKIN(0, TINT);
 			STKPOP(1);
-			FRAMEMERGE(pc + lclw, sp);
+			FRAMEMERGE(pc + WORD(pc+1), sp);
 			FRAMEMERGE(pc + 3, sp);
 			INCPC(3);
 			break;
@@ -1243,7 +1273,7 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			STACKIN(0, TINT);
 			STACKIN(1, TINT);
 			STKPOP(2);
-			FRAMEMERGE(pc + lclw, sp);
+			FRAMEMERGE(pc + WORD(pc+1), sp);
 			FRAMEMERGE(pc + 3, sp);
 			INCPC(3);
 			break;
@@ -1253,25 +1283,25 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			STACKIN(0, TOBJ);
 			STACKIN(1, TOBJ);
 			STKPOP(2);
-			FRAMEMERGE(pc + lclw, sp);
+			FRAMEMERGE(pc + WORD(pc+1), sp);
 			FRAMEMERGE(pc + 3, sp);
 			INCPC(3);
 			break;
 
 		case GOTO:
-			FRAMEMERGE(pc + lclw, sp);
+			FRAMEMERGE(pc + WORD(pc+1), sp);
 			INCPC(3);
 			break;
 
 		case GOTO_W:
-			FRAMEMERGE(pc + lclww, sp);
+			FRAMEMERGE(pc + DWORD(pc+1), sp);
 			INCPC(5);
 			break;
 
 		case JSR:
 			STKPUSH(1);
 			STACKOUT(0, TADDR);
-			FRAMEMERGE(pc + lclw, sp);
+			FRAMEMERGE(pc + WORD(pc+1), sp);
 			STKPOP(1);
 			FRAMEMERGE(pc + 3, sp);
 			INCPC(3);
@@ -1280,14 +1310,14 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 		case JSR_W:
 			STKPUSH(1);
 			STACKOUT(0, TADDR);
-			FRAMEMERGE(pc + lclww, sp);
+			FRAMEMERGE(pc + DWORD(pc+1), sp);
 			STKPOP(1);
 			FRAMEMERGE(pc + 5, sp);
 			INCPC(5);
 			break;
 
 		case RET:
-			LOCALIN(lcl, TADDR);
+			LOCALIN(BYTE(pc+1), TADDR);
 			INCPC(2);
 			break;
 
@@ -1352,11 +1382,11 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			break;
 
 		case GETSTATIC:
-			if (getField(lclw, meth->class, true, &finfo, einfo) == 0) {
+			if (getField(WORD(pc+1), meth->class, true, &finfo, einfo) == 0) {
 				failed = true;
 				goto done;
 			}
-			if (FIELD_ISREF(finfo.field)) {
+			if (!FIELD_ISPRIM(finfo.field)) {
 				STKPUSH(1);
 				STACKOUT(0, TOBJ);
 			}
@@ -1391,11 +1421,11 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			break;
 
 		case PUTSTATIC:
-			if (getField(lclw, meth->class, true, &finfo, einfo) == 0) {
+			if (getField(WORD(pc+1), meth->class, true, &finfo, einfo) == 0) {
 				failed = true;
 				goto done;
 			}
-			if (FIELD_ISREF(finfo.field)) {
+			if (!FIELD_ISPRIM(finfo.field)) {
 				STACKIN(0, TOBJ);
 				STKPOP(1);
 			}
@@ -1430,12 +1460,12 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			break;
 
 		case GETFIELD:
-			if (getField(lclw, meth->class, false, &finfo, einfo) == 0) {
+			if (getField(WORD(pc+1), meth->class, false, &finfo, einfo) == 0) {
 				failed = true;
 				goto done;
 			}
 			STACKIN(0, TOBJ);
-			if (FIELD_ISREF(finfo.field)) {
+			if (!FIELD_ISPRIM(finfo.field)) {
 				STACKOUT(0, TOBJ);
 			}
 			else switch (CLASS_PRIM_SIG(FIELD_TYPE(finfo.field))){
@@ -1467,11 +1497,11 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			break;
 
 		case PUTFIELD:
-			if (getField(lclw, meth->class, false, &finfo, einfo) == 0) {
+			if (getField(WORD(pc+1), meth->class, false, &finfo, einfo) == 0) {
 				failed = true;
 				goto done;
 			}
-			if (FIELD_ISREF(finfo.field)) {
+			if (!FIELD_ISPRIM(finfo.field)) {
 				STACKIN(0, TOBJ);
 				STACKIN(1, TOBJ);
 				STKPOP(2);
@@ -1512,9 +1542,11 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 
 		case INVOKEVIRTUAL:
 		case INVOKESPECIAL:
-			if (getMethodSignatureClass(lclw, meth->class, true, false, &call, einfo) == false) {
-				failed = true;
-				goto done;
+			if (getMethodSignatureClass(WORD(pc+1), meth->class, true, false, &call, einfo) == false) {
+				if (!checkNoClassDefFoundError(einfo) || call.signature == 0) {
+					failed = true;
+					goto done;
+				}
 			}
 
 			sig = call.signature->data;
@@ -1613,9 +1645,11 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			break;
 
 		case INVOKEINTERFACE:
-			if (getMethodSignatureClass(lclw, meth->class, true, false, &call, einfo) == false) {
-				failed = true;
-				goto done;
+			if (getMethodSignatureClass(WORD(pc+1), meth->class, true, false, &call, einfo) == false) {
+				if (!checkNoClassDefFoundError(einfo) || call.signature == 0) {
+					failed = true;
+					goto done;
+				}
 			}
 
 			sig = call.signature->data;
@@ -1713,9 +1747,11 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			break;
 
 		case INVOKESTATIC:
-			if (getMethodSignatureClass(lclw, meth->class, true, false, &call, einfo) == false) {
-				failed = true;
-				goto done;
+			if (getMethodSignatureClass(WORD(pc+1), meth->class, true, false, &call, einfo) == false) {
+				if (!checkNoClassDefFoundError(einfo) || call.signature == 0) {
+					failed = true;
+					goto done;
+				}
 			}
 
 			sig = call.signature->data;
@@ -1811,10 +1847,11 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			break;
 
 		case NEW:
-			class = getClass(lclw, meth->class, einfo);
-			if (class == 0) {
-				failed = true;
-				goto done;
+			if (getClass(WORD(pc+1), meth->class, einfo) == 0) {
+				if (!checkNoClassDefFoundError(einfo)) {
+					failed = true;
+					goto done;
+				}
 			}
 			STKPUSH(1);
 			STACKOUT(0, TOBJ);
@@ -1828,10 +1865,11 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			break;
 
 		case ANEWARRAY:
-			class = getClass(lclw, meth->class, einfo);
-			if (class == 0) {
-				failed = true;
-				goto done;
+			if (getClass(WORD(pc+1), meth->class, einfo) == 0) {
+				if (!checkNoClassDefFoundError(einfo)) {
+					failed = true;
+					goto done;
+				}
 			}
 			STACKIN(0, TINT);
 			STACKOUT(0, TOBJ);
@@ -1839,10 +1877,11 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			break;
 
 		case MULTIANEWARRAY:
-			class = getClass(lclw, meth->class, einfo);
-			if (class == 0) {
-				failed = true;
-				goto done;
+			if (getClass(WORD(pc+1), meth->class, einfo) == 0) {
+				if (!checkNoClassDefFoundError(einfo)) {
+					failed = true;
+					goto done;
+				}
 			}
 			for (idx = INSN(pc+3) - 1; idx >= 0; idx--) {
 				STACKIN(idx, TINT);
@@ -1865,10 +1904,11 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			break;
 
 		case CHECKCAST:
-			class = getClass(lclw, meth->class, einfo);
-			if (class == 0) {
-				failed = true;
-				goto done;
+			if (getClass(WORD(pc+1), meth->class, einfo) == 0) {
+				if (!checkNoClassDefFoundError(einfo)) {
+					failed = true;
+					goto done;
+				}
 			}
 			/* SET_INSN(pc, CHECKCAST_FAST); */
 			STACKIN(0, TOBJ);
@@ -1877,10 +1917,11 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 			break;
 
 		case INSTANCEOF:
-			class = getClass(lclw, meth->class, einfo);
-			if (class == 0) {
-				failed = true;
-				goto done;
+			if (getClass(WORD(pc+1), meth->class, einfo) == 0) {
+				if (!checkNoClassDefFoundError(einfo)) {
+					failed = true;
+					goto done;
+				}
 			}
 			/* SET_INSN(pc, INSTANCEOF_FAST); */
 			STACKIN(0, TOBJ);
@@ -1899,7 +1940,7 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 		case IFNONNULL:
 			STACKIN(0, TOBJ);
 			STKPOP(1);
-			FRAMEMERGE(pc + lclw, sp);
+			FRAMEMERGE(pc + WORD(pc+1), sp);
 			FRAMEMERGE(pc + 3, sp);
 			INCPC(3);
 			break;
@@ -1919,9 +1960,14 @@ IDBG(		dprintf("%d: %d\n", pc, INSN(pc));		)
 	 * and merge in the frame.
 	 */
 	if (pc < meth->c.bcode.codelen && IS_NORMALFLOW(pc)) {
+		assert(IS_STARTOFBASICBLOCK(pc) || IS_STARTOFEXCEPTION(pc));
 		FRAMEMERGE(pc, sp);
 	}
 
+	if (firsttime) {
+		updateLocals(codeInfo, opc, activeFrame);
+	}
+
 done:
 	/* Discard active frame */
 	KFREE(activeFrame);
@@ -1944,7 +1990,7 @@ mergeFrame(codeinfo* codeInfo, int pc, int sp, frameElement* from, Method* meth)
 
 	/* Merge locals */
 	for (m = 0; m < meth->localsz; m++) {
-		if (from[m].type != TUNASSIGNED && from[m].type != to[m].type) {
+		if (from[m].type != TUNASSIGNED && from[m].type != to[m].type && to[m].type != TUNSTABLE) {
 			SET_NEEDVERIFY(pc);
 			if (to[m].type == TUNASSIGNED) {
 				to[m].type = from[m].type;
@@ -1957,7 +2003,7 @@ mergeFrame(codeinfo* codeInfo, int pc, int sp, frameElement* from, Method* meth)
 
 	/* Merge stacks */
 	for (m = sp; m < meth->localsz + meth->stacksz; m++) {
-		if (from[m].type != TUNASSIGNED && from[m].type != to[m].type) {
+		if (from[m].type != TUNASSIGNED && from[m].type != to[m].type && to[m].type != TUNSTABLE) {
 			SET_NEEDVERIFY(pc);
 			if (to[m].type == TUNASSIGNED) {
 				to[m].type = from[m].type;
@@ -1973,20 +2019,102 @@ mergeFrame(codeinfo* codeInfo, int pc, int sp, frameElement* from, Method* meth)
  * Tidy up after verfication data has been finished with.
  */
 void
-tidyVerifyMethod(codeinfo *codeInfo)
+tidyVerifyMethod(codeinfo** codeInfo)
 {
 	int pc;
 
 	/* Free the old data */
-	for (pc = 0; pc < codeInfo->codelen; pc++) {
-		if (codeInfo->perPC[pc].frame != 0) {
-			KFREE(codeInfo->perPC[pc].frame);
+	if (!*codeInfo) {
+		return;
+	}
+	for (pc = 0; pc < (*codeInfo)->codelen; pc++) {
+		if ((*codeInfo)->perPC[pc].frame != 0) {
+			KFREE((*codeInfo)->perPC[pc].frame);
 		}
 	}
-	KFREE(codeInfo);
+	KFREE((*codeInfo)->localuse);
+	KFREE(*codeInfo);
+	*codeInfo = 0;
 DBG(CODEANALYSE,
 	dprintf(__FUNCTION__" %p: clearing codeInfo %p\n", 
 		THREAD_NATIVE(), codeInfo);
     )
 }
 
+static
+void
+updateLocals(codeinfo* codeInfo, int32 pc, frameElement* frame)
+{
+	int i;
+	localUse* l;
+
+	for (i = 0; i < codeInfo->localsz; i++) {
+		if (frame[i].used == 0) {
+			continue;
+		}
+		l = &codeInfo->localuse[i];
+		if (pc < l->first) {
+			l->first = pc;
+		}
+		if (pc > l->last) {
+			l->last = pc;
+		}
+		if (frame[i].modified != 0 && pc > l->write) {
+			l->write = pc;
+		}
+		l->use++;
+		if (l->type == TUNASSIGNED) {
+			l->type = frame[i].type;
+		}
+		else if (frame[i].type == TUNASSIGNED || l->type == frame[i].type) {
+			/* Do nothing */
+		}
+		else {
+			l->type = TUNSTABLE;
+		}
+	}
+}
+
+
+
+/*
+ * Verification requires us to resolve the catch type and check that
+ * its actually a Throwable.  But, we also do it to ensure that the
+ * classes are ready when we need them and have little room to work
+ * (e.g. a stack overflow).
+ */
+static bool
+verifyCatchClause(jexceptionEntry* eptr, Hjava_lang_Class* class, errorInfo *einfo)
+{
+	if( eptr->catch_idx == 0 ) {
+		/* A finally clause... */
+	} else {
+		eptr->catch_type =
+			getClass(eptr->catch_idx, class, einfo);
+
+		/*
+		 * If we could not resolve the catch class, then we
+		 * must record that fact to guard against possible
+		 * recursive attempts to load it.  Throw whatever
+		 * error getClass() generated.
+		 */
+		if (eptr->catch_type == NULL) {
+			eptr->catch_type =
+				UNRESOLVABLE_CATCHTYPE;
+			
+			/* Pass on einfo generated by getClass() */
+			return false;
+		}
+		
+		/*
+		 * Make sure the exception is a subclass of Throwable.
+		 */
+		if( !instanceof(javaLangThrowable, eptr->catch_type) ) {
+			postException(einfo, JAVA_LANG(VerifyError));
+			return false;
+		}
+	}
+
+	/* Catch clause is okay. */
+	return true;
+}
