@@ -43,6 +43,7 @@
 #include "soft.h"
 #include "thread.h"
 #include "gcRefs.h"
+#include "methodCache.h"
 
 #if GUARANA
 #include "GuaraNative.h"
@@ -59,17 +60,15 @@ static void
 /* ARGSUSED */
 destroyClass(Collector *collector, void* c)
 {
-        int i;
+        int i, j;
 	int idx;
 	Hjava_lang_Class* clazz = c;
 	constants* pool;
-	extern void checkClass(Hjava_lang_Class*, Hjava_lang_ClassLoader*);
 
 DBG(CLASSGC,
         dprintf("destroying class %s @ %p\n", 
 		clazz->name ? clazz->name->data : "newborn", c);
    )
-	checkClass(clazz, clazz->loader);	/* temporary! */
 	assert(!CLASS_IS_PRIMITIVE(clazz));
 
 	/* NB: Make sure that we don't unload fully loaded classes without
@@ -82,7 +81,7 @@ DBG(CLASSGC,
 	assert(clazz->state != CSTATE_COMPLETE || clazz->loader != 0);
 
 	if (Kaffe_JavaVMArgs[0].enableVerboseGC > 0 && clazz->name) {
-		fprintf(stderr, "<GC: unloading class `%s'>\n", 
+		dprintf("<GC: unloading class `%s'>\n", 
 			CLASS_CNAME(clazz));
 	}
 
@@ -105,18 +104,37 @@ DBG(CLASSGC,
         if (!CLASS_IS_ARRAY(clazz) && CLASS_METHODS(clazz) != 0) {
                 Method *m = CLASS_METHODS(clazz);
                 for (i = 0; i < CLASS_NMETHODS(clazz); i++) {
+			void *ncode = 0;
+
+			if (!CLASS_IS_INTERFACE(clazz))
+			{
+				ncode = METHOD_NATIVECODE(m);
+#if defined(TRANSLATOR) && (defined (MD_UNREGISTER_JIT_EXCEPTION_INFO) || defined (JIT3))
+				if (METHOD_JITTED(m)) {
+#if defined(MD_UNREGISTER_JIT_EXCEPTION_INFO)
+					MD_UNREGISTER_JIT_EXCEPTION_INFO (m->c.ncode.ncode_start,
+									  ncode,
+									  m->c.ncode.ncode_end);
+#endif
+#if defined(JIT3)
+					makeMethodInactive(m);
+#endif
+				}
+#endif
+			}
                         utf8ConstRelease(m->name);
-                        utf8ConstRelease(m->signature);
+                        utf8ConstRelease(METHOD_SIG(m));
+                        KFREE(METHOD_PSIG(m));
                         KFREE(m->lines);
                         KFREE(m->declared_exceptions);
                         KFREE(m->exception_table);
                         KFREE(m->c.bcode.code);	 /* aka c.ncode.ncode_start */
 
-			/* Free ncode if necessary: this only concerns
-			 * trampolines for interface <clinit> methods
+			/* Free ncode if necessary: this concerns
+			 * any uninvoked trampolines
 			 */
-			if (GC_getObjectIndex(collector, m->ncode) != -1) {
-				KFREE(m->ncode);
+			if (GC_getObjectIndex(collector, ncode) != -1) {
+				KFREE(ncode);
 			}
 			m++;
                 }
@@ -140,11 +158,55 @@ DBG(CLASSGC,
 
         /* free various other fixed things */
         KFREE(CLASS_STATICDATA(clazz));
-        KFREE(clazz->dtable);
+	if( clazz->dtable )
+	{
+		for( i = 0; i < clazz->msize; i++ )
+		{
+			if( DMETHOD_ADDR(clazz->dtable->method[i]) == 0 )
+				continue;
+			/* Free ncode if necessary: this concerns
+			 * any uninvoked trampolines
+			 */
+			if (GC_getObjectIndex(collector,
+					      DMETHOD_ADDR(clazz->dtable->method[i]))
+			    == GC_ALLOC_DISPATCHTABLE) {
+				KFREE(DMETHOD_ADDR(clazz->dtable->method[i]));
+			}
+		}
+		KFREE(clazz->dtable);
+	}
         KFREE(clazz->if2itable);
-        KFREE(clazz->itable2dtable);
+	if( clazz->itable2dtable )
+	{
+		for (i = 0; i < clazz->total_interface_len; i++) {
+			Hjava_lang_Class* iface = clazz->interfaces[i];
+
+			/* only if interface has not been freed already */
+			if (GC_getObjectIndex(collector, iface) 
+			    == GC_ALLOC_CLASSOBJECT) 
+			{
+				iface->implementors[clazz->impl_index] = -1;
+			}
+		}
+
+		/* NB: we can't just sum up the msizes of the interfaces
+		 * here because they might be destroyed simultaneously
+		 */
+		j = GC_getObjectSize(collector, clazz->itable2dtable) 
+			/ sizeof (void*);
+		for( i = 0; i < j; i++ )
+		{
+			if (GC_getObjectIndex(collector,
+					      DMETHOD_ADDR(clazz->itable2dtable[i]))
+			    == GC_ALLOC_DISPATCHTABLE) {
+				GC_free(collector, DMETHOD_ADDR(clazz->itable2dtable[i]));
+			}
+		}
+		GC_free(collector, clazz->itable2dtable);
+	}
         KFREE(clazz->gc_layout);
 	KFREE(clazz->sourcefile);
+	KFREE(clazz->implementors);
 
         /* The interface table for array classes points to static memory */
         if (!CLASS_IS_ARRAY(clazz)) {
@@ -161,7 +223,7 @@ void
 walkMethods(Collector* collector, Method* m, int nm)
 {               
         while (nm-- > 0) {
-#if defined(TRANSLATOR)
+#if defined(TRANSLATOR) && 0
                 /* walk the block of jitted code conservatively.
                  * Is this really needed?
  		 */
@@ -206,6 +268,10 @@ walkClass(Collector* collector, void* base, uint32 size)
 
         class = (Hjava_lang_Class*)base;
 
+DBG(GCPRECISE,
+        dprintf("walkClass `%s' state=%d\n", CLASS_CNAME(class), class->state);
+    )   
+
 #if GUARANA
 	GC_markObject(collector, class->head.meta_object);
 #endif
@@ -251,7 +317,9 @@ walkClass(Collector* collector, void* base, uint32 size)
                         if (FIELD_RESOLVED(fld) 
 				&& !CLASS_IS_PRIMITIVE(fld->type)) 
 			{
-                                GC_markObject(collector, fld->type);
+				if (!CLASS_GCJ(fld->type)) {
+					GC_markObject(collector, fld->type);
+				}
                         } /* else it's an Utf8Const that is not subject to gc */
                         fld++;
                 }
@@ -259,13 +327,25 @@ walkClass(Collector* collector, void* base, uint32 size)
                 /* walk static fields that contain references */
                 fld = CLASS_SFIELDS(class);
                 for (n = 0; n < CLASS_NSFIELDS(class); n++) {
+		    	/* Note that we must resolve field types eagerly
+			 * in processClass for gcj code cause it may never
+			 * call anything like getField to get the field
+			 * type resolved.  This can happen for such types as [C
+			 */
                         if (FIELD_RESOLVED(fld) && FIELD_ISREF(fld)) {
-				/* 
-				 * NB: This would break if static fields could
-				 * refer to non gc-allocated objects.
-				 * Check when doing GCJ.
-				 */
-                                GC_markObject(collector, *(void**)FIELD_ADDRESS(fld));
+				void **faddr = (void**)FIELD_ADDRESS(fld);
+#if defined (HAVE_GCJ_SUPPORT)
+/* 1. GCJ work-around, see 
+ * http://sourceware.cygnus.com/ml/java-discuss/1999-q4/msg00379.html 
+ */
+				if (FIELD_TYPE(fld) == StringClass) {
+					GC_markAddress(collector, *faddr);
+				} else {
+					GC_markObject(collector, *faddr);
+				}
+#else
+				GC_markObject(collector, *faddr);
+#endif
                         }
                         fld++;
                 }
@@ -330,47 +410,7 @@ walkRefArray(Collector* collector, void* base, uint32 size)
 		 * NB: This would break if some objects (i.e. class objects)
 		 * are not gc-allocated.
 		 */
-                GC_markObject(collector, el);
-        }
-}
-
-/*      
- * Walk the thread's internal context.
- * This is invoked by the garbage collector thread, which is not
- * stopped.
- *      
- * We will iterate through all threads, including the garbage collector
- * and those threads that haven't been started yet.
- */     
-static  
-void    
-TwalkThread(Collector* collector, Hjava_lang_Thread* tid)
-{       
-        void *from;
-        unsigned len;  
-        jthread_t jtid = (jthread_t)unhand(tid)->PrivateInfo;
-        
-        /* Don't walk the gc thread's stack.  It was not stopped and
-         * we hence don't have valid sp information.  In addition, there's
-         * absolutely no reason why we should walk it at all.
-         */
-        if (jtid == 0 || tid == jthread_getcookie((void*)jthread_current())) {
-DBG(JTHREAD,
-                dprintf("%d NOT walking jtid %p\n", jthread_current(), jtid);
-    )   
-                return;
-        }
-        
-        /* Ask threading system what the interesting stack range is;
-         * If the thread is too young, the threading system will return
-         * 0 from extract_stack.  In that case, we don't have walk anything.
-         */
-        if (jthread_extract_stack(jtid, &from, &len)) {
-DBG(JTHREAD|DBG_GCWALK,
-                dprintf("walking stack of `%s' thread\n", nameThread(tid));
-    )
-                /* and walk it if needed */
-                GC_walkConservative(collector, from, len);
+		GC_markObject(collector, el);
         }
 }
 
@@ -440,7 +480,8 @@ DBG(GCPRECISE,
                                  * mark it.  Note that p may or may not point
                                  * to a "real" Java object.
                                  */
-                                GC_markObject(collector, *(void **)mem);
+				void *p = *(void **)mem;
+				GC_markObject(collector, p);
                         }
                         i++;
                         l <<= 1;
@@ -448,11 +489,17 @@ DBG(GCPRECISE,
                 }
                 nbits -= BITMAP_BPI;
         }
+}
 
-        /* Special magic to handle thread objects */
-        if (soft_instanceof(ThreadClass, obj)) {
-                TwalkThread(collector, (Hjava_lang_Thread*)base);
-        }
+/*
+ * Walk a loader object.
+ */
+static
+void
+walkLoader(Collector* collector, void* base, uint32 size)
+{
+        walkObject(collector, base, size);
+        walkClassEntries(collector, (Hjava_lang_ClassLoader*)base);
 }
 
 static
@@ -462,12 +509,22 @@ finalizeObject(Collector* collector, void* ob)
 { 
 	extern JNIEnv Kaffe_JNIEnv;
 	JNIEnv *env = &Kaffe_JNIEnv;
+	Hjava_lang_Class* objclass;
+        Hjava_lang_Object* obj = (Hjava_lang_Object*)ob;
+	Method* final;
 
-        Hjava_lang_Object* obj = (Hjava_lang_Object*)ob;   
-        Hjava_lang_Class* objclass = OBJECT_CLASS(obj);    
-        Method* final = objclass->finalizer;               
+	if (!obj->dtable) {
+		/* Suppose we catch ThreadDeath inside newObject() */
+		return;
+	}
+        objclass = OBJECT_CLASS(obj);    
+        final = objclass->finalizer;               
+
+	if (!final) {
+		assert(objclass->alloc_type == GC_ALLOC_JAVALOADER);
+		return;
+	}
   
-        assert(final != 0);
 	(*env)->CallVoidMethod(env, obj, final);
 	/* ignore any resulting exception */
 	(*env)->ExceptionClear(env);
@@ -510,6 +567,7 @@ describeObject(const void* mem)
 			CLASS_CNAME(clazz) : "name unknown");
 		break;
 
+	case GC_ALLOC_JAVALOADER:
 	case GC_ALLOC_NORMALOBJECT:
 	case GC_ALLOC_FINALIZEOBJECT:
 	case GC_ALLOC_REFARRAY:
@@ -558,6 +616,9 @@ initCollector(void)
 	    walkClass, GC_OBJECT_NORMAL, destroyClass, "j.l.Class");
 	GC_registerGcTypeByIndex(gc, GC_ALLOC_FINALIZEOBJECT,
 	    walkObject, finalizeObject, 0, "obj-final");
+	GC_registerGcTypeByIndex(gc, GC_ALLOC_JAVALOADER,
+	    walkLoader, finalizeObject, destroyClassLoader, 
+	    "j.l.ClassLoader");
 
 	GC_registerFixedTypeByIndex(gc, GC_ALLOC_BYTECODE, "java-bytecode");
 	GC_registerFixedTypeByIndex(gc, GC_ALLOC_EXCEPTIONTABLE, "exc-table");
@@ -573,5 +634,6 @@ initCollector(void)
 	GC_registerFixedTypeByIndex(gc, GC_ALLOC_LOCK, "locks");
 	GC_registerFixedTypeByIndex(gc, GC_ALLOC_THREADCTX, "thread-ctxts");
 	GC_registerFixedTypeByIndex(gc, GC_ALLOC_REF, "gc-refs");
+	GC_registerFixedTypeByIndex(gc, GC_ALLOC_JITTEMP, "jit-temp-data");
 	return (gc);
 }
