@@ -41,8 +41,8 @@
 #define	cvt_long_int(t, f)			(t)[0].v.tint = (f)[0].v.tlong
 #define	cvt_long_float(t, f)			(t)[0].v.tfloat = (f)[0].v.tlong
 #define	cvt_long_double(t, f)			(t)[0].v.tdouble = (f)[0].v.tlong
-#define	cvt_float_long(t, f)			(t)[0].v.tlong = floor((f)[0].v.tfloat)
-#define	cvt_double_long(t, f)			(t)[0].v.tlong = floor((f)[0].v.tdouble)
+#define	cvt_float_long(t, f)			(t)[0].v.tlong = soft_cvtfl((f)[0].v.tfloat)
+#define	cvt_double_long(t, f)			(t)[0].v.tlong = soft_cvtdl((f)[0].v.tdouble)
 
 #define	move_int_const(t, c)			(t)[0].v.tint = (c)
 #define	move_ref_const(t, c)			(t)[0].v.taddr = (void*)(c)
@@ -162,6 +162,10 @@
 #define	end_function()				/* Not needed for interpreter */
 #define	start_sub_block()			/* Not needed for interpreter */
 #define	end_sub_block()				/* Not needed for interpreter */
+#define	begin_func_sync()			/* Not needed for interpreter */
+#define	end_func_sync()				/* Not needed for interpreter */
+#define	begin_sync()				/* Not needed for interpreter */
+#define	end_sync()				/* Not needed for interpreter */
 
 #define	set_label(i, l)				label_##i##l:
 #define	reference_code_label(l)			npc = (l)
@@ -177,7 +181,7 @@
 						 ((uint8*)(f)->v.taddr)[3])
 #define	load_code_ref				load_key
 
-#define	breakpoint()				ABORT()
+#define	softcall_breakpoint()			ABORT()
 
 #define	move_double(t, f)			(t)[0].v.tdouble = (f)[0].v.tdouble
 #define	move_double_const(t, c)			(t)[0].v.tdouble = (c)
@@ -272,12 +276,12 @@
 
 #define	cvt_int_float(t, f)			(t)[0].v.tfloat = (f)[0].v.tint
 #define	cvt_int_double(t, f)			(t)[0].v.tdouble = (f)[0].v.tint
-#define	cvt_float_int(t, f)			(t)[0].v.tint = floor((f)[0].v.tfloat)
+#define	cvt_float_int(t, f)			(t)[0].v.tint = soft_cvtfi((f)[0].v.tfloat)
 #define	cvt_float_double(t, f)			(t)[0].v.tdouble = (f)[0].v.tfloat
-#define	cvt_double_int(t, f)			(t)[0].v.tint = floor((f)[0].v.tdouble)
+#define	cvt_double_int(t, f)			(t)[0].v.tint = soft_cvtdi((f)[0].v.tdouble)
 #define	cvt_double_float(t, f)			(t)[0].v.tfloat = (f)[0].v.tdouble
 
-#define	softcall_lookupmethod(r, n, t)		(r)[0].v.taddr = soft_lookupmethod((t)[0].v.taddr, (n)->class, (n)->idx)
+#define	softcall_lookupinterfacemethod(r, n, t)	(r)[0].v.taddr = soft_lookupinterfacemethod((t)[0].v.taddr, (n)->class, (n)->idx)
 #if GUARANA
 #define CREATOR ((meth->accflags & ACC_STATIC) ? &meth->class->head : (Hjava_lang_Object*)local(0)->v.taddr)
 #define	softcall_new(r, t)			(r)->v.taddr = soft_new(t, CREATOR)
@@ -293,18 +297,25 @@
 #define softcall_initialise_class(cl)		soft_initialise_class(cl)
 #define	softcall_instanceof(r, o, t)		(r)->v.tint = soft_instanceof(t, (o)->v.taddr)
 
-#define	softcall_monitorenter(o)		lockObjMutex((o)[0].v.taddr)
-#define	softcall_monitorexit(o)			unlockObjMutex((o)[0].v.taddr)
+/* Thread.stop() knows how to back out of java frames, so we do not
+ * want to disable stop here
+ */
+#define	softcall_monitorenter(o)		lockObject((o)[0].v.taddr)
+#define	softcall_monitorexit(o)			unlockObject((o)[0].v.taddr)
 
 #if GUARANA
-#define	softcall_multianewarray(r, z, s, t)	(r)->v.taddr = soft_multianewarray(t, CREATOR, z, s)
+#define	softcall_multianewarray(r, z, s, t)	(r)->v.taddr = soft_multianewarray(t, CREATOR, z, s); \
+						pop((z) - 1)
 #else
-#define	softcall_multianewarray(r, z, s, t)	(r)->v.taddr = soft_multianewarray(t, z, s)
+#define	softcall_multianewarray(r, z, s, t)	(r)->v.taddr = soft_multianewarray(t, z, s); \
+						pop((z) - 1)
 #endif
 
 #define	softcall_badarrayindex()		soft_badarrayindex()
 #define	softcall_nullpointer()			soft_nullpointer()
 #define	softcall_nosuchmethod(c,n,s)		soft_nosuchmethod(c,n,s)
+#define	softcall_nosuchfield(c,n)		soft_nosuchfield(c,n)
+#define	softcall_nosuchclass(c)			soft_nosuchclass(c)
 
 #define	softcall_checkarraystore(a, o)		soft_checkarraystore((a)[0].v.taddr, (o)[0].v.taddr)
 #define	softcall_addreference(f, t)		soft_addreference((f)[0].v.taddr, (t)[0].v.taddr)
@@ -326,7 +337,7 @@
 
 #define softcall_reify_field_load(obj) do { \
   jvalue *res; \
-  if (FIELD_ISREF(finfo.field)) \
+  if (!FIELD_ISPRIM(finfo.field)) \
     res = (jvalue *)wstack(0); \
   else \
     switch (CLASS_PRIM_SIG(FIELD_TYPE(finfo.field))) { \
@@ -347,7 +358,7 @@
 
 #define softcall_reify_field_store(obj) do { \
   jvalue *res; \
-  if (FIELD_ISREF(finfo.field)) { \
+  if (!FIELD_ISPRIM(finfo.field)) { \
     res = (jvalue *)rstack(0); \
     pop(1); \
   } else \
