@@ -53,15 +53,6 @@ void fixupFunctionCall(sequence*);
 void syncRegisters(sequence*);
 void nowritebackSlot(sequence*);
 
-extern uint32 pc;
-extern uint32 npc;
-extern int maxPush;
-extern int maxArgs;
-extern int maxTemp;
-extern int maxLocal;
-extern int maxStack;
-extern int isStatic;
-
 bool used_ieee_rounding;
 bool used_ieee_division;
 
@@ -256,7 +247,7 @@ mon_enter(methods* meth, SlotInfo* obj)
 	else {
 		pusharg_ref(obj, 0);
 	}
-	call_soft(lockObjMutex);
+	call_soft(lockObject);
 	popargs();
 	start_sub_block();
 }
@@ -271,7 +262,7 @@ mon_exit(methods* meth, SlotInfo* obj)
 	else {
 		pusharg_ref(obj, 0);
 	}
-	call_soft(unlockObjMutex);
+	call_soft(unlockObject);
 	popargs();
 	start_sub_block();
 }
@@ -1092,7 +1083,7 @@ void
 neg_float(SlotInfo* dst, SlotInfo* src)
 {
 #if defined(HAVE_neg_float)
-	lslot_lslot_lslot(dst, 0, src, HAVE_neg_float, Tcomplex);
+	slot_slot_slot(dst, 0, src, HAVE_neg_float, Tcomplex);
 #else
 	SlotInfo* zero;
 	slot_alloctmp(zero);
@@ -1467,6 +1458,7 @@ load_offset_ref(SlotInfo* dst, SlotInfo* src, jint offset)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, src, offset);
 		load_ref(dst, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1493,6 +1485,7 @@ load_long(SlotInfo* dst, SlotInfo* src)
 	/* Don't use LSLOT & HSLOT here */
 	load_int(dst, src);
 	load_int(dst+1, tmp);
+	slot_freetmp(tmp);
 #endif
 }
 
@@ -1500,7 +1493,7 @@ void
 load_addr_long(SlotInfo* dst, void* addr)
 {
 	load_addr_int(dst, addr);
-	load_addr_int(dst+1, addr+4);
+	load_addr_int(dst+1, (char*)addr+4);
 }
 
 #if defined(HAVE_load_float)
@@ -1641,6 +1634,7 @@ load_offset_byte(SlotInfo* dst, SlotInfo* src, jint offset)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, src, offset);
 		load_byte(dst, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1662,6 +1656,7 @@ load_offset_char(SlotInfo* dst, SlotInfo* src, jint offset)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, src, offset);
 		load_char(dst, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1683,6 +1678,7 @@ load_offset_short(SlotInfo* dst, SlotInfo* src, jint offset)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, src, offset);
 		load_short(dst, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1704,6 +1700,7 @@ load_offset_float(SlotInfo* dst, SlotInfo* src, jint offset)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, src, offset);
 		load_float(dst, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1725,6 +1722,7 @@ load_offset_double(SlotInfo* dst, SlotInfo* src, jint offset)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, src, offset);
 		load_double(dst, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1734,7 +1732,7 @@ load_offset_scaled_int(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset)
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jint);
-	add_ref(tmp, tmp, src);
+	add_ref(tmp, src, tmp);
 	load_offset_int(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1745,7 +1743,7 @@ load_offset_scaled_ref(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset)
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jref);
-	add_ref(tmp, tmp, src);
+	add_ref(tmp, src, tmp);
 	load_offset_ref(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1753,18 +1751,12 @@ load_offset_scaled_ref(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset)
 void
 load_offset_scaled_long(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset)
 {
-	SlotInfo* nidx;
-	slot_alloctmp(nidx);
-	lshl_int_const(nidx, idx, 1);
-	if (src != LSLOT(dst)) {
-		load_offset_scaled_int(LSLOT(dst), src, nidx, offset);
-		load_offset_scaled_int(HSLOT(dst), src, nidx, offset+4);
-	}
-	else {
-		load_offset_scaled_int(HSLOT(dst), src, nidx, offset+4);
-		load_offset_scaled_int(LSLOT(dst), src, nidx, offset);
-	}
-	slot_freetmp(nidx);
+	SlotInfo* tmp;
+	slot_alloctmp(tmp);
+	lshl_int_const(tmp, idx, SHIFT_jlong);
+	add_ref(tmp, src, tmp);
+	load_offset_long(dst, tmp, offset);
+	slot_freetmp(tmp);
 }
 
 void
@@ -1773,7 +1765,7 @@ load_offset_scaled_float(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jfloat);
-	add_ref(tmp, tmp, src);
+	add_ref(tmp, src, tmp);
 	load_offset_float(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1784,7 +1776,7 @@ load_offset_scaled_double(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offse
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jdouble);
-	add_ref(tmp, tmp, src);
+	add_ref(tmp, src, tmp);
 	load_offset_double(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1794,7 +1786,7 @@ load_offset_scaled_byte(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset)
 {
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
-	add_ref(tmp, idx, src);
+	add_ref(tmp, src, idx);
 	load_offset_byte(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1805,7 +1797,7 @@ load_offset_scaled_char(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset)
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jchar);
-	add_ref(tmp, tmp, src);
+	add_ref(tmp, src, tmp);
 	load_offset_char(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1816,7 +1808,7 @@ load_offset_scaled_short(SlotInfo* dst, SlotInfo* src, SlotInfo* idx, int offset
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jshort);
-	add_ref(tmp, tmp, src);
+	add_ref(tmp, src, tmp);
 	load_offset_short(dst, tmp, offset);
 	slot_freetmp(tmp);
 }
@@ -1863,6 +1855,7 @@ store_offset_int(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_int(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1902,6 +1895,7 @@ store_offset_ref(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_ref(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -1987,6 +1981,7 @@ store_offset_float(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_float(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2026,6 +2021,7 @@ store_offset_double(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_double(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2056,6 +2052,8 @@ store_byte(SlotInfo* dst, SlotInfo* src)
 	and_int_const(tmp2, tmp2, -(1 << (8 * sizeof(jbyte))));
 	or_int(tmp2, tmp2, tmp);
 	store_int(dst, tmp2);
+	slot_freetmp(tmp2);
+	slot_freetmp(tmp);
 #endif
 }
 
@@ -2077,6 +2075,7 @@ store_offset_byte(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_byte(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2107,6 +2106,8 @@ store_char(SlotInfo* dst, SlotInfo* src)
 	and_int_const(tmp2, tmp2, -(1 << (8 * sizeof(jchar))));
 	or_int(tmp2, tmp2, tmp);
 	store_int(dst, tmp2);
+	slot_freetmp(tmp2);
+	slot_freetmp(tmp);
 #endif
 }
 
@@ -2128,6 +2129,7 @@ store_offset_char(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_char(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2158,6 +2160,8 @@ store_short(SlotInfo* dst, SlotInfo* src)
 	and_int_const(tmp2, tmp2, -(1 << (8 * sizeof(jshort))));
 	or_int(tmp2, tmp2, tmp);
 	store_int(dst, tmp2);
+	slot_freetmp(tmp2);
+	slot_freetmp(tmp);
 #endif
 }
 
@@ -2179,6 +2183,7 @@ store_offset_short(SlotInfo* dst, jint offset, SlotInfo* src)
 		slot_alloctmp(tmp);
 		add_ref_const(tmp, dst, offset);
 		store_short(tmp, src);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2198,7 +2203,7 @@ store_offset_scaled_int(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* src)
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jint);
-	add_ref(tmp, tmp, dst);
+	add_ref(tmp, dst, tmp);
 	store_offset_int(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2209,7 +2214,7 @@ store_offset_scaled_ref(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* src)
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jref);
-	add_ref(tmp, tmp, dst);
+	add_ref(tmp, dst, tmp);
 	store_offset_ref(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2217,12 +2222,12 @@ store_offset_scaled_ref(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* src)
 void
 store_offset_scaled_long(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* src)
 {
-	SlotInfo* nidx;
-	slot_alloctmp(nidx);
-	lshl_int_const(nidx, idx, 1);
-	store_offset_scaled_int(dst, nidx, offset, LSLOT(src));
-	store_offset_scaled_int(dst, nidx, offset+4, HSLOT(src));
-	slot_freetmp(nidx);
+	SlotInfo* tmp;
+	slot_alloctmp(tmp);
+	lshl_int_const(tmp, idx, SHIFT_jlong);
+	add_ref(tmp, dst, tmp);
+	store_offset_long(tmp, offset, src);
+	slot_freetmp(tmp);
 }
 
 void
@@ -2231,7 +2236,7 @@ store_offset_scaled_float(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* sr
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jfloat);
-	add_ref(tmp, tmp, dst);
+	add_ref(tmp, dst, tmp);
 	store_offset_float(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2242,7 +2247,7 @@ store_offset_scaled_double(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* s
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jdouble);
-	add_ref(tmp, tmp, dst);
+	add_ref(tmp, dst, tmp);
 	store_offset_double(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2252,7 +2257,7 @@ store_offset_scaled_byte(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* src
 {
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
-	add_ref(tmp, idx, dst);
+	add_ref(tmp, dst, idx);
 	store_offset_byte(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2263,7 +2268,7 @@ store_offset_scaled_char(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* src
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jchar);
-	add_ref(tmp, tmp, dst);
+	add_ref(tmp, dst, tmp);
 	store_offset_char(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2274,7 +2279,7 @@ store_offset_scaled_short(SlotInfo* dst, SlotInfo* idx, int offset, SlotInfo* sr
 	SlotInfo* tmp;
 	slot_alloctmp(tmp);
 	lshl_int_const(tmp, idx, SHIFT_jshort);
-	add_ref(tmp, tmp, dst);
+	add_ref(tmp, dst, tmp);
 	store_offset_short(tmp, offset, src);
 	slot_freetmp(tmp);
 }
@@ -2295,6 +2300,8 @@ store_const_offset_int(SlotInfo* dst, jint offset, jint val)
 		add_ref_const(tmp, dst, offset);
 		move_int_cosnt(tmp2, val);
 		store_int(tmp, tmp2);
+		slot_freetmp(tmp2);
+		slot_freetmp(tmp);
 	}
 }
 #endif
@@ -2315,6 +2322,8 @@ store_const_offset_byte(SlotInfo* dst, jint offset, jint val)
 		add_ref_const(tmp, dst, offset);
 		move_int_cosnt(tmp2, val);
 		store_byte(tmp, tmp2);
+		slot_freetmp(tmp2);
+		slot_freetmp(tmp);
 	}
 }
 #endif
@@ -2339,6 +2348,7 @@ pusharg_int_const(int val, int idx)
 		slot_alloctmp(tmp);
 		move_int_const(tmp, val);
 		pusharg_int(tmp, idx);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2375,6 +2385,7 @@ pusharg_ref_const(void* val, int idx)
 		slot_alloctmp(tmp);
 		move_ref_const(tmp, val);
 		pusharg_ref(tmp, idx);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2483,6 +2494,7 @@ cbranch_int_const(SlotInfo* s1, jint val, label* dst, int type)
 		slot_alloctmp(tmp);
 		move_int_const(tmp, val);
 		cbranch_int(s1, tmp, dst, type);
+		slot_freetmp(tmp);
 	}
 #else
 	cmp_int_const(0, s1, val);
@@ -2516,6 +2528,7 @@ cbranch_ref_const(SlotInfo* s1, void *val, label* dst, int type)
 		slot_alloctmp(tmp);
 		move_ref_const(tmp, val);
 		cbranch_ref(s1, tmp, dst, type);
+		slot_freetmp(tmp);
 	}
 #else
 	cmp_ref_const(0, s1, val);
@@ -2547,7 +2560,7 @@ call_indirect_method(Method *meth)
 {
 	void* ptr;
 
-	ptr = &METHOD_NATIVECODE(meth);
+	ptr = PMETHOD_NATIVECODE(meth);
 
 #if defined(HAVE_call_indirect_const)
 	slot_const_const(0, (jword)ptr, ba, HAVE_call_indirect_const, Tnull);
@@ -2559,6 +2572,7 @@ call_indirect_method(Method *meth)
 		move_ref_const(tmp, ptr);
 		load_ref(tmp, tmp);
 		call(tmp);
+		slot_freetmp(tmp);
 	}
 #endif
 }
@@ -2568,7 +2582,7 @@ call_soft(void *routine)
 {
 #if defined(HAVE_call_soft)
 	label* l = newLabel();
-	l->type = Labsolute|Lexternal;
+	l->type = Labsolute|Lexternal|Lnoprofile;
 	l->at = 0;
 	l->to = (uintp)routine;	/* What place does it goto */
 	l->from = 0;
@@ -2577,7 +2591,7 @@ call_soft(void *routine)
 #elif defined(HAVE_call_ref)
 	label* l;
 	l = newLabel();
-	l->type = Lexternal;
+	l->type = Lexternal|Lnoprofile;
 	l->at = 0;
 	l->to = (uintp)routine;	/* What place does it goto */
 	l->from = 0;
@@ -2599,6 +2613,7 @@ call_soft(void *routine)
 	move_label_const(tmp, l);
 	load_ref(tmp, tmp);
 	call(tmp);
+	slot_freetmp(tmp);
 #endif
 }
 
@@ -2830,6 +2845,7 @@ cmp_int_const(SlotInfo* dst, SlotInfo* src, jint val)
 		slot_alloctmp(tmp);
 		move_int_const(tmp, val);
 		cmp_int(dst, src, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -2855,6 +2871,7 @@ cmp_ref_const(SlotInfo* dst, SlotInfo* src, void* val)
 		slot_alloctmp(tmp);
 		move_ref_const(tmp, val);
 		cmp_ref(dst, src, tmp);
+		slot_freetmp(tmp);
 	}
 }
 
@@ -3155,7 +3172,7 @@ cvt_int_short(SlotInfo* dst, SlotInfo* src)
 /*									   */
 
 void
-breakpoint()
+softcall_breakpoint()
 {
 	ABORT();
 }
@@ -3289,7 +3306,7 @@ build_call_frame(Utf8Const* sig, SlotInfo* obj, int sp_idx)
 /*									   */
 
 void
-softcall_lookupmethod(SlotInfo* dst, Method* meth, SlotInfo* obj)
+softcall_lookupinterfacemethod(SlotInfo* dst, Method* meth, SlotInfo* obj)
 {
 	/* 'obj' must be written back since it will be reused */
 	prepare_function_call();
@@ -3297,7 +3314,7 @@ softcall_lookupmethod(SlotInfo* dst, Method* meth, SlotInfo* obj)
 	pusharg_ref_const((void*)(int)meth->idx, 2);		
 	pusharg_class_const(meth->class, 1);
 	pusharg_ref(obj, 0);
-	call_soft(soft_lookupmethod);
+	call_soft(soft_lookupinterfacemethod);
 	popargs();
 	fixup_function_call();
 	return_ref(dst);
@@ -3313,11 +3330,10 @@ check_array_index(SlotInfo* obj, SlotInfo* idx)
 #if defined(HAVE_ccall_ugt)
 	ccall_int_ugt(tmp, idx, soft_badarrayindex);
 #else
+	end_sub_block();
 	cbranch_int_ult(idx, tmp, reference_label(1, 1));
-	if (canCatch(BADARRAYINDEX)) {
-		sync_registers();
-	}
 	call_soft(soft_badarrayindex);
+	start_sub_block();
 	set_label(1, 1);
 #endif
 	slot_freetmp(tmp);
@@ -3333,11 +3349,10 @@ check_array_constindex(SlotInfo* obj, jint idx)
 #if defined(HAVE_ccall_ugt)
 	ccall_int_const_ugt(tmp, idx, soft_badarrayindex);
 #else
+	end_sub_block();
 	cbranch_int_const_ugt(tmp, idx, reference_label(1, 1));
-	if (canCatch(BADARRAYINDEX)) {
-		sync_registers();
-	}
 	call_soft(soft_badarrayindex);
+	start_sub_block();
 	set_label(1, 1);
 #endif
 	slot_freetmp(tmp);
@@ -3372,6 +3387,18 @@ softcall_nullpointer(void)
 	call_soft(soft_nullpointer);
 }
 
+
+void
+softcall_nosuchclass(Utf8Const* name)
+{
+	sync_registers();
+	prepare_function_call();
+	pusharg_utf8_const(name, 0);
+	call_soft(soft_nosuchclass);
+	popargs();
+	fixup_function_call();
+}
+
 void
 softcall_nosuchmethod(Hjava_lang_Class* cls, Utf8Const* name, Utf8Const* sig)
 {
@@ -3385,7 +3412,19 @@ softcall_nosuchmethod(Hjava_lang_Class* cls, Utf8Const* name, Utf8Const* sig)
 	fixup_function_call();
 }
 
-void 
+void
+softcall_nosuchfield(Utf8Const* cls, Utf8Const* name)
+{
+	sync_registers();
+	prepare_function_call();
+	pusharg_utf8_const(name, 1);
+	pusharg_utf8_const(cls, 0);
+	call_soft(soft_nosuchfield);
+	popargs();
+	fixup_function_call();
+}
+
+void
 #if GUARANA
 softcall_new_(SlotInfo* dst, Hjava_lang_Class* classobj, Method *meth)
 #else
@@ -3493,7 +3532,9 @@ softcall_multianewarray(SlotInfo* dst, int size, SlotInfo* stktop, Hjava_lang_Cl
 	pusharg_class_const(classobj, 0);
 	call_soft(soft_multianewarray);
 	popargs();
+	pop(size);
 	fixup_function_call();
+	push(1);
 	return_ref(dst);
 }
 
@@ -3542,7 +3583,7 @@ softcall_monitorenter(SlotInfo* mon)
 	slot_nowriteback(mon);
 	prepare_function_call();
 	pusharg_ref(mon, 0);
-	call_soft(lockObjMutex);
+	call_soft(lockObject);
 	popargs();
 	fixup_function_call();
 }
@@ -3553,7 +3594,7 @@ softcall_monitorexit(SlotInfo* mon)
 	slot_nowriteback(mon);
 	prepare_function_call();
 	pusharg_ref(mon, 0);
-	call_soft(unlockObjMutex);
+	call_soft(unlockObject);
 	popargs();
 	fixup_function_call();
 }
