@@ -56,7 +56,7 @@
 char* engine_name = "Just-in-time";
 char* engine_version = KVER;
 
-iLock translatorlock;		/* lock to protect the variables below */
+iLock* translatorlock;		/* lock to protect the variables below */
 int stackno;
 int maxStack;
 int maxLocal;
@@ -70,19 +70,17 @@ int argcount = 0;		/* Function call argument count */
 uint32 pc;
 uint32 npc;
 
-/* Various exception related things */
-extern Hjava_lang_Class javaLangArrayIndexOutOfBoundsException;
-extern Hjava_lang_Class javaLangNullPointerException;
-
 jitflags willcatch;
 
+#define EXPLICIT_CHECK_NULL(_i, _s, _n)                       \
+      cbranch_ref_const_ne((_s), 0, reference_label(_i, _n)); \
+      softcall_nullpointer();                                 \
+      set_label(_i, _n)
 /* Define CREATE_NULLPOINTER_CHECKS in md.h when your machine cannot use the
  * MMU for detecting null pointer accesses */
 #if defined(CREATE_NULLPOINTER_CHECKS)
 #define CHECK_NULL(_i, _s, _n)                                  \
-      cbranch_ref_const_ne((_s), 0, reference_label(_i, _n)); \
-      softcall_nullpointer();                                 \
-      set_label(_i, _n)
+    EXPLICIT_CHECK_NULL(_i, _s, _n)
 #else
 /*
  * If we rely on the MMU to catch null pointer accesses, we must spill
@@ -96,6 +94,13 @@ jitflags willcatch;
 	}
 #endif
 
+/* For JIT3 compatibility */
+#define check_array_store(a,b)		softcall_checkarraystore(a,b)
+#define explicit_check_null(x,obj,y)	EXPLICIT_CHECK_NULL(x,obj,y)
+#define check_null(x,obj,y)		CHECK_NULL(x,obj,y)
+#define check_div(x,obj,y)
+#define check_div_long(x,obj,y)
+
 /* Unit in which code block is increased when overrun */
 #define	ALLOCCODEBLOCKSZ	8192
 /* Codeblock redzone - allows for safe overrun when generating instructions */
@@ -109,12 +114,17 @@ static int codeperbytecode;
 
 int CODEPC;
 
+#if defined(KAFFE_PROFILER)
+int profFlag; /* flag to control profiling */
+Method *globalMethod;
+
+static void printProfilerStats(void);
+#endif
+
 struct {
 	int time;
 } jitStats;
 
-extern int enable_readonce;
-
 static void generateInsnSequence(codeinfo* codeInfo);
 static void checkCaughtExceptions(Method* meth, int pc);
 
@@ -156,7 +166,7 @@ checkStackOverflow(void)
 
 	if (overflow != 0) {
 		if (*needOnStack == STACK_LOW) {
-			fprintf(stderr, 
+			dprintf(
 				"Panic: unhandled StackOverflowError()\n");
 			ABORT();
 		}
@@ -176,12 +186,11 @@ translate(Method* meth, errorInfo *einfo)
 {
 	jint low;
 	jint high;
-	jlong tmpl;
+	jvalue tmpl;
 	int idx;
 	SlotInfo* tmp;
 	SlotInfo* tmp2;
 	SlotInfo* mtable;
-	const char* str;
 
 	bytecode* base;
 	int len;
@@ -189,7 +198,6 @@ translate(Method* meth, errorInfo *einfo)
 	fieldInfo finfo;
 	Hjava_lang_Class* crinfo;
 	bool success = true;
-	iLock* lock;
 
 	nativeCodeInfo ncode;
 	codeinfo* codeInfo;
@@ -197,15 +205,20 @@ translate(Method* meth, errorInfo *einfo)
 	int64 tms = 0;
 	int64 tme;
 
+	int iLockRoot;
+
 	static Method* jitting = 0;	/* DEBUG */
 
-	/* lock class to protect the method */
-	lock = lockObjMutex((Hjava_lang_Object*)meth->class);
+	/* lock class to protect the method.  Even though meth->class
+	   is a Java Object, this lock is not in the Java abstraction
+	   layer; it is just a detail of implementation, so we must
+	   use lockMutex instead of lockObject.  */
+	lockMutex(&meth->class->head);
 	/* Must check the translation
 	 * hasn't been done by someone else once we get it.
 	 */
 	if (METHOD_TRANSLATED(meth)) {
-		goto done2;
+		goto done3;
 	}
 
 	if (Kaffe_JavaVMArgs[0].enableVerboseJIT) {
@@ -214,41 +227,59 @@ translate(Method* meth, errorInfo *einfo)
 
 DBG(MOREJIT,
 	dprintf("asked to translate = %s.%s(%s)\n", 
-	    meth->class->name->data, meth->name->data, meth->signature->data);	
+	    meth->class->name->data, meth->name->data, METHOD_SIGD(meth));	
     )
 
 	/* If this code block is native, then just set it up and return */
 	if ((meth->accflags & ACC_NATIVE) != 0) {
 		success = native(meth, einfo);
 		if (success == false) {
-			goto done2;
+			goto done3;
 		}
 		KAFFEJIT_TO_NATIVE(meth);
 		/* Note that this is a real function not a trampoline.  */
-		meth->c.ncode.ncode_end = METHOD_NATIVECODE(meth);
-		goto done2;
+		if (meth->c.ncode.ncode_end == 0)
+			meth->c.ncode.ncode_end = METHOD_NATIVECODE(meth);
+		goto done3;
 	}
 
 	/* Scan the code and determine the basic blocks */
 	success = verifyMethod(meth, &codeInfo, einfo);
 	if (success == false) {
-		goto done2;
+		goto done3;
 	}
 
 DBG(MOREJIT,
 	dprintf("successfully verified = %s.%s(%s)\n", 
-	    meth->class->name->data, meth->name->data, meth->signature->data);	
+	    meth->class->name->data, meth->name->data, METHOD_SIGD(meth));	
     )
 
 	/* Only one in the translator at once. */
 	enterTranslator();
 	startTiming(&jit_time, "jittime");
 
+#if defined(KAFFE_PROFILER)
+	if (profFlag) {
+		static int init = 0;
+
+		if (!init) {
+			atexit(printProfilerStats);
+			init = 1;
+		}
+
+		profiler_get_clicks(meth->jitClicks);
+		meth->callsCount = 0;
+		meth->totalClicks = 0;
+		meth->totalChildrenClicks = 0;
+		globalMethod = meth;
+	}
+#endif
+
 DBG(MOREJIT,
 	if (jitting) {
 		dprintf("but still jitting = %s.%s(%s)\n", 
 			jitting->class->name->data, jitting->name->data, 
-			jitting->signature->data);	
+			METHOD_SIGD(jitting));	
 	}
     )
 	/* start modifying global variables now */
@@ -257,8 +288,10 @@ DBG(MOREJIT,
 
 	maxLocal = meth->localsz;
 	maxStack = meth->stacksz;
-	str = meth->signature->data;
-        maxArgs = sizeofSig(&str, false);
+        maxArgs = sizeofSigMethod(meth, false);
+	if (maxArgs == -1) {
+		goto done2;
+	}
 
 	if (meth->accflags & ACC_STATIC) {
 		isStatic = 1;
@@ -275,13 +308,18 @@ DBG(MOREJIT,
 	/* Next reduce bytecode to native code */
 	/***************************************/
 
-	initInsnSequence(codeperbytecode * len, meth->localsz, meth->stacksz);
+	if (!(success = initInsnSequence(meth, codeperbytecode * len, meth->localsz,
+					 meth->stacksz, einfo))) {
+		goto done1;
+	}
 
 	start_basic_block();
 	start_function();
 
 #if defined(CHECK_STACKOVERFLOW)
+	prepare_function_call();
 	call_soft(checkStackOverflow);
+	fixup_function_call();
 #endif
 
 	monitor_enter();
@@ -325,7 +363,7 @@ DBG(JIT,	dprintf("pc = %d, npc = %d\n", pc, npc);	)
 
 		switch (base[pc]) {
 		default:
-			fprintf(stderr, "Unknown bytecode %d\n", base[pc]);
+			dprintf("Unknown bytecode %d\n", base[pc]);
 			postException(einfo, JAVA_LANG(VerifyError));
 			success = false;
 			goto done;
@@ -353,15 +391,18 @@ DBG(JIT,	dprintf("pc = %d, npc = %d\n", pc, npc);	)
 
 	assert(maxTemp < MAXTEMPS);
 
-	finishInsnSequence(codeInfo, &ncode);
+	if (finishInsnSequence(codeInfo, &ncode, einfo) == false) {
+		success = false;
+		goto done;
+	}
 	installMethodCode(codeInfo, meth, &ncode);
 
 done:
-	tidyVerifyMethod(codeInfo);
+	tidyVerifyMethod(&codeInfo);
 
 DBG(JIT,
 	dprintf("Translated %s.%s%s (%s) %p\n", meth->class->name->data, 
-		meth->name->data, meth->signature->data, 
+		meth->name->data, METHOD_SIGD(meth), 
 		isStatic ? "static" : "normal", meth->ncode);	
     )
 
@@ -370,29 +411,44 @@ DBG(JIT,
 		jitStats.time += (tme - tms);
 		printf("<JIT: %s.%s%s time %dms (%dms) @ %p>\n",
 		       CLASS_CNAME(meth->class),
-		       meth->name->data, meth->signature->data,
+		       meth->name->data, METHOD_SIGD(meth),
 		       (int)(tme - tms), jitStats.time,
 		       METHOD_NATIVECODE(meth));
 	}
+done1:
+#if defined(KAFFE_PROFILER)
+	if (profFlag) {
+		profiler_click_t end;
 
+		profiler_get_clicks(end);
+		meth->jitClicks = end - meth->jitClicks;
+		globalMethod = 0;
+	}
+#endif
+done2:
 	jitting = 0;	/* DEBUG */
 	stopTiming(&jit_time);
 	leaveTranslator();
-done2:
-	unlockObjKnownMutex(lock);
+done3:
+	unlockMutex(&meth->class->head);
 	return (success);
 }
 
 /*
  * Generate the code.
  */
-void
-finishInsnSequence(codeinfo* codeInfo, nativeCodeInfo* code)
+bool
+finishInsnSequence(codeinfo* codeInfo, nativeCodeInfo* code, errorInfo *einfo)
 {
 #if defined(CALLTARGET_ALIGNMENT)
 	int align = CALLTARGET_ALIGNMENT;
 #else
 	int align = 0;
+#endif
+#if defined(MD_JIT_EXCEPTION_INFO_LENGTH)
+	int exc_len = MD_JIT_EXCEPTION_INFO_LENGTH;
+#else
+	int exc_len = 0;
 #endif
 	uint32 constlen;
 	nativecode* methblock;
@@ -407,14 +463,18 @@ finishInsnSequence(codeinfo* codeInfo, nativeCodeInfo* code)
 	 * NB: we assume the allocator returns at least 8-byte aligned 
 	 * addresses.   XXX: this should really be gc_memalign
 	 */  
-	methblock = gc_malloc(constlen + CODEPC + (align ? (align - 8) : 0), GC_ALLOC_JITCODE);
-	codebase = methblock + constlen;
+	methblock = gc_malloc(exc_len + constlen + CODEPC + (align ? (align - 8) : 0), GC_ALLOC_JITCODE);
+	if (methblock == 0) {
+		postOutOfMemory(einfo);
+		return (false);
+	}
+	codebase = methblock + exc_len + constlen;
 	/* align entry point if so desired */
-	if (align != 0 && (unsigned int)codebase % align != 0) {
-		int pad = (align - (unsigned int)codebase % align);
+	if (align != 0 && (unsigned long)codebase % align != 0) {
+		int pad = (align - (unsigned long)codebase % align);
 		/* assert the allocator indeed returned 8 bytes aligned addrs */
 		assert(pad <= align - 8);	
-		codebase = (void*)codebase + pad;
+		codebase = (char*)codebase + pad;
 	}
 	memcpy(codebase, codeblock, CODEPC);
 	addToCounter(&jitcodeblock, "jitmem-codeblock", 1,
@@ -422,16 +482,17 @@ finishInsnSequence(codeinfo* codeInfo, nativeCodeInfo* code)
 	gc_free(codeblock);
 
 	/* Establish any code constants */
-	establishConstants(methblock);
+	establishConstants(methblock + exc_len);
 
 	/* Link it */
 	linkLabels(codeInfo, (uintp)codebase);
 
 	/* Note info on the compiled code for later installation */
 	code->mem = methblock;
-	code->memlen = constlen + CODEPC;
+	code->memlen = exc_len + constlen + CODEPC;
 	code->code = codebase;
 	code->codelen = CODEPC;
+	return (true);
 }
 
 /*
@@ -452,18 +513,26 @@ installMethodCode(codeinfo* codeInfo, Method* meth, nativeCodeInfo* code)
 	 * before overwriting it.
 	 */
 	if (CLASS_IS_INTERFACE(meth->class) && utf8ConstEqual(meth->name, init_name)) {
-		KFREE(METHOD_NATIVECODE(meth));
+		/* Workaround for KFREE() ? : bug on gcc 2.7.2 */
+		void *nc = METHOD_NATIVECODE(meth);
+		KFREE(nc);
 	}
-	SET_METHOD_NATIVECODE(meth, code->code);
+	SET_METHOD_JITCODE(meth, code->code);
 	/* Free bytecode before replacing it with native code */
 	if (meth->c.bcode.code != 0) {
 		KFREE(meth->c.bcode.code);
 	}
 	meth->c.ncode.ncode_start = code->mem;
-	meth->c.ncode.ncode_end = code->code + code->codelen;
+	meth->c.ncode.ncode_end = (char*)code->code + code->codelen;
 
 	/* Flush code out of cache */
-	FLUSH_DCACHE(meth->ncode, meth->c.ncode.ncode_end);
+	FLUSH_DCACHE(METHOD_NATIVECODE(meth), meth->c.ncode.ncode_end);
+
+#if defined(MD_REGISTER_JIT_EXCEPTION_INFO)
+	MD_REGISTER_JIT_EXCEPTION_INFO (meth->c.ncode.ncode_start,
+					METHOD_NATIVECODE(meth),
+					meth->c.ncode.ncode_end);
+#endif
 
 	/* Translate exception table and make it available */
 	if (meth->exception_table != 0) {
@@ -508,8 +577,9 @@ installMethodCode(codeinfo* codeInfo, Method* meth, nativeCodeInfo* code)
 /*
  * Init instruction generation.
  */
-void
-initInsnSequence(int codesize, int localsz, int stacksz)
+bool
+initInsnSequence(Method* meth, int codesize, int localsz, int stacksz,
+		 struct _errorInfo *einfo)
 {
 	/* Clear various counters */
 	tmpslot = 0;
@@ -533,9 +603,14 @@ initInsnSequence(int codesize, int localsz, int stacksz)
 		codeblock_size = ALLOCCODEBLOCKSZ;
 	}
 	codeblock = KMALLOC(codeblock_size + CODEBLOCKREDZONE);
+	if (!codeblock) {
+		postOutOfMemory(einfo);
+		return false;
+	}
 	addToCounter(&jitcodeblock, "jitmem-codeblock", 1,
-		     (jlong)GCSIZEOF(codeblock));
+		     (jlong) GCSIZEOF(codeblock));
 	CODEPC = 0;
+	return true;
 }
 
 /*
@@ -812,10 +887,10 @@ checkCaughtExceptions(Method* meth, int pc)
 			willCatch(NULLPOINTER);
 		}
 		else {
-			if (instanceof(&javaLangArrayIndexOutOfBoundsException, etype)) {
+			if (instanceof(javaLangArrayIndexOutOfBoundsException, etype)) {
 				willCatch(BADARRAYINDEX);
 			}
-			if (instanceof(&javaLangNullPointerException, etype)) {
+			if (instanceof(javaLangNullPointerException, etype)) {
 				willCatch(NULLPOINTER);
 			}
 		}
@@ -830,3 +905,83 @@ getEngine()
 {
 	return "kaffe.jit";
 }
+
+
+#if defined(KAFFE_PROFILER)
+static jlong click_multiplier;
+static profiler_click_t click_divisor;
+static FILE *prof_output;
+
+static int
+profilerClassStat(Hjava_lang_Class *clazz, void *param)
+{
+	Method *meth;
+	int mindex;
+
+	for (mindex = clazz->nmethods, meth = clazz->methods; mindex-- > 0; meth++) {
+		if (meth->callsCount == 0)
+			continue;
+
+		fprintf(prof_output,
+			"%10d %10.6g %10.6g %10.6g %10.6g %s.%s%s\n",
+			meth->callsCount,
+			(click_multiplier * ((double)meth->totalClicks)) / click_divisor,
+			(click_multiplier * ((double)(meth->totalClicks
+						      - meth->totalChildrenClicks)))
+			/ click_divisor,
+			(click_multiplier * ((double)meth->totalChildrenClicks)) / click_divisor,
+			(click_multiplier * ((double)meth->jitClicks)) / click_divisor,
+
+			meth->class->name->data,
+			meth->name->data,
+			METHOD_SIGD(meth)
+		       );
+	}
+	return 0;
+}
+
+
+static void
+printProfilerStats(void)
+{
+	profiler_click_t start, end;
+	jlong m_start, m_end;
+
+	/* If the profFlag == 0, don't bother printing anything */
+	if (profFlag == 0)
+		return;
+
+ 	/* open file */
+ 	prof_output = fopen("prof.out", "w");
+	if (prof_output == NULL) {
+		return;
+	}
+
+	/* Need to figure out what profiler_click  _really_ is.
+	 * Use currentTime() to guest how long is a second.  Call it before
+	 * to don't count dynamic linker resolve time.  */
+	m_start = currentTime();
+	profiler_get_clicks(start);
+	sleep (1);
+	m_end = currentTime();
+	profiler_get_clicks(end);
+
+	click_multiplier = m_end - m_start;
+	click_divisor = end - start;
+
+	fprintf(prof_output,
+		"# clockTick: %lld per 1/%lld seconds aka %lld per milliseconds\n",
+		click_divisor,
+		click_multiplier,
+		click_divisor / click_multiplier);
+
+ 	fprintf(prof_output, "%10s %10s %10s %10s %10s %s\n",
+ 		"#    count", "total", "self", "children", "jit",
+ 		"method-name");
+
+	/* Traverse through class hash table */
+	walkClassPool(profilerClassStat, NULL);
+
+	fclose(prof_output);
+}
+#endif
