@@ -24,13 +24,16 @@
 #include "errors.h"
 #include "md.h"
 #include "stats.h"
+#include "classMethod.h"
 #if GUARANA
 #include "baseClasses.h" /* for ObjectClass */
-#include "classMethod.h" /* for final_name */
 #include "lookup.h" /* for findMethodLocal */
 #include "GuaraNative.h"
 #endif
 
+/* Avoid recursively allocating OutOfMemoryError */
+#define OOM_ALLOCATING		((void *) -1)
+
 #define GCSTACKSIZE		16384
 #define FINALIZERSTACKSIZE	THREADSTACKSIZE
 
@@ -42,6 +45,10 @@ static struct CollectorImpl {
 	/* XXX include static below here for encapsulation */
 } gc_obj;
 
+/* XXX don't use these types ! */
+Hjava_lang_Thread* garbageman;
+static Hjava_lang_Thread* finalman;
+
 static gcList gclists[5];
 static const int mustfree = 4;		/* temporary list */
 static const int white = 3;
@@ -60,6 +67,15 @@ static counter gcgcablemem;
 static counter gcfixedmem;
 #endif /* KAFFE_STATS */
 
+/* Is this pointer within our managed heap? */
+#define IS_A_HEAP_POINTER(from) \
+        ((uintp) (from) >= gc_heap_base && \
+	 (uintp) (from) < gc_heap_base + gc_heap_range)
+
+static void *reserve;
+static void *outOfMem;
+static void *outOfMem_allocator;
+
 #if defined(SUPPORT_VERBOSEMEM)
 
 static void objectStatsChange(gc_unit*, int);
@@ -87,9 +103,9 @@ void record_marked(int nr_of_objects, uint32 size)
         gcStats.markedmem += size;
 } 
 
-static iLock gcman;
-static iLock finman;
-iLock gc_lock;			/* allocator mutex */
+iLock* gcman;
+iLock* finman;
+iLock* gc_lock;			/* allocator mutex */
 
 static void gcFree(Collector* gcif, void* mem);
 
@@ -151,6 +167,7 @@ struct _gcStats gcStats;
 
 static void startGC(Collector *gcif);
 static void finishGC(Collector *gcif);
+static void startFinalizer(void);
 static void markObjectDontCheck(gc_unit *unit, gc_block *info, int idx);
 
 /* Return true if gc_unit is pointer to an allocated object */
@@ -175,6 +192,43 @@ gc_heap_isobject(gc_block *info, gc_unit *unit)
 	return 0;
 }
 
+static inline void
+markObjectDontCheck(gc_unit *unit, gc_block *info, int idx)
+{
+	/* If object's been traced before, don't do it again */
+	if (GC_GET_COLOUR(info, idx) != GC_COLOUR_WHITE) {
+		return;
+	}
+DBG(GCWALK,	
+	dprintf("  marking @%08p: %s\n", UTOMEM(unit),
+			describeObject(UTOMEM(unit)));
+    )
+
+	DBG(GCSTAT,
+	    switch (GC_GET_FUNCS(info, idx)) {
+	    case GC_ALLOC_NORMALOBJECT:
+	    case GC_ALLOC_FINALIZEOBJECT:
+	    case GC_ALLOC_PRIMARRAY:
+	    case GC_ALLOC_REFARRAY: {
+		    Hjava_lang_Object *obj;
+		    obj = (Hjava_lang_Object *)(unit+1);
+		    if (obj->dtable != NULL) {
+			    Hjava_lang_Class *c;
+			    c = OBJECT_CLASS(obj);
+			    if (c)
+				    c->live_count++;
+		    }
+	    }});
+	    
+
+	/* If we found a new white object, mark it as grey and
+	 * move it into the grey list.
+	 */
+	GC_SET_COLOUR(info, idx, GC_COLOUR_GREY);
+	UREMOVELIST(unit);
+	UAPPENDLIST(gclists[grey], unit);
+}
+
 /*
  * Mark the memory given by an address if it really is an object.
  */
@@ -197,40 +251,17 @@ gcMarkAddress(Collector* gcif, const void* mem)
 	}
 }
 
-static void
-markObjectDontCheck(gc_unit *unit, gc_block *info, int idx)
-{
-	/* If object's been traced before, don't do it again */
-	if (GC_GET_COLOUR(info, idx) != GC_COLOUR_WHITE) {
-		return;
-	}
-DBG(GCWALK,	
-	dprintf("  marking @%08p: %s\n", UTOMEM(unit),
-			describeObject(UTOMEM(unit)));
-    )
-
-	/* If we found a new white object, mark it as grey and
-	 * move it into the grey list.
-	 */
-	GC_SET_COLOUR(info, idx, GC_COLOUR_GREY);
-	UREMOVELIST(unit);
-	UAPPENDLIST(gclists[grey], unit);
-}
-
 /*
- * Mark an object.  Argument is assumed to point to a valid object.
+ * Mark an object.  Argument is assumed to point to a valid object,
+ * and never, ever, be null.
  */
 static void
 gcMarkObject(Collector* gcif, const void* objp)
 {
-	if (objp != 0) {
-		gc_unit *unit = UTOUNIT(objp);
-		gc_block *info = GCMEM2BLOCK(unit);
-#if defined(DEBUG)
-		assert(gc_heap_isobject(info, unit));
-#endif
-		markObjectDontCheck(unit, info, GCMEM2IDX(info, unit));
-	}
+  gc_unit *unit = UTOUNIT(objp);
+  gc_block *info = GCMEM2BLOCK(unit);
+  DBG(GCDIAG, assert(gc_heap_isobject(info, unit)));
+  markObjectDontCheck(unit, info, GCMEM2IDX(info, unit));
 }
 
 static void
@@ -279,6 +310,52 @@ gcGetObjectIndex(Collector* gcif, const void* mem)
 	}
 }
 
+/*
+ * Given a pointer within an object, find the base of the object.
+ * This works for both gcable and fixed object types.
+ *
+ * This method uses many details of the allocator implementation.
+ * Specifically, it relies on the contiguous layout of block infos
+ * and the way GCMEM2BLOCK and GCMEM2IDX are implemented.
+ */
+static
+void*
+gcGetObjectBase(Collector *gcif, const void* mem)
+{
+	int idx;
+	gc_block* info;
+
+	/* quickly reject pointers that are not part of this heap */
+	if (!IS_A_HEAP_POINTER(mem)) {
+		return (0);
+	}
+
+	info = GCMEM2BLOCK(mem);
+	if (!info->inuse) {
+		/* go down block list to find out whether we were hitting
+		 * in a large object
+		 */
+		while (!info->inuse && (uintp)info > (uintp)gc_block_base) {
+			info--;
+		}
+		/* must be a large block, hence nr must be 1 */
+		if (!info->inuse || info->nr != 1) {
+			return (0);
+		}
+	}
+
+	idx = GCMEM2IDX(info, mem);
+
+	/* report fixed objects as well */
+	if (idx < info->nr && 
+	    ((GC_GET_COLOUR(info, idx) & GC_COLOUR_INUSE) || 
+	     (GC_GET_COLOUR(info, idx) & GC_COLOUR_FIXED))) 
+	{
+	    	return (UTOMEM(GCBLOCK2MEM(info, idx)));
+	}
+	return (0);
+}
+
 static
 const char*
 gcGetObjectDescription(Collector* gcif, const void* mem)
@@ -321,12 +398,29 @@ DBG(GCWALK,
 	}
 }
 
+#ifdef DEBUG
+static int
+gcClearCounts(Hjava_lang_Class *c, void *_)
+{
+	c->live_count = 0;
+	return 0;
+}
+
+static int
+gcDumpCounts(Hjava_lang_Class *c, void *_)
+{
+	if (c->live_count)
+		dprintf("%7d %s\n", c->live_count,	c->name->data);
+	return 0;
+}
+#endif
+
 /*
  * The Garbage Collector sits in a loop starting a collection, waiting
  * until it's finished incrementally, then tidying up before starting
  * another one.
  */
-void
+static void
 gcMan(void* arg)
 {
 	gc_unit* unit;
@@ -334,19 +428,22 @@ gcMan(void* arg)
 	gc_block* info;
 	int idx;
 	Collector *gcif = (Collector*)arg;
-
-	if (!staticLockIsInitialized(&gcman)) {
-		initStaticLock(&gcman);
-	}
-	lockStaticMutex(&gcman);
+	int iLockRoot;
 
 	/* Wake up anyone waiting for the GC to finish every time we're done */
-	for(;; gcRunning = 0, broadcastStaticCond(&gcman)) {
+	for (;;) {
+		lockStaticMutex(&gcman);
 
 		while (gcRunning == 0) {
 			waitStaticCond(&gcman, 0);
 		}
+		/* We have observed that gcRunning went from 0 to 1 or 2 
+		 * One thread requested a gc.  We will decide whether to gc
+		 * or not, and then we will set gcRunning back to 0 and
+		 * inform the calling thread of the change
+		 */
 		assert(gcRunning > 0);
+
 		/* 
 		 * gcRunning will either be 1 or 2.  If it's 1, we can apply
 		 * some heuristics for when we skip a collection.
@@ -369,7 +466,7 @@ gcMan(void* arg)
 DBG(GCSTAT,
 			dprintf("skipping collection cause allocmem==0...\n");
     )
-			continue;
+			goto gcend;
                 }
 
 		/*
@@ -399,8 +496,12 @@ DBG(GCSTAT,
 				gcStats.totalmem/1024,
 				gcStats.allocmem/(double)gcStats.totalmem);
     )
-			continue;
+			goto gcend;
 		}
+
+		lockStaticMutex(&gc_lock);
+		DBG(GCSTAT, walkClassPool(gcClearCounts, 0));
+		    
 		startGC(gcif);
 
 		for (unit = gclists[grey].cnext; unit != &gclists[grey]; unit = gclists[grey].cnext) {
@@ -429,15 +530,27 @@ DBG(GCSTAT,
 
 		finishGC(gcif);
 
+		DBG(GCSTAT,
+		    dprintf("REACHABLE OBJECT HISTOGRAM\n");
+		    dprintf("%-7s %s\n", "COUNT", "CLASS");
+		    dprintf("%-7s %s\n", "-------",
+			    "-----------------------------------"
+			    "-----------------------------------");
+		    walkClassPool(gcDumpCounts, 0));
+		    
+		unlockStaticMutex(&gc_lock);
+
+		startFinalizer();
+
 		if (Kaffe_JavaVMArgs[0].enableVerboseGC > 0) {
 			/* print out all the info you ever wanted to know */
-			fprintf(stderr, 
+			dprintf(
 			    "<GC: heap %dK, total before %dK,"
 			    " after %dK (%d/%d objs)\n %2.1f%% free,"
 			    " alloced %dK (#%d), marked %dK, "
 			    "swept %dK (#%d)\n"
 			    " %d objs (%dK) awaiting finalization>\n",
-			gc_heap_total/1024, 
+			(int)(gc_heap_total/1024), 
 			gcStats.totalmem/1024, 
 			(gcStats.totalmem-gcStats.freedmem)/1024, 
 			gcStats.totalobj,
@@ -459,6 +572,12 @@ DBG(GCSTAT,
 		gcStats.totalobj -= gcStats.freedobj;
 		gcStats.allocobj = 0;
 		gcStats.allocmem = 0;
+
+gcend:;
+		/* now signal any waiters */
+		gcRunning = 0;
+		broadcastStaticCond(&gcman);
+		unlockStaticMutex(&gcman);
 	}
 }
 
@@ -477,7 +596,6 @@ startGC(Collector *gcif)
 	gcStats.markedobj = 0;
 	gcStats.markedmem = 0;
 
-	lockStaticMutex(&gc_lock);
 	/* disable the mutator to protect colour lists */
 	STOPWORLD();
 
@@ -599,16 +717,6 @@ finishGC(Collector *gcif)
 		gc_heap_free(unit);
 	}
 	stopTiming(&sweep_time);
-	unlockStaticMutex(&gc_lock);
-
-	/* If there's stuff to be finalised then we'd better do it */
-	if (gclists[finalise].cnext != &gclists[finalise]) {
-		lockStaticMutex(&finman);
-		finalRunning = true;
-		signalStaticCond(&finman);
-		unlockStaticMutex(&finman);
-	}
-
 }
 
 #if GUARANA
@@ -662,26 +770,49 @@ forcedFinalizer(void *ob)
 }
 #endif
 
+static
+void
+startFinalizer(void)
+{
+	int iLockRoot;
+	int start;
+
+        start = 0;
+
+	lockStaticMutex(&gc_lock);
+	/* If there's stuff to be finalised then we'd better do it */
+	if (gclists[finalise].cnext != &gclists[finalise]) {
+		start = 1;
+	}
+	unlockStaticMutex(&gc_lock);
+
+	lockStaticMutex(&finman);
+	if (start != 0 && finalRunning == false) {
+		finalRunning = true;
+		signalStaticCond(&finman);
+	}
+	unlockStaticMutex(&finman);
+
+}
+
 /*
  * The finaliser sits in a loop waiting to finalise objects.  When a
  * new finalised list is available, it is woken by the GC and finalises
  * the objects in turn.  An object is only finalised once after which
  * it is deleted.
  */
-void
+static void
 finaliserMan(void* arg)
 {
 	gc_block* info;
 	gc_unit* unit;
 	int idx;
 	Collector *gcif = (Collector*)arg;
+	int iLockRoot;
 
-	if (!staticLockIsInitialized(&finman)) {
-		initStaticLock(&finman);
-	}
-	lockStaticMutex(&finman);
 
 	for (;;) {
+		lockStaticMutex(&finman);
 
 		finalRunning = false;
 		while (finalRunning == false) {
@@ -719,6 +850,7 @@ finaliserMan(void* arg)
 
 		/* Wake up anyone waiting for the finalizer to finish */
 		broadcastStaticCond(&finman);
+		unlockStaticMutex(&finman);
 	}
 }
 
@@ -728,15 +860,19 @@ static
 void
 gcInvokeGC(Collector* gcif, int mustgc)
 {
-	if (!staticLockIsInitialized(&gcman)) {
-		initStaticLock(&gcman);
-	}
+	int iLockRoot;
+
 	lockStaticMutex(&gcman);
 	if (gcRunning == 0) {
 		gcRunning = mustgc ? 2 : 1;
 		signalStaticCond(&gcman);
 	}
-	waitStaticCond(&gcman, 0);
+	unlockStaticMutex(&gcman);
+
+	lockStaticMutex(&gcman);
+	while (gcRunning != 0) {
+		waitStaticCond(&gcman, 0);
+	}
 	unlockStaticMutex(&gcman);
 }
 
@@ -747,13 +883,11 @@ static
 void
 gcInvokeFinalizer(Collector* gcif)
 {
+	int iLockRoot;
+
 	/* First invoke the GC */
 	GC_invoke(gcif, 1);
 
-	if (!staticLockIsInitialized(&finman)) {
-		initStaticLock(&finman);
-	}
-
 	/* Run the finalizer (if might already be running as a result of
 	 * the GC)
 	 */
@@ -784,19 +918,21 @@ gcMalloc(Collector* gcif, size_t size, int fidx)
 	void * volatile mem;	/* needed on SGI, see comment below */
 	int i;
 	size_t bsz;
+	int iLockRoot;
 
 	assert(gc_init != 0);
 	assert(fidx < nrTypes && size != 0);
-	lockStaticMutex(&gc_lock);
+
 	unit = gc_heap_malloc(size + sizeof(gc_unit));
 
 	/* keep pointer to object */
 	mem = UTOMEM(unit);
 	if (unit == 0) {
-		unlockStaticMutex(&gc_lock);
-		throwOutOfMemory();		/* Big XXX */
+		return 0;
 	}
 
+	lockStaticMutex(&gc_lock);
+
 	info = GCMEM2BLOCK(mem);
 	i = GCMEM2IDX(info, unit);
 
@@ -840,10 +976,74 @@ gcMalloc(Collector* gcif, size_t size, int fidx)
 		GC_SET_COLOUR(info, i, GC_COLOUR_WHITE);
 		UAPPENDLIST(gclists[white], unit);
 	}
+	if (!reserve) {
+		reserve = gc_primitive_reserve();
+	}
+
+	/* It is not safe to allocate java objects the first time
+	 * gcMalloc is called, but it should be safe after gcEnable
+	 * has been called.
+	 */
+	if (garbageman && !outOfMem && !outOfMem_allocator) {
+		outOfMem_allocator = jthread_current();
+	}
+
 	unlockStaticMutex(&gc_lock);
+
+	/* jthread_current() will be null in some window before we
+	 * should try allocating java objects
+	 */
+	if (!outOfMem && outOfMem_allocator
+	    && outOfMem_allocator == jthread_current()) { 
+		outOfMem = OOM_ALLOCATING;
+		outOfMem = OutOfMemoryError; /* implicit allocation */
+		outOfMem_allocator = 0;
+		gc_add_ref(outOfMem);
+	}
 	return (mem);
 }
 
+static
+Hjava_lang_Throwable *
+gcThrowOOM(Collector *gcif)
+{
+	Hjava_lang_Throwable *ret = 0;
+	int reffed;
+	int iLockRoot;
+
+	/*
+	 * Make sure we are the only thread to use this exception
+	 * object.
+	 */
+	lockStaticMutex(&gc_lock);
+	ret = outOfMem;
+	reffed = outOfMem != 0;
+	outOfMem = 0;
+	/* We try allocating reserved pages before we allocate the
+	 * outOfMemory error.  We can use some or all of the reserved
+	 * pages to actually grab an error.
+	 */
+	if (reserve) {
+		gc_primitive_free(reserve);
+		reserve = 0;
+		if (!ret || ret == OOM_ALLOCATING) {
+			unlockStaticMutex(&gc_lock);
+			ret = OutOfMemoryError; /* implicit allocation */
+			lockStaticMutex(&gc_lock);
+		}
+	}
+	if (ret == OOM_ALLOCATING || ret == 0) {
+		/* die now */
+		unlockStaticMutex(&gc_lock);
+		dprintf(
+			"Not enough memory to throw OutOfMemoryError!\n");
+		ABORT();
+	}
+	unlockStaticMutex(&gc_lock);
+	if (reffed) gc_rm_ref(ret);
+	return ret;
+}
+
 /*
  * Reallocate an object.
  */
@@ -856,8 +1056,9 @@ gcRealloc(Collector* gcif, void* mem, size_t size, int fidx)
 	void* newmem;
 	gc_unit* unit;
 	int osize;
+	int iLockRoot;
 
-	assert(fidx == GC_ALLOC_FIXED);
+	assert(gcFunctions[fidx].final == GC_OBJECT_FIXED);
 
 	/* If nothing to realloc from, just allocate */
 	if (mem == NULL) {
@@ -868,7 +1069,7 @@ gcRealloc(Collector* gcif, void* mem, size_t size, int fidx)
 	unit = UTOUNIT(mem);
 	info = GCMEM2BLOCK(unit);
 	idx = GCMEM2IDX(info, unit);
-	osize = GCBLOCKSIZE(info);
+	osize = GCBLOCKSIZE(info) - sizeof(gc_unit);
 
 	/* Can only handled fixed objects at the moment */
 	assert(GC_GET_COLOUR(info, idx) == GC_COLOUR_FIXED);
@@ -876,7 +1077,7 @@ gcRealloc(Collector* gcif, void* mem, size_t size, int fidx)
 	unlockStaticMutex(&gc_lock);
 
 	/* If we'll fit into the current space, just send it back */
-	if (osize >= size + sizeof(gc_unit)) {
+	if (osize >= size) {
 		return (mem);
 	}
 
@@ -898,6 +1099,7 @@ gcFree(Collector* gcif, void* mem)
 	gc_block* info;
 	int idx;
 	gc_unit* unit;
+	int iLockRoot;
 
 	if (mem != 0) {
 		lockStaticMutex(&gc_lock);
@@ -928,14 +1130,9 @@ static
 void
 gcInit(Collector *collector)
 {
-	initStaticLock(&gc_lock);
 	gc_init = 1;
 }
 
-/* XXX don't use these types ! */
-Hjava_lang_Thread* garbageman;
-static Hjava_lang_Thread* finalman;
-
 /*
  * Start gc threads, which enable collection
  */
@@ -944,13 +1141,18 @@ void
 /* ARGSUSED */
 gcEnable(Collector* collector)
 {
+	errorInfo info;
+
         if (DBGEXPR(NOGC, false, true))
         {
                 /* Start the GC daemons we need */
                 finalman = createDaemon(&finaliserMan, "finaliser", 
-				collector, THREAD_MAXPRIO, FINALIZERSTACKSIZE);
+				collector, THREAD_MAXPRIO,
+					FINALIZERSTACKSIZE, &info);
                 garbageman = createDaemon(&gcMan, "gc", 
-				collector, THREAD_MAXPRIO, GCSTACKSIZE);
+				collector, THREAD_MAXPRIO,
+					  GCSTACKSIZE, &info);
+		assert(finalman && garbageman);
         }
 }
 
@@ -979,23 +1181,23 @@ objectStatsPrint(void)
 {
 	int cnt = 0;
 
-	fprintf(stderr, "Memory statistics:\n");
-	fprintf(stderr, "------------------\n");
+	dprintf("Memory statistics:\n");
+	dprintf("------------------\n");
 
 	while (cnt < nrTypes) {
-		fprintf(stderr, "%14.14s: Nr %6d  Mem %6dK",
+		dprintf("%14.14s: Nr %6d  Mem %6dK",
 			gcFunctions[cnt].description, 
 			gcFunctions[cnt].nr, 
 			gcFunctions[cnt].mem/1024);
 		if (++cnt % 2 != 0) {
-			fprintf(stderr, "   ");
+			dprintf("   ");
 		} else {
-			fprintf(stderr, "\n");
+			dprintf("\n");
 		}
 	}
 
 	if (cnt % 2 != 0) {
-		fprintf(stderr, "\n");
+		dprintf("\n");
 	}
 }
 #endif
@@ -1019,10 +1221,12 @@ static struct GarbageCollectorInterface_Ops GC_Ops = {
 	gcGetObjectSize,
 	gcGetObjectDescription,
 	gcGetObjectIndex,
+	gcGetObjectBase,
 	gcWalkMemory,
 	gcWalkConservative,
 	gcRegisterFixedTypeByIndex,
-	gcRegisterGcTypeByIndex
+	gcRegisterGcTypeByIndex,
+	gcThrowOOM
 #if GUARANA
 	,
 	gcForceFinalization
