@@ -20,7 +20,7 @@
 #include "support.h"
 #include "stats.h"
 #include "locks.h"
-#include "jthread.h"
+#include "thread.h"
 #include "gc.h"
 #include "gc-mem.h"
 #include "jni.h"
@@ -28,18 +28,20 @@
 #include <unistd.h>
 #endif
 
-extern iLock gc_lock;
+extern iLock* gc_lock;
 
+#if defined(KAFFE_STATS)
 static counter gcpages;
+#endif
 static gc_block* gc_small_block(size_t);
 static gc_block* gc_large_block(size_t);
 
 static gc_block* gc_primitive_alloc(size_t);
-static void gc_primitive_free(gc_block*);
+void gc_primitive_free(gc_block*);
 static void* gc_system_alloc(size_t);
 
-size_t gc_heap_base;
-size_t gc_block_base;
+uintp gc_heap_base;
+uintp gc_block_base;
 uintp gc_heap_range;
 
 typedef struct {
@@ -70,7 +72,7 @@ static gc_freelist freelist[NR_FREELISTS+1]
 	S(1000),
 	S(2016),
 	S(4040),
-	{ (gc_block *)0xffffffff, 0 }
+	{ (gc_block *)-1, 0 }
 }
 #endif /* PREDEFINED_NUMBER_OF_TILES */
 ;
@@ -109,7 +111,7 @@ static int totalsmallobjs;
 static void 
 printslack(void)
 {
-	fprintf(stderr, 
+	dprintf(
 		"allocated %d small objects, total slack %d, slack/per "
 		"object %8.2f\n", 
 		totalsmallobjs, totalslack, totalslack/(double)totalsmallobjs);
@@ -118,14 +120,14 @@ printslack(void)
 /*
  * check whether the heap is still in a consistent state
  */
-void
+static void
 gc_heap_check(void)
 {
 	int i; 
 
 	for (i = 0; i < NR_FREELISTS; i++) {
 		gc_block* blk = freelist[i].list;
-		if (blk == 0 || blk == (gc_block*)0xffffffff) {
+		if (blk == 0 || blk == (gc_block*)-1) {
 			continue;
 		} else {
 			gc_freeobj* mem = blk->free;
@@ -164,9 +166,9 @@ gc_heap_initialise(void)
 	 * Perform some sanity checks.
 	 */
 	if (gc_heap_initial_size > gc_heap_limit) {
-		fprintf(stderr, 
+		dprintf(
 		    "Initial heap size (%dK) > Maximum heap size (%dK)\n",
-		    gc_heap_initial_size/1024, gc_heap_limit/1024);
+		    (int) (gc_heap_initial_size/1024), (int)(gc_heap_limit/1024));
 		EXIT(-1);
 	}
 
@@ -272,6 +274,7 @@ gc_heap_malloc(size_t sz)
 	gc_block* blk;
 	size_t nsz;
 	int times;
+	int iLockRoot;
 
 	/* Initialise GC heap first time in - we must assume single threaded
 	 * operation here so we can do the lock initialising.
@@ -281,6 +284,8 @@ gc_heap_malloc(size_t sz)
 		gc_heap_init = 1;
 	}
 
+	lockStaticMutex(&gc_lock);
+
 	times = 0;
 
 DBG(SLACKANAL,
@@ -363,6 +368,8 @@ DBG(GCALLOC,	dprintf("gc_heap_malloc: large block %d at %p\n", sz, mem);	)
 
 	assert(GC_OBJECT_SIZE(mem) >= sz);
 
+	unlockStaticMutex(&gc_lock);
+
 	return (mem);
 
 	/* --------------------------------------------------------------- */
@@ -406,6 +413,7 @@ DBG(GCALLOC,	dprintf("gc_heap_malloc: large block %d at %p\n", sz, mem);	)
 			assert (ranout++ == 0 || !!!"Ran out of memory!");
 		}
 		/* Guess we've really run out */
+		unlockStaticMutex(&gc_lock);
 		return (0);
 	}
 
@@ -483,7 +491,9 @@ DBG(GCFREE,
 		info->size = msz;
 		gc_primitive_free(info);
 	}
-
+DBG(GCDIAG,
+	gc_heap_check();
+    )
 }
 
 /*
@@ -581,6 +591,10 @@ gc_large_block(size_t sz)
  * compiled with DEBUG.
  */ 
 
+#ifndef PROT_NONE
+#define PROT_NONE 0
+#endif
+
 #if !defined(HAVE_MPROTECT) || !defined(DEBUG)
 #define mprotect(A,L,P)
 #define ALL_PROT
@@ -653,7 +667,6 @@ DBG(GCPRIM,		dprintf("gc_primitive_alloc: %d bytes from freelist @ %p\n", ptr->s
 /*
  * Return a block of memory to the free list.
  */
-static
 void
 gc_primitive_free(gc_block* mem)
 {
@@ -739,6 +752,27 @@ DBG(GCPRIM,	dprintf("gc_primitive_free: Append (%d,%p) onto last in list\n", mem
 	}
 }
 
+/*
+ * Try to reserve some memory for OOM exception handling.  Gc once at
+ * the beginning.  We start out looking for an arbitrary number of
+ * pages (4), and cut our expectations in half until we are able to
+ * meet them.
+ */
+gc_block *
+gc_primitive_reserve(void)
+{
+	gc_block *r = 0;
+	size_t size = 4 * gc_pgsize;
+	
+	while (size >= gc_pgsize && !(r = gc_primitive_alloc(size))) {
+		if (size == gc_pgsize) {
+			break;
+		}
+		size /= 2;
+	}
+	return r;
+}
+
 /*
  * System memory management:  Obtaining additional memory from the
  * OS.  This looks more complicated than it is, since it does not require
@@ -834,7 +868,9 @@ gc_block_alloc(size_t size)
 		memset((void *)gc_block_base, 0, nblocks * sizeof(gc_block));
 	}
 
+	DBG(GCSYSALLOC, dprintf("pagealloc(%d)", size));
 	heap_addr = pagealloc(size);
+	DBG(GCSYSALLOC, dprintf(" => %p\n", heap_addr));
 
 	if (!heap_addr) return 0;
 	
@@ -848,7 +884,9 @@ gc_block_alloc(size_t size)
 		uintp old_blocks = gc_block_base;
 		int onb = nblocks;
 		int min_nb;	/* minimum size of array to hold heap_addr */
+#if defined(KAFFE_STATS)
 		static timespent growtime;
+#endif
 
 		startTiming(&growtime, "gctime-blockrealloc");
 		/* Pick a new size for the gc_block array.  Remember,
@@ -905,7 +943,7 @@ gc_block_alloc(size_t size)
 
 			R(gc_prim_freelist);
 
-			for (i = 0; freelist[i].list != (void *) -1; i++) 
+			for (i = 0; freelist[i].list != (void*)-1; i++) 
 				R(freelist[i].list);
 #undef R
 		}
@@ -929,12 +967,13 @@ gc_system_alloc(size_t sz)
 
 	assert(sz % gc_pgsize == 0);
 
-	/* If we will pass the heap boundary, return 0 to indicate that
-	 * we're run out.
-	 */
-	if (gc_heap_total <= gc_heap_limit && gc_heap_total + sz > gc_heap_limit) {
-		gc_heap_total += sz;
+	if (gc_heap_total == gc_heap_limit) {
 		return (0);
+	} else 	if (gc_heap_total + sz > gc_heap_limit) {
+		/* take as much memory as we can */
+		sz = gc_heap_limit - gc_heap_total;
+		assert(sz % gc_pgsize == 0);
+		DBG(GCSYSALLOC, dprintf("allocating up to limit\n"));
 	}
 #ifdef DEBUG
 	gc_system_alloc_cnt++;
@@ -949,6 +988,7 @@ DBG(GCSYSALLOC,
 		return (0);
 	}
 	gc_heap_total += sz;
+	assert(gc_heap_total <= gc_heap_limit);
 
 	/* Place block into the freelist for subsequent use */
 	DBG(GCDIAG, blk->magic = GC_MAGIC);
