@@ -54,20 +54,26 @@ soft_new(Hjava_lang_Class* c)
 	Hjava_lang_Object* obj;
 	errorInfo info;
 
-	if (processClass(c, CSTATE_COMPLETE, &info) == false) {
-		throwError(&info);
+	if (c->state != CSTATE_COMPLETE && processClass(c, CSTATE_COMPLETE, &info) == false) {
+		goto bad;
 	}
 #if GUARANA
-	obj = guarana_newObject(c, creator);
+	obj = guarana_newObject(c, creator, &info);
 #else
-	obj = newObject(c);
+	obj = newObjectChecked(c, &info);
 #endif
+	if (obj == 0) {
+		goto bad;
+	}
 
 DBG(NEWINSTR,	
 	dprintf("New object of type %s (%d,%x)\n", 
 		c->name->data, c->bfsize, obj); )
 
 	return (obj);
+bad:
+	throwError(&info);
+	return (0);
 }
 
 /*
@@ -81,18 +87,22 @@ soft_newarray(jint type, jint size)
 #endif
 {
 	Hjava_lang_Object* obj;
+	errorInfo info;
 
 	if (size < 0) {
 		throwException(NegativeArraySizeException);
 	}
 
-
 #if GUARANA
-	obj = guarana_newArray(TYPE_CLASS(type), size, creator);
+	obj = guarana_newArray(TYPE_CLASS(type), size, creator, &info);
 #else
-	obj = newArray(TYPE_CLASS(type), size);
+	obj = newArrayChecked(TYPE_CLASS(type), size, &info);
 #endif
 
+	if (obj == 0) {
+		throwError(&info);
+	}
+
 DBG(NEWINSTR,	
 	dprintf("New array of %s [%d] (%x)\n", 
 		TYPE_CLASS(type)->name->data, size, obj); )
@@ -111,17 +121,22 @@ soft_anewarray(Hjava_lang_Class* elclass, jint size)
 #endif
 {
 	Hjava_lang_Object* obj;
+	errorInfo info;
 
 	if (size < 0) {
 		throwException(NegativeArraySizeException);
 	}
 
 #if GUARANA
-	obj = guarana_newArray(elclass, size, creator);
+	obj = guarana_newArray(elclass, size, creator, &info);
 #else
-	obj = newArray(elclass, size);
+	obj = newArrayChecked(elclass, size, &info);
 #endif
 
+	if (obj == 0) {
+		throwError(&info);
+	}
+
 DBG(NEWINSTR,	
 	dprintf("New array object [%d] of %s (%x)\n", size,
 		elclass->name->data, obj); )
@@ -141,6 +156,7 @@ soft_multianewarray(Hjava_lang_Class* class, struct Hjava_lang_Object *creator,
 soft_multianewarray(Hjava_lang_Class* class, jint dims, slots* args)
 #endif
 {
+	errorInfo einfo;
         int array[MAXDIMS];
         Hjava_lang_Object* obj;
         jint arg;
@@ -151,7 +167,7 @@ soft_multianewarray(Hjava_lang_Class* class, jint dims, slots* args)
 		arraydims = array;
 	}
 	else {
-		arraydims = KCALLOC(dims+1, sizeof(int));
+		arraydims = checkPtr(KCALLOC(dims+1, sizeof(int)));
 	}
 
 	/* stack grows up, so move to the first dimension */
@@ -169,16 +185,19 @@ soft_multianewarray(Hjava_lang_Class* class, jint dims, slots* args)
 
         /* Mmm, okay now build the array using the wonders of recursion */
 #if GUARANA
-	obj = guarana_newMultiArray(class, creator, arraydims);
+	obj = guarana_newMultiArray(class, creator, arraydims, &einfo);
 #else
-        obj = newMultiArray(class, arraydims);
+        obj = newMultiArrayChecked(class, arraydims, &einfo);
 #endif
 
-
 	if (arraydims != array) {
 		KFREE(arraydims);
 	}
 
+	if (!obj) {
+		throwError(&einfo);
+	}
+	
         /* Return the base object */
 	return (obj);
 }
@@ -187,31 +206,29 @@ soft_multianewarray(Hjava_lang_Class* class, jint dims, slots* args)
 #if defined(TRANSLATOR)
 void*
 #if GUARANA
-soft_multianewarray(Hjava_lang_Class* class, struct Hjava_lang_Object *creator, jint dims, ...)
+soft_vmultianewarray(Hjava_lang_Class* class, struct Hjava_lang_Object *creator, jint dims, va_list ap)
 #else
-soft_multianewarray(Hjava_lang_Class* class, jint dims, ...)
+soft_vmultianewarray(Hjava_lang_Class* class, jint dims, va_list ap)
 #endif
 {
-	va_list ap;
+	errorInfo einfo;
 	int array[MAXDIMS];
 	int i;
 	jint arg;
 	Hjava_lang_Object* obj;
 	int* arraydims;
 
-        if (dims < MAXDIMS) {
+        if (dims < MAXDIMS-1) {
 		arraydims = array;
 	}
 	else {
-		arraydims = KCALLOC(dims+1, sizeof(int));
+		arraydims = checkPtr(KCALLOC(dims+1, sizeof(int)));
 	}
 
 	/* Extract the dimensions into an array */
-	va_start(ap, dims);
 	for (i = 0; i < dims; i++) {
 		arg = va_arg(ap, jint);
 		if (arg < 0) {
-			va_end(ap);
 			if (arraydims != array) {
 				KFREE(arraydims);
 			}
@@ -220,111 +237,138 @@ soft_multianewarray(Hjava_lang_Class* class, jint dims, ...)
 		arraydims[i] = arg;
 	}
 	arraydims[i] = -1;
-	va_end(ap);
 
 	/* Mmm, okay now build the array using the wonders of recursion */
 #if GUARANA
-	obj = guarana_newMultiArray(class, creator, arraydims);
+	obj = guarana_newMultiArray(class, creator, arraydims, &einfo);
 #else
-        obj = newMultiArray(class, arraydims);
+        obj = newMultiArrayChecked(class, arraydims, &einfo);
 #endif
 
 	if (arraydims != array) {
 		KFREE(arraydims);
 	}
 
+	if (!obj) {
+		throwError(&einfo);
+	}
+
 	/* Return the base object */
 	return (obj);
 }
+
+void*
+#if GUARANA
+soft_multianewarray(Hjava_lang_Class* class, struct Hjava_lang_Object *creator, jint dims, ...)
+#else
+soft_multianewarray(Hjava_lang_Class* class, jint dims, ...)
+#endif
+{
+	void* obj;
+	va_list ap;
+	va_start(ap, dims);
+#ifdef GUARANA
+	obj = soft_vmultianewarray(class, creator, dims, ap);
+#else	
+	obj = soft_vmultianewarray(class, dims, ap);
+#endif
+	va_end(ap);
+	return (obj);
+}
 #endif
 
 /*
- * soft_lookupmethod.
+ * soft_lookupinterfacemethod.
  */
 void*
-soft_lookupmethod(Hjava_lang_Object* obj, Hjava_lang_Class* ifclass, int idx)
+soft_lookupinterfacemethod(Hjava_lang_Object* obj, Hjava_lang_Class* ifclass, int idx)
 {
 	Hjava_lang_Class* cls;
-	Hjava_lang_Class** c;
 	Method* meth;
-	int i;
-	int dtableidx;
-	errorInfo info;
+	void*	ncode;
+	register int i;
+	register short* implementors;
+
+	if (obj == 0) {
+		throwException(NullPointerException);
+	}
 
 	cls = OBJECT_CLASS(obj);
+	implementors = ifclass->implementors;
+	i = cls->impl_index;
+
+#if 1	/* it should never be necessary to initialize the class here
+	 * cause we're invoking a method on an existing, created object
+	 */
+        assert (cls->state >= CSTATE_USABLE);
+#else
 	/* initialize class if necessary */
         if (cls->state < CSTATE_USABLE) {
+		errorInfo info;
 		if (processClass(cls, CSTATE_COMPLETE, &info) == false) {
 			throwError(&info);
 		}
 	}
+#endif
 
-	c = cls->interfaces;
-	i = 0;
-	while (*c != ifclass) {
-		c++;
-		/* do this check inside the loop to avoid loading 
-		 * total_interface_len from memory if we hit the first one. 
-		 */
-		if (++i == cls->total_interface_len) {
-			goto notfound;
-		}
-	}
-
-	dtableidx = cls->itable2dtable[cls->if2itable[i] + idx];
-	if (dtableidx == -1) {
+	/* handle invocations on java.lang.Object methods via INVOKEINTERFACE */
+	if (implementors == 0 || i > implementors[0]) {
 		goto notfound;
 	}
 
-#if GUARANA
-	meth = DMETHOD_DESC(*(dispatchMethod*)
-			    ((void*)cls->dtable + DTABLE_METHODOFFSET
-			     + dtableidx * DTABLE_METHODSIZE));
+	/* skip word at the beginning of itable2dtable */
+	ncode = DMETHOD_ADDR(cls->itable2dtable[implementors[i] + idx + 1]);
+
+	/* This means this class does not implement this interface method 
+	 * at all.  This is something we detect at the time the interface
+	 * dispatch table is built.  To avoid this test, we could instead
+	 * point at a nosuchmethod routine there.  However, we would have
+	 * to find a way to pass along the information which methods it
+	 * is that's missing (or create multiple nosuch_method routines,
+	 * given that they should be rare---minus possible DoS.)
+	 */
+	if (ncode == (void *)-1) {
+		goto notfound;
+	}
+
+	assert(ncode != NULL);
+#if defined GUARANA && defined TRANSLATOR
+	return &cls->itable2dtable[implementors[i] + idx + 1];
 #else
-	meth = *(Method**)((void*)cls->dtable 
-			+ DTABLE_METHODOFFSET + dtableidx * DTABLE_METHODSIZE);
+	return (ncode);
 #endif
-	return (meth);
 
 notfound:
+	/*
+	 * Compilers following the latest version of the JLS emit a 
+	 * INVOKEINTERFACE instruction for methods that aren't defined in 
+	 * an interface, but inherited from Object.
+	 *
+	 * In this case, the JVM must
+	 * a) check that the object really implements the interface given
+	 * b) find and invoke the method on object.
+	 *
+	 * The best way to jit that would be a checkcast <interface_type>
+	 * followed by an INVOKEVIRTUAL.
+	 *
+	 * For now, we simply detect the case where an object method is called
+	 * and find it by hand using its (name, signature). 
+	 */
+	if (ifclass == ObjectClass) {
+		Method* objm = CLASS_METHODS(ifclass) + idx;
+		errorInfo info;
+		
+		meth = findMethod(cls, objm->name, METHOD_SIG(objm), &info);
+		if (meth == 0) {
+			throwError(&info);
+		}
+		return (METHOD_INDIRECTMETHOD(meth));
+	}
 	meth = CLASS_METHODS(ifclass) + idx;
-	soft_nosuchmethod(cls, meth->name, meth->signature);
+	soft_nosuchmethod(cls, meth->name, METHOD_SIG(meth));
 	return (0);
 }
 
-#if 0
-/*
- * soft_lookupinterfacemethod
- * Lookup an interface method, given its name and signature.  Throw exceptions
- * if the method found doesn't fit with wht invokeinterface is expecting.
- */
-void*
-soft_lookupinterfacemethod(Hjava_lang_Class* class, Utf8Const* name, Utf8Const* signature)
-{
-	processClass(class, CSTATE_LINKED);
-	for (; class != 0; class = class->superclass) {
-		Method* mptr = findMethodLocal(class, name, signature);
-		if (mptr != NULL) {
-			if (mptr->accflags & ACC_STATIC) {
-				throwException(IncompatibleClassChangeError);
-			}
-			if (mptr->accflags & ACC_ABSTRACT) {
-				throwException(AbstractMethodError);
-			}
-			if (!(mptr->accflags & ACC_PUBLIC)) {
-				throwException(IllegalAccessError);
-			}
-#if defined(TRANSLATOR)
-			return (METHOD_NATIVECODE(mptr));
-#else
-			return (mptr);
-#endif
-		}
-	}
-	throwException(IncompatibleClassChangeError);
-}
-#endif
-
 inline
 jint
 instanceof_class(Hjava_lang_Class* c, Hjava_lang_Class* oc)
@@ -442,8 +486,8 @@ soft_checkcast(Hjava_lang_Class* c, Hjava_lang_Object* o)
 		const char *fromtype = CLASS_CNAME(OBJECT_CLASS(o));
 		const char *totype = CLASS_CNAME(c);
 		char *format = "can't cast `%s' to `%s'";
-		char *buf = KMALLOC(strlen(fromtype) 
-			+ strlen(totype) + strlen(format));
+		char *buf = checkPtr(KMALLOC(strlen(fromtype) 
+			+ strlen(totype) + strlen(format)));
 		sprintf(buf, format, fromtype, totype);
 		ccexc = ClassCastException(buf);
 		KFREE(buf);
@@ -492,17 +536,60 @@ soft_nullpointer(void)
 	throwException(NullPointerException);
 }
 
+/*
+ * soft_divzero.
+ */
+void
+soft_divzero(void)
+{
+	throwException(ArithmeticException);
+}
+
+/*
+ * soft_stackoverflow.
+ */
+void
+soft_stackoverflow(void)
+{
+	throwException(unhand(getCurrentThread())->stackOverflowError);
+}
+
+/*
+ * soft_nosuchclass.
+ */
+void            
+soft_nosuchclass(Utf8Const* c)
+{
+	char buf[256];
+
+	sprintf(buf, "%.80s", c->data);
+	throwException(NoClassDefFoundError(buf));
+}
+
 /*
  * soft_nosuchmethod.
  */
 void            
 soft_nosuchmethod(Hjava_lang_Class* c, Utf8Const* n, Utf8Const* s)
 {
-	char buf[200];	/* FIXME: unchecked buffer */
-	sprintf(buf, "%s.%s%s", CLASS_CNAME(c), n->data, s->data);
+	char buf[256];
+
+	sprintf(buf, "%.80s.%.80s%.80s", CLASS_CNAME(c), n->data, s->data);
 	throwException(NoSuchMethodError(buf));
 }
 
+/*
+ * soft_nosuchfield.
+ */
+void
+soft_nosuchfield(Utf8Const* c, Utf8Const* n)
+{
+	char buf[256];
+
+	sprintf(buf, "%.100s.%.100s", c->data, n->data);
+	throwException(NoSuchFieldError(buf));
+}
+
 /*
  * soft_initialise_class.
  */
@@ -526,8 +613,13 @@ nativecode*
 soft_fixup_trampoline(FIXUP_TRAMPOLINE_DECL)
 {
 	Method* meth;
+	void **where;
+	void *tramp;
 	errorInfo info;
+
+	/* FIXUP_TRAMPOLINE_INIT sets tramp and where */
 	FIXUP_TRAMPOLINE_INIT;
+	tramp = *where;
 
 	/* If this class needs initializing, do it now.  */
 	if (meth->class->state < CSTATE_USABLE &&
@@ -542,13 +634,21 @@ soft_fixup_trampoline(FIXUP_TRAMPOLINE_DECL)
 		}
 	}
 
-	/* Update dispatch table */
-	if (meth->idx >= 0) {
-#if GUARANA
-		DMETHOD_ADDR(meth->class->dtable->method[meth->idx]) = METHOD_NATIVECODE(meth);
-#else
-		meth->class->dtable->method[meth->idx] = METHOD_NATIVECODE(meth);
-#endif
+	/* Update the origin of this trampoline --- this could be a 
+	 * dispatch table, interface dispatch table, or a gcj trampoline
+	 *
+	 * This is rather tricky.  We may jump through the same trampoline
+	 * twice.  For instance, class A refers to Double.longBitsToDouble,
+	 * but Double.<clinit> also jumps through the same trampoline
+	 * before it is patched.
+	 */
+	jthread_spinon(0);
+	if (*where != *(void**)PMETHOD_NATIVECODE(meth)) {
+		*where = METHOD_NATIVECODE(meth);
+		jthread_spinoff(0);
+		KFREE(tramp);
+	} else {
+		jthread_spinoff(0);
 	}
 
 #if 0
@@ -565,7 +665,7 @@ soft_fixup_trampoline(FIXUP_TRAMPOLINE_DECL)
 	}
 #endif
 
-TDBG(	fprintf(stderr, "Calling %s:%s%s @ 0x%x\n", meth->class->name->data, meth->name->data, meth->signature->data, METHOD_NATIVECODE(meth));	)
+TDBG(	dprintf("Calling %s:%s%s @ 0x%x\n", meth->class->name->data, meth->name->data, METHOD_SIGD(meth), METHOD_NATIVECODE(meth));	)
 
 	return (METHOD_NATIVECODE(meth));
 }
@@ -634,7 +734,13 @@ jint
 soft_fcmpg(jfloat v1, jfloat v2)
 {
         jint ret;
-	if ((!isinf(v1) && isnan(v1)) || (!isinf(v2) && isnan(v2))) {
+	jint v1bits;
+	jint v2bits;
+
+	v1bits = floatToInt(v1);
+	v2bits = floatToInt(v2);
+
+        if (FISNAN(v1bits) || FISNAN(v2bits)) {
 		ret = 1;
 	}
         else if (v1 > v2) {
@@ -656,7 +762,13 @@ jint
 soft_fcmpl(jfloat v1, jfloat v2)
 {
         jint ret;
-	if ((!isinf(v1) && isnan(v1)) || (!isinf(v2) && isnan(v2))) {
+	jint v1bits;
+	jint v2bits;
+
+	v1bits = floatToInt(v1);
+	v2bits = floatToInt(v2);
+
+        if (FISNAN(v1bits) || FISNAN(v2bits)) {
 		ret = -1;
 	}
         else if (v1 > v2) {
@@ -689,6 +801,43 @@ soft_lrem(jlong v1, jlong v2)
 	return (v1 % v2);
 }
 
+jfloat
+soft_fadd(jfloat v1, jfloat v2)
+{
+	return floatAdd(v1, v2);
+}
+
+jdouble
+soft_faddl(jdouble v1, jdouble v2)
+{
+	return doubleAdd(v1, v2);
+}
+
+jfloat
+soft_fsub(jfloat v1, jfloat v2)
+{
+	return floatSubtract(v1, v2);
+}
+
+jdouble
+soft_fsubl(jdouble v1, jdouble v2)
+{
+	return doubleSubtract(v1, v2);
+}
+
+
+jfloat
+soft_fmul(jfloat v1, jfloat v2)
+{
+	return floatMultiply(v1, v2);
+}
+
+jdouble
+soft_fmull(jdouble v1, jdouble v2)
+{
+	return doubleMultiply(v1, v2);
+}
+
 jfloat
 soft_fdiv(jfloat v1, jfloat v2)
 {
@@ -802,13 +951,29 @@ soft_cvtld(jlong v)
 jdouble
 soft_cvtfd(jfloat v)
 {
-	return ((jdouble)v);
+	jint vbits;
+
+	vbits = floatToInt(v);
+        if (FISNAN(vbits)) {
+		return (longToDouble(DNANBITS));
+	}
+	else {
+		return ((jdouble)v);
+	}
 }
 
 jfloat
 soft_cvtdf(jdouble v)
 {
-	return ((jfloat)v);
+	jlong vbits;
+
+	vbits = doubleToLong(v);
+        if (DISNAN(vbits)) {
+		return (intToFloat(FNANBITS));
+	}
+	else {
+		return ((jfloat)v);
+	}
 }
 
 /*
@@ -819,43 +984,130 @@ soft_cvtdf(jdouble v)
 jlong
 soft_cvtfl(jfloat v)
 {
+	jint vbits;
+
+	vbits = floatToInt(v);
+        if (FISNAN(vbits)) {
+		return ((jlong)0);
+	}
+
 	if (v < 0.0) {
-		return ((jlong)ceil(v));
+		v = ceil(v);
 	}
 	else {
-		return ((jlong)floor(v));
+		v = floor(v);
+	}
+	/* If too small return smallest long */
+	if (v <= -9223372036854775808.0) {
+	    return ((jlong)1) << 63;
+	}
+	/* If too big return biggest long */
+	else if (v >= 9223372036854775807.0) {
+	    return ~(((jlong)1) << 63);
+	}
+	else {
+	    return ((jlong)v);
 	}
 }
 
 jlong
 soft_cvtdl(jdouble v)
 {
+	jlong vbits;
+
+	vbits = doubleToLong(v);
+        if (DISNAN(vbits)) {
+		return ((jlong)0);
+	}
+
 	if (v < 0.0) {
-		return ((jlong)ceil(v));
+		v = ceil(v);
 	}
 	else {
-		return ((jlong)floor(v));
+		v = floor(v);
+	}
+	/* If too small return smallest long */
+	if (v <= -9223372036854775808.0) {
+	    return ((jlong)1) << 63;
+	}
+	/* If too big return biggest long */
+	else if (v >= 9223372036854775807.0) {
+	    return ~(((jlong)1) << 63);
+	}
+	else {
+	    return ((jlong)v);
 	}
 }
 
 jint
 soft_cvtfi(jfloat v)
 {
+        jint vbits;
+
+	vbits = floatToInt(v);
+        if (FISNAN(vbits)) {
+		return (0);
+	}
+
 	if (v < 0.0) {
-		return ((jint)ceil(v));
+		v = ceil(v);
 	}
 	else {
-		return ((jint)floor(v));
+		v = floor(v);
+	}
+	/* If too small return smallest int */
+	if (v <= -2147483648.0) {
+		return (-2147483647-1);
+	}
+	/* If too big return biggest int */
+	else if (v >= 2147483647.0) {
+		return (2147483647);
+	}
+	else {
+		return ((jint)v);
 	}
 }
 
 jint
 soft_cvtdi(jdouble v)
 {
+        jlong vbits;
+
+	vbits = doubleToLong(v);
+        if (DISNAN(vbits)) {
+		return (0);
+	}
+
 	if (v < 0.0) {
-		return ((jint)ceil(v));
+		v = ceil(v);
 	}
 	else {
-		return ((jint)floor(v));
+		v = floor(v);
+	}
+	/* If too small return smallest int */
+	if (v <= -2147483648.0) {
+		return (-2147483647-1);
+	}
+	/* If too big return biggest int */
+	else if (v >= 2147483647.0) {
+		return (2147483647);
+	}
+	else {
+		return ((jint)v);
 	}
 }
+
+void
+soft_debug1(void* a0, void* a1, void* a2)
+{
+}
+
+void
+soft_debug2(void* a0, void* a1, void* a2)
+{
+}
+
+void
+soft_trace(Method* meth, void* args)
+{
+}
