@@ -16,7 +16,6 @@
 #include "jni.h"
 #include <stdarg.h>
 #include "errors.h"
-#include "classMethod.h"
 #include "jtypes.h"
 #include "access.h"
 #include "object.h"
@@ -27,7 +26,10 @@
 #include "exception.h"
 #include "slots.h"
 #include "support.h"
-#define NEED_sysdepCallMethod 1
+#ifndef HAVE_LIBFFI
+# define NEED_sysdepCallMethod 1
+#endif /* HAVE_LIBFFI */
+#include "classMethod.h"
 #include "machine.h"
 #include "md.h"
 #include "itypes.h"
@@ -45,6 +47,7 @@
 #endif
 
 #define	MAXEXCEPTIONLEN		200
+#define	ERROR_SIGNATURE0	"()V"
 #define	ERROR_SIGNATURE		"(Ljava/lang/String;)V"
 
 /* Anchor point for user defined properties */
@@ -58,6 +61,60 @@ nativeFunction* native_funcs = null_funcs;
 
 extern struct JNIEnv_ Kaffe_JNIEnv;
 
+#ifdef HAVE_LIBFFI
+#include <ffi.h>
+static inline ffi_type *j2ffi(char type)
+{
+	ffi_type *ftype;
+
+	switch (type) {
+	case 'J':
+		ftype = &ffi_type_sint64;
+		break;
+	case 'D':
+		ftype = &ffi_type_double;
+		break;
+	case 'F':
+		ftype = &ffi_type_float;
+		break;
+	default:
+		ftype = &ffi_type_uint;
+	}
+	return ftype;
+}
+
+#define sysdepCallMethod(CALL)						\
+do {									\
+	int i;								\
+	ffi_cif cif;							\
+	ffi_type *rtype;						\
+	ffi_type *argtypes[(CALL)->nrargs + 1];				\
+	void *argvals[(CALL)->nrargs + 1];				\
+	int fargs = 0;							\
+	for (i = 0; i < (CALL)->nrargs; i++) {				\
+		switch ((CALL)->callsize[i]) {				\
+		case 1:							\
+			argtypes[fargs] = j2ffi((CALL)->calltype[i]);	\
+			argvals[fargs] = &((CALL)->args[i].i);		\
+			fargs++;					\
+			break;						\
+		case 2:							\
+			argtypes[fargs] = j2ffi((CALL)->calltype[i]);	\
+			argvals[fargs] = &((CALL)->args[i].j);		\
+			fargs++;					\
+		default:						\
+			break;						\
+		}							\
+	}								\
+	rtype = j2ffi ((CALL)->rettype);				\
+	if (ffi_prep_cif(&cif, FFI_DEFAULT_ABI, fargs,			\
+	    rtype, argtypes) == FFI_OK) {				\
+		ffi_call(&cif, (CALL)->function, (CALL)->ret, argvals);	\
+	}								\
+	else								\
+		ABORT();						\
+} while (0);
+#endif /* HAVE_LIBFFI */
 
 /*
  * Call a Java method from native code.
@@ -71,6 +128,10 @@ do_execute_java_method_v(void* obj, const char* method_name, const char* signatu
 	struct HBR_unicamp_Guarana_MetaObject *mo;
 #endif
 
+	if (obj == 0 && (! isStaticCall || ! mb)) {
+		throwException(NullPointerException);
+	}
+
 	if (mb == 0) {
 		assert(method_name != 0 && signature != 0);
 		if (isStaticCall) {
@@ -129,25 +190,29 @@ do_execute_java_method(void* obj, const char* method_name, const char* signature
  * Call a Java static method on a class from native code.
  */
 jvalue
-do_execute_java_class_method_v(const char* cname, const char* method_name, const char* signature, va_list argptr)
+do_execute_java_class_method_v(const char* cname,
+	Hjava_lang_ClassLoader* loader, const char* method_name,
+	const char* signature, va_list argptr)
 {
-	Method* mb = 0;
-	jvalue retval;
-	char cnname[CLASSMAXSIG];	/* Unchecked buffer - FIXME! */
 	Hjava_lang_Class* clazz;
 	errorInfo info;
+	Method* mb = 0;
+	jvalue retval;
+	char *buf;
 #if GUARANA
 	struct HBR_unicamp_Guarana_MetaObject *mo;
 #endif
 
-	/* Convert "." to "/" */
-	classname2pathname(cname, cnname);
+	/* Convert "." to "/" and lookup class */
+	buf = checkPtr(KMALLOC(strlen(cname) + 1));
+	classname2pathname(cname, buf);
+	clazz = lookupClass(buf, loader, &info);
+	KFREE(buf);
 
-	clazz = lookupClass(cnname, &info);
+	/* Get method */
 	if (clazz != 0) {
 		mb = lookupClassMethod(clazz, method_name, signature, &info);
 	}
-
 	if (mb == 0) {
 		throwError(&info);
 	}
@@ -174,13 +239,14 @@ do_execute_java_class_method_v(const char* cname, const char* method_name, const
 }
 
 jvalue
-do_execute_java_class_method(const char* cname, const char* method_name, const char* signature, ...)
+do_execute_java_class_method(const char* cname, Hjava_lang_ClassLoader*loader,
+	const char* method_name, const char* signature, ...)
 {
 	va_list argptr;
 	jvalue retval;
 
 	va_start(argptr, signature);
-	retval = do_execute_java_class_method_v(cname, method_name, signature, argptr);
+	retval = do_execute_java_class_method_v(cname, loader, method_name, signature, argptr);
 	va_end(argptr);
 
 	return (retval);
@@ -190,11 +256,11 @@ do_execute_java_class_method(const char* cname, const char* method_name, const c
  * Allocate an object and execute the constructor.
  */
 Hjava_lang_Object*
-execute_java_constructor_v(const char* cname, Hjava_lang_Class* cc, const char* signature, va_list argptr)
+execute_java_constructor_v(const char* cname, Hjava_lang_ClassLoader* loader,
+	Hjava_lang_Class* cc, const char* signature, va_list argptr)
 {
 	Hjava_lang_Object* obj;
 	Method* mb;
-	char buf[MAXEXCEPTIONLEN];
 	jvalue retval;
 	errorInfo info;
 	Utf8Const* sig;
@@ -203,11 +269,16 @@ execute_java_constructor_v(const char* cname, Hjava_lang_Class* cc, const char*
 #endif
 
 	if (cc == 0) {
-		/* Convert "." to "/" */
-		classname2pathname(cname, buf);
+		char *buf;
 
-		cc = lookupClass(buf, &info);
-		assert(cc != 0);
+		/* Convert "." to "/" and lookup class */
+		buf = checkPtr(KMALLOC(strlen(cname) + 1));
+		classname2pathname(cname, buf);
+		cc = lookupClass(buf, loader, &info);
+		KFREE(buf);
+		if (!cc) {
+			throwError(&info);
+		}
 	}
 
 	/* We cannot construct interfaces or abstract classes */
@@ -221,7 +292,7 @@ execute_java_constructor_v(const char* cname, Hjava_lang_Class* cc, const char*
 		}
 	}
 
-	sig = utf8ConstNew(signature, -1);
+	sig = checkPtr(utf8ConstNew(signature, -1));
 	mb = findMethodLocal(cc, constructor_name, sig);
 	utf8ConstRelease(sig);
 	if (mb == 0) {
@@ -247,13 +318,14 @@ execute_java_constructor_v(const char* cname, Hjava_lang_Class* cc, const char*
 }
 
 Hjava_lang_Object*
-execute_java_constructor(const char* cname, Hjava_lang_Class* cc, const char* signature, ...)
+execute_java_constructor(const char* cname, Hjava_lang_ClassLoader* loader,
+	Hjava_lang_Class* cc, const char* signature, ...)
 {
 	va_list argptr;
 	Hjava_lang_Object* obj;
 
 	va_start(argptr, signature);
-	obj = execute_java_constructor_v(cname, cc, signature, argptr);
+	obj = execute_java_constructor_v(cname, loader, cc, signature, argptr);
 	va_end(argptr);
 
 	return (obj);
@@ -317,8 +389,8 @@ void
 callMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
 	    int promoted)
 {
-	const char* sig;
 	int i;
+	int j;
 	int s;
 	/* XXX call.callsize and call.calltype arrays are statically sized 
 	   and are not checked for running out of bounds */
@@ -329,7 +401,6 @@ callMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
 	if (ret == 0) {
 		ret = &tmp;
 	}
-	sig = meth->signature->data;
 	i = 0;
 	s = 0;
 
@@ -342,7 +413,7 @@ callMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
 	 */
 	meth = (Method*)func;
 	if (meth->accflags & ACC_NATIVE) {
-		if (METHOD_NATIVECODE(meth) == 0) {
+		if (!METHOD_TRANSLATED(meth)) {
 			errorInfo info;
 			if (native(meth, &info) == false) {
 				throwError(&info);
@@ -384,10 +455,9 @@ callMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
 		args--; /* because args[i] would be off by one */
 	}
 
-	sig++;	/* Skip leading '(' */
-	for (; *sig != ')'; i++, sig++) {
-		call.calltype[i] = *sig;
-		switch (*sig) {
+	for (j = 0; j < METHOD_NARGS(meth); i++, j++) {
+		call.calltype[i] = *METHOD_ARG_TYPE(meth, j);
+		switch (call.calltype[i]) {
 		case 'Z':
 			if (promoted) goto use_int;
 			call.callsize[i] = 1;
@@ -439,47 +509,46 @@ callMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
 			call.callsize[i] = 0;
 #endif
 			break;
-
 		case '[':
-			call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
-			call.calltype[i] = 'L';	/* Looks like an object */
-			in[i] = args[i];
-			while (*sig == '[') {
-				sig++;
-			}
-			if (*sig == 'L') {
-				while (*sig != ';') {
-					sig++;
-				}
-			}
-			break;
+			call.calltype[i] = 'L';
+			/* fall through */
 		case 'L':
 			call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
 			in[i] = args[i];
-			while (*sig != ';') {
-				sig++;
-			}
 			break;
 		default:
 			ABORT();
 		}
 		s += call.callsize[i];
 	}
-	sig++;	/* Skip trailing ')' */
+
+#if defined(STACK_LIMIT)
+	call.calltype[i] = 'L';
+	call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
+	in[i].l = jthread_stacklimit();
+	s += PTR_TYPE_SIZE / SIZEOF_INT;
+	i++;
+#endif
 
 	/* Return info */
-	call.rettype = *sig;
-	if (*sig == 'L' || *sig == '[') {
+	call.rettype = *METHOD_RET_TYPE(meth);
+	switch (call.rettype) {
+	case '[':
+		call.rettype = 'L';
+		/* fall through */
+	case 'L':
 		call.retsize = PTR_TYPE_SIZE / SIZEOF_INT;
-	}
-	else if (*sig == 'V') {
+		break;
+	case 'V':
 		call.retsize = 0;
-	}
-	else if (*sig == 'D' || *sig == 'J') {
+		break;
+	case 'D':
+	case 'J':
 		call.retsize = 2;
-	}
-	else {
+		break;
+	default:
 		call.retsize = 1;
+		break;
 	}
 
 	/* Call info and arguments */
@@ -498,27 +567,37 @@ callMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
 		virtualMachine(meth, (slots*)call.args, (slots*)call.ret, getCurrentThread());
 	}
 	else {
-		iLock* sync;
+		Hjava_lang_Object* syncobj = 0;
+		vmException mjbuf;
+		Hjava_lang_Thread* tid = getCurrentThread();
 
 		if (meth->accflags & ACC_SYNCHRONISED) {
-			Hjava_lang_Object* syncobj;
 			if (meth->accflags & ACC_STATIC) {
 				syncobj = &meth->class->head;
 			}
 			else {
 				syncobj = (Hjava_lang_Object*)call.args[0].l;
 			}
-			sync = lockObjMutex(syncobj);
+			lockObject(syncobj);
 		}
-		else {
-			sync = 0;
+
+		mjbuf.pc = 0;
+		mjbuf.mobj = syncobj;
+		mjbuf.meth = meth;
+		if (tid != NULL && unhand(tid)->PrivateInfo != 0) {
+			mjbuf.prev = (vmException*)unhand(tid)->exceptPtr;
+			unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)&mjbuf;
 		}
 
 		/* Make the call - system dependent */
 		sysdepCallMethod(&call);
 
-		if (sync != 0) {
-			unlockObjKnownMutex(sync);
+		if (syncobj != 0) {
+			unlockObject(syncobj);
+		}
+
+		if (tid != NULL && unhand(tid)->PrivateInfo != 0) {
+			unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)mjbuf.prev;
 		}
 	}
 #endif
@@ -546,9 +625,10 @@ callMethodA(Method* meth, void* func, void* obj, jvalue* args, jvalue* ret,
 void
 callMethodV(Method* meth, void* func, void* obj, va_list args, jvalue* ret)
 {
-	const char* sig;
+  /* const char* sig; FIXME */
 	int i;
 	int s;
+	int j;
 	/* XXX call.callsize and call.calltype arrays are statically sized 
 	   and are not checked for running out of bounds */
 	callMethodInfo call;
@@ -558,7 +638,6 @@ callMethodV(Method* meth, void* func, void* obj, va_list args, jvalue* ret)
 	if (ret == 0) {
 		ret = &tmp;
 	}
-	sig = meth->signature->data;
 	i = 0;
 	s = 0;
 
@@ -606,10 +685,9 @@ callMethodV(Method* meth, void* func, void* obj, va_list args, jvalue* ret)
 		i++;
 	}
 
-	sig++;	/* Skip leading '(' */
-	for (; *sig != ')'; i++, sig++) {
-		call.calltype[i] = *sig;
-		switch (*sig) {
+	for (j = 0; j < METHOD_NARGS(meth); i++, j++) {
+		call.calltype[i] = *METHOD_ARG_TYPE(meth, j);
+		switch (call.calltype[i]) {
 		case 'I':
 		case 'Z':
 		case 'S':
@@ -643,45 +721,45 @@ callMethodV(Method* meth, void* func, void* obj, va_list args, jvalue* ret)
 #endif
 			break;
 		case '[':
-			call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
-			call.calltype[i] = 'L';	/* Looks like an object */
-			in[i].l = va_arg(args, jref);
-			while (*sig == '[') {
-				sig++;
-			}
-			if (*sig == 'L') {
-				while (*sig != ';') {
-					sig++;
-				}
-			}
-			break;
+			call.calltype[i] = 'L';
+			/* fall through */
 		case 'L':
 			call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
 			in[i].l = va_arg(args, jref);
-			while (*sig != ';') {
-				sig++;
-			}
 			break;
 		default:
 			ABORT();
 		}
 		s += call.callsize[i];
 	}
-	sig++;	/* Skip trailing ')' */
+
+#if defined(STACK_LIMIT)
+	call.calltype[i] = 'L';
+	call.callsize[i] = PTR_TYPE_SIZE / SIZEOF_INT;
+	in[i].l = jthread_stacklimit();
+	s += PTR_TYPE_SIZE / SIZEOF_INT;
+	i++;
+#endif
 
 	/* Return info */
-	call.rettype = *sig;
-	if (*sig == 'L' || *sig == '[') {
+	call.rettype = *METHOD_RET_TYPE(meth);
+	switch (call.rettype) {
+	case '[':
+		call.rettype = 'L';
+		/* fall through */
+	case 'L':
 		call.retsize = PTR_TYPE_SIZE / SIZEOF_INT;
-	}
-	else if (*sig == 'V') {
+		break;
+	case 'V':
 		call.retsize = 0;
-	}
-	else if (*sig == 'D' || *sig == 'J') {
+		break;
+	case 'D':
+	case 'J':
 		call.retsize = 2;
-	}
-	else {
+		break;
+	default:
 		call.retsize = 1;
+		break;
 	}
 
 	/* Call info and arguments */
@@ -700,27 +778,37 @@ callMethodV(Method* meth, void* func, void* obj, va_list args, jvalue* ret)
 		virtualMachine(meth, (slots*)call.args, (slots*)call.ret, getCurrentThread());
 	}
 	else {
-		iLock* sync;
+		Hjava_lang_Object* syncobj = 0;
+		vmException mjbuf;
+		Hjava_lang_Thread* tid = getCurrentThread();
 
 		if (meth->accflags & ACC_SYNCHRONISED) {
-			Hjava_lang_Object* syncobj;
 			if (meth->accflags & ACC_STATIC) {
 				syncobj = &meth->class->head;
 			}
 			else {
 				syncobj = (Hjava_lang_Object*)call.args[0].l;
 			}
-			sync = lockObjMutex(syncobj);
+			lockObject(syncobj);
 		}
-		else {
-			sync = 0;
+
+		mjbuf.pc = 0;
+		mjbuf.mobj = syncobj;
+		mjbuf.meth = meth;
+		if (tid != NULL && unhand(tid)->PrivateInfo != 0) {
+			mjbuf.prev = (vmException*)unhand(tid)->exceptPtr;
+			unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)&mjbuf;
 		}
 
 		/* Make the call - system dependent */
 		sysdepCallMethod(&call);
 
-		if (sync != 0) {
-			unlockObjKnownMutex(sync);
+		if (syncobj != 0) {
+			unlockObject(syncobj);
+		}
+
+		if (tid != NULL && unhand(tid)->PrivateInfo != 0) {
+			unhand(tid)->exceptPtr = (struct Hkaffe_util_Ptr*)mjbuf.prev;
 		}
 	}
 #endif
@@ -738,7 +826,16 @@ lookupClassMethod(Hjava_lang_Class* cls, const char* name, const char* sig, erro
 	assert(cls != 0 && name != 0 && sig != 0);
 
 	name_utf8 = utf8ConstNew(name, -1);
+	if (!name_utf8) {
+		postOutOfMemory(einfo);
+		return 0;
+	}
 	sig_utf8 = utf8ConstNew(sig, -1);
+	if (!sig_utf8) {
+		utf8ConstRelease(name_utf8);
+		postOutOfMemory(einfo);
+		return 0;
+	}
 	meth = findMethod(cls, name_utf8, sig_utf8, einfo);
 	utf8ConstRelease(name_utf8);
 	utf8ConstRelease(sig_utf8);
@@ -764,8 +861,13 @@ SignalError(const char* cname, const char* str)
 {
 	Hjava_lang_Throwable* obj;
 
-	obj = (Hjava_lang_Throwable*)execute_java_constructor(cname,
-		0, ERROR_SIGNATURE, stringC2Java(str));
+	if (str == NULL || *str == '\0') {
+		obj = (Hjava_lang_Throwable*)execute_java_constructor(cname,
+			0, 0, ERROR_SIGNATURE0);
+	} else {
+		obj = (Hjava_lang_Throwable*)execute_java_constructor(cname,
+			0, 0, ERROR_SIGNATURE, checkPtr(stringC2Java(str)));
+	}
 	throwException(obj);
 }
 
@@ -836,6 +938,10 @@ currentTime(void)
 	struct timeb tm;
 	ftime(&tm);
 	tme = (((jlong)tm.time * (jlong)1000) + (jlong)tm.millitm);
+#elif defined(HAVE__FTIME)
+	struct timeb tm;
+	_ftime(&tm);
+	tme = (((jlong)tm.time * (jlong)1000) + (jlong)tm.millitm);
 #elif defined(HAVE_TIME)
 	tme = (jlong)1000 * (jlong)time(0);
 #else
@@ -853,8 +959,8 @@ setProperty(void* properties, char* key, char* value)
 	Hjava_lang_String* jkey;
 	Hjava_lang_String* jvalue;
 
-	jkey = stringC2Java(key);
-	jvalue = stringC2Java(value);
+	jkey = checkPtr(stringC2Java(key));
+	jvalue = checkPtr(stringC2Java(value));
 
 	do_execute_java_method(properties, "put",
 		"(Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;",
@@ -865,11 +971,12 @@ setProperty(void* properties, char* key, char* value)
  * Allocate a new object of the given class name.
  */
 Hjava_lang_Object*
-AllocObject(const char* classname)
+AllocObject(const char* classname, Hjava_lang_ClassLoader* loader)
 {
+	Hjava_lang_Class* clazz;
 	errorInfo info;
-	Hjava_lang_Class* clazz = lookupClass(classname, &info);
 
+	clazz = lookupClass(classname, loader, &info);
 	if (clazz == 0) {
 		throwError(&info);
 	}
@@ -889,7 +996,7 @@ AllocArray(int len, int type)
  * Allocate a new array of the given size and class name.
  */
 Hjava_lang_Object*
-AllocObjectArray(int sz, const char* classname)
+AllocObjectArray(int sz, const char* classname, Hjava_lang_ClassLoader* loader)
 {
 	Hjava_lang_Class *elclass;
 	errorInfo info;
@@ -897,7 +1004,7 @@ AllocObjectArray(int sz, const char* classname)
 	if (sz < 0) {
 		throwException(NegativeArraySizeException);
 	}
-        elclass = getClassFromSignature(classname, NULL, &info);
+        elclass = getClassFromSignature(classname, loader, &info);
 	if (elclass == 0) {
 		throwError(&info);
 	}
