@@ -45,12 +45,12 @@ static void (*runOnExit)(void);		/* run when all non-daemons die */
 static struct jthread* liveThreads;	/* list of all live threads */
 static sem_id threadLock;		/* static lock to protect liveThreads */
 
+static int map_Java_priority(int jprio);
 static void remove_thread(jthread_t tid);
 static void mark_thread_dead(void);
 void dumpLiveThreads(int);
 
 static void deathcallback(int sig);	/* runs when STOPped and stop is OK */
-static void undeathcallback(int sig);	/* runs when stop is not OK */
 
 /*
  * the following variables are set by jthread_init, and show how the
@@ -124,7 +124,14 @@ jthread_unsuspendall(void)
 void 
 jthread_spinon(void *arg)
 {
-	/* Unimplemented */
+	int32* lock = arg;
+	int32  prev;
+
+	if (NULL != lock) {
+		do {
+			prev = atomic_or(lock, 1);
+		} while (1 == prev);
+	}
 }
 
 /*
@@ -133,18 +140,17 @@ jthread_spinon(void *arg)
 void 
 jthread_spinoff(void *arg)
 {
-	/* Unimplemented */
+	int32* lock = arg;
+
+	if (NULL != lock) {
+		atomic_and(lock, 0);
+	}
 }
 
 /*============================================================================
  *
- * Functions related to interrupt handling
+ * Functions related to the stack
  *
- *
- * It has been pointed out that I could use _kget_thread_stacks_ to
- * get a more accurate snapshot of the thread stack.  I tried it,
- * and Kaffe/BeOS promptly failed a number of tests in the regression
- * suite.  Since the stuff below works, I'm going to let it Be for now.
  */
 
 /*
@@ -153,36 +159,12 @@ jthread_spinoff(void *arg)
 int
 jthread_extract_stack(jthread_t jtid, void **from, unsigned *len)
 {
-	thread_info tinfo;
-
-	if (B_OK != get_thread_info(jtid->native_thread, &tinfo)) {
-		/* Error */
-		return (0);
+	if (NULL == jtid) {
+		return(0);
 	}
-
-	*from = tinfo.stack_base;
-	*len = tinfo.stack_end - tinfo.stack_base;
-
-	if (*len < 0 || *len > (256*1024)) {
-	    printf("(%lx) get_thread_info(%lx) reported obscene numbers: "
-		  "base = 0x%p, end = 0x%p, sp = 0x%p\n", 
-		  find_thread(NULL),
-		  jtid->native_thread,
-		  tinfo.stack_base, tinfo.stack_end, tinfo.stack_base);
-	    exit(-1);
-	}
-
-DBG(JTHREADDETAIL,
-	dprintf("%s: extract_stack: base=%p size=%d sp=%p; from=%p len=%d\n", 
-		THREAD_NAME(jtid),
-		tinfo.stack_base,
-		tinfo.stack_end-tinfo.stack_base,
-		tinfo.stack_base,
-		*from,
-		*len);
-    )
-
-	return (1);
+	*from = jtid->stack_bottom;
+	*len = jtid->stack_top - jtid->stack_bottom;
+	return(1);
 }
 
 /*
@@ -191,27 +173,9 @@ DBG(JTHREADDETAIL,
 int
 jthread_on_current_stack(void *bp)
 {
-	void* curr_sp;
-	thread_info tinfo;
-	int rc;
-
-	if (B_OK != get_thread_info(find_thread(NULL), &tinfo)) {
-		return (0);
-	}
-
-	curr_sp = (void*)get_stack_frame();
-	rc = (bp >= curr_sp && bp < tinfo.stack_end);
-
-DBG(JTHREADDETAIL,
-	dprintf("%s: on current stack: base=%p size=%d bp=%p %s\n",
-		THREAD_NAME(GET_JTHREAD()),
-		tinfo.stack_base,
-		tinfo.stack_end-tinfo.stack_base,
-		bp,
-		(rc ? "yes" : "no"));
-    )
-
-	return rc;
+	int dummy = 0;
+	jthread_t currentJThread = GET_JTHREAD();
+	return(bp >= (void*)&dummy && bp < currentJThread->stack_top);
 }       
 
 /*
@@ -220,29 +184,14 @@ DBG(JTHREADDETAIL,
 int
 jthread_stackcheck(int need)
 {
-	void* curr_sp;
-	thread_info tinfo;
-	int room;
-
-	if (B_OK != get_thread_info(find_thread(NULL), &tinfo)) {
-		/* Error */
-		return (0);
+	int dummy = 0;
+	if (0 == need) {
+		return(1);
+	}
+	else {
+		jthread_t currentJThread = GET_JTHREAD();
+		return(((void*)&dummy - currentJThread->stack_bottom) >= need);
 	}
-
-	curr_sp = (void*)get_stack_frame();
-	room = curr_sp - tinfo.stack_base;
-
-DBG(JTHREADDETAIL,
-	dprintf("%s: stackcheck: need=%d base=%p size=%d sp=%p room=%d\n",
-		THREAD_NAME(GET_JTHREAD()),
-		need,
-		tinfo.stack_base,
-		tinfo.stack_end-tinfo.stack_base,
-		curr_sp,
-		room);
-    )
-
-	return (room >= need);
 }
 
 /*============================================================================
@@ -258,15 +207,13 @@ void
 jthread_destroy(jthread_t tid)
 {
 	status_t status;
-	void (*old_stop_handler)(int);
 
 	assert(tid);
-DBG(JTHREAD, 
-	dprintf("destroying %s\n", THREAD_NAME(tid));	
-    )
-	old_stop_handler = signal(STOP_SIGNAL, undeathcallback);
+	DBG(JTHREAD, dprintf("destroying %s\n", THREAD_NAME(tid));)
+
+	atomic_and(&tid->stop_allowed, 0);
 	wait_for_thread(tid->native_thread, &status);
-	signal(STOP_SIGNAL, old_stop_handler);
+	atomic_or(&tid->stop_allowed, 1);
 	deallocator(tid);
 }
 
@@ -274,6 +221,7 @@ DBG(JTHREAD,
  * find a native BeOS thread's cookie
  * (Maps BeOS threads to java.lang.Threads)
  */
+/*
 void* 
 jthread_getcookie(void* ntid)
 {
@@ -286,6 +234,7 @@ jthread_getcookie(void* ntid)
 	}
 	return (0);
 }
+*/
 
 /*
  * iterate over all live threads
@@ -317,13 +266,30 @@ jthread_frames(jthread_t thrd)
  *
  */
 
+static int
+map_Java_priority(int prio)
+{
+	double range = max_priority - min_priority;
+	double diff = prio - min_priority;
+
+	if (0 == range) {
+		return(B_NORMAL_PRIORITY);
+	}
+	else {
+		return(B_LOW_PRIORITY +
+			(double)(B_URGENT_DISPLAY_PRIORITY-B_LOW_PRIORITY) *
+				(diff / range));
+	}
+}
+
 /*
  * Initialize the threading system.
  */
 jthread_t 
-jthread_init(int pre,
-	int maxpr, int minpr, int mainthreadpr, 
-	size_t mainThreadStackSize,
+jthread_init(
+	int preemptive,
+	int maxpr,
+	int minpr,
 	void *(*_allocator)(size_t), 
 	void (*_deallocator)(void*),
 	void (*_destructor1)(void*),
@@ -333,6 +299,7 @@ jthread_init(int pre,
 	thread_id	pmain;
 	jthread_t	jtid;
 	void*		pti_addr;
+	thread_info	tinfo;
 
 	max_priority = maxpr;
 	min_priority = minpr;
@@ -351,6 +318,8 @@ jthread_init(int pre,
 	memset(pti_addr, PTI_AREA_SIZE, 0);
 	per_thread_info = (per_thread_info_t*)pti_addr;
 
+	threadLock = create_sem(1, "Kaffe threadLock");
+
 	pmain = find_thread(NULL);
 	rename_thread(pmain, "Kaffe main thread");
 
@@ -359,33 +328,61 @@ jthread_init(int pre,
 	 */
 	the_main_thread = pmain;
 
-        /*
-         * XXX: ignore mapping of min/max priority for now and assume
-         * BeOS priorities include java priorities.
-         */
-        set_thread_priority(pmain, mainthreadpr);
-
-	threadLock = create_sem(1, "Kaffe threadLock");
-
         jtid = allocator(sizeof (*jtid));
         SET_JTHREAD(jtid);
 
 	jtid->native_thread = pmain;
+
+	/*
+	 * Determine the top and bottom addrs of this thread's stack
+	 */
+	get_thread_info(pmain, &tinfo);
+	jtid->stack_top = tinfo.stack_end;
+	jtid->stack_bottom = tinfo.stack_base;
+
 	/*
 	 * In case of unnatural death (induced by receiving a STOP_SIGNAL),
 	 * we install the appropriate signal handler.
 	 */
 	signal(STOP_SIGNAL, deathcallback);
-	jtid->stop_pending = FALSE;
+	jtid->stop_allowed = 1;
+	jtid->stop_pending = 0;
 
         jtid->nextlive = liveThreads;
         liveThreads = jtid;
 	jtid->status = THREAD_RUNNING;
         talive++;
 
-DBG(JTHREAD,
-	dprintf("main thread has id %x\n", jtid->native_thread);
-    )
+	DBG(JTHREAD, dprintf("main thread has id %x\n", jtid->native_thread);)
+	return (jtid);
+}
+
+/*
+ * Create the first thread - actually bind the first thread to the java
+ * context.
+ */
+jthread_t
+jthread_createfirst(size_t mainThreadStackSize, 
+		    unsigned char prio, 
+		    void* jlThread)
+{
+        jthread_t jtid; 
+
+	jtid = GET_JTHREAD();
+	assert(jtid != NULL);
+	assert(jtid->native_thread != 0);
+	assert(jtid->status == THREAD_RUNNING);
+
+	jtid->jlThread = jlThread;
+	SET_COOKIE(jtid->jlThread);
+
+	signal(STOP_SIGNAL, deathcallback);
+	jtid->stop_allowed = 1;
+	jtid->stop_pending = 0;
+
+	jthread_setpriority(jtid, prio);
+	rename_thread(jtid->native_thread, "Kaffe main thread");
+
 	return (jtid);
 }
 
@@ -404,7 +401,11 @@ jthread_atexit(void (*f)(void))
 void 
 jthread_disable_stop(void)
 {
-	signal(STOP_SIGNAL, undeathcallback);
+	jthread_t currentJThread = GET_JTHREAD();
+
+	if (NULL != currentJThread) {
+		atomic_and(&currentJThread->stop_allowed, 0);
+	}
 }
 
 /*
@@ -415,9 +416,13 @@ jthread_enable_stop(void)
 {
 	jthread_t currentJThread = GET_JTHREAD();
 
-	signal(STOP_SIGNAL, deathcallback);
-	if (currentJThread->stop_pending) {
-		send_signal(currentJThread->native_thread, STOP_SIGNAL);
+	if (NULL != currentJThread) {
+		atomic_or(&currentJThread->stop_allowed, 1);
+		if (currentJThread->stop_pending) {
+			currentJThread->status = THREAD_DYING;
+			onstop();
+			jthread_exit();
+		}
 	}
 }
 
@@ -429,8 +434,8 @@ jthread_interrupt(jthread_t tid)
 {
 	thread_info tinfo;
 
-DBG(JTHREAD,
-        dprintf("interrupting thread %p, %s\n", tid, THREAD_NAME(tid)); )
+	DBG(JTHREAD, dprintf("interrupting thread %p, %s\n",
+		tid, THREAD_NAME(tid));)
 
 	if (THREAD_DYING != tid->status && THREAD_DEAD != tid->status) {
 		get_thread_info(tid->native_thread, &tinfo);
@@ -464,23 +469,21 @@ DBG(JTHREAD,
 static void
 deathcallback(int sig)
 {
-	jthread_t tid = GET_JTHREAD();
-	tid->status = THREAD_DYING;
+	jthread_t currentJThread = GET_JTHREAD();
+
+	atomic_or(&currentJThread->stop_pending, 1);
+	if (0 == atomic_or(&currentJThread->stop_allowed, 0)) {
+		DBG(JTHREAD, dprintf("%s rejects the STOP request",
+			THREAD_NAME(currentJThread));)
+		return;
+	}
+
+	currentJThread->status = THREAD_DYING;
 	onstop();
 	mark_thread_dead();
 	/* by returning, we proceed with the STOP and exit that thread */
 }
 
-/*
- * this handler catches STOP_SIGNAL when stops are disabled
- */
-static void
-undeathcallback(int sig)
-{
-	jthread_t tid = GET_JTHREAD();
-	tid->stop_pending = TRUE;
-}
-
 /*
  * start function for each thread.  
  * This function install the cleanup handler, sets jthread-specific 
@@ -489,12 +492,14 @@ undeathcallback(int sig)
 int32
 start_me_up(void *arg)
 {
-	jthread_t tid = (jthread_t)arg;
+	jthread_t   tid = (jthread_t)arg;
+	thread_info tinfo;
+
 	extern void initExceptions(void);
 
-DBG(JTHREAD, 
-	dprintf("start_me_up: setting up for %s\n", THREAD_NAME(tid)); 
-    )
+	DBG(JTHREAD, dprintf("start_me_up: setting up for %s\n",
+		THREAD_NAME(tid));)
+
 	acquire_sem(threadLock);
 
 	/* GROSS HACK ALERT -- On R4, child threads don't inherit the
@@ -507,19 +512,26 @@ DBG(JTHREAD,
 	 */
 
 	signal(STOP_SIGNAL, deathcallback);
-	tid->stop_pending = FALSE;
+	tid->stop_allowed = 1;
+	tid->stop_pending = 0;
+
+	/*
+	 * Determine the top and bottom addrs of this thread's stack
+	 */
+	get_thread_info(tid->native_thread, &tinfo);
+	tid->stack_top = tinfo.stack_end;
+	tid->stack_bottom = tinfo.stack_base;
+
 	SET_JTHREAD(tid);
 	SET_COOKIE(tid->jlThread);
         tid->status = THREAD_RUNNING;
 	release_sem(threadLock);
 
-DBG(JTHREAD, 
-	dprintf("start_me_up: calling t-func for %s\n", THREAD_NAME(tid)); )
+	DBG(JTHREAD, dprintf("start_me_up: calling t-func for %s\n",
+		THREAD_NAME(tid));)
 	tid->func(tid->jlThread);
-DBG(JTHREAD, 
-	dprintf("start_me_up: thread %s returned\n", 
-		THREAD_NAME(tid)); 
-    )
+	DBG(JTHREAD, dprintf("start_me_up: thread %s returned\n", 
+		THREAD_NAME(tid));)
 
 	assert (tid->status != THREAD_DYING);
 	mark_thread_dead();
@@ -544,8 +556,8 @@ jthread_create(unsigned int pri, void (*func)(void *), int daemon,
 	 * thread system to free its resources.
 	 */
 
-        tid = allocator(sizeof (*tid));
-        assert(tid != 0);      /* XXX */
+        tid = allocator(sizeof(*tid));
+        assert(tid != 0);
 
 	acquire_sem(threadLock);
         tid->jlThread = jlThread;
@@ -555,7 +567,8 @@ jthread_create(unsigned int pri, void (*func)(void *), int daemon,
         liveThreads = tid;
         tid->status = THREAD_NEWBORN;
 
-	ntid = spawn_thread(start_me_up, nameThread(jlThread), pri, tid);
+	ntid = spawn_thread(start_me_up, nameThread(jlThread),
+				map_Java_priority(pri), tid);
 	tid->native_thread = ntid;
 
         talive++;       
@@ -569,15 +582,15 @@ jthread_create(unsigned int pri, void (*func)(void *), int daemon,
 	 * but let's get this working first, shall we?
 	 */
 	if (NULL == per_thread_info[ntid % MAX_THREADS].jtid) {
-DBG(JTHREAD,
-	dprintf("created thread %s, daemon=%d\n", nameThread(jlThread), daemon); )
+		DBG(JTHREAD, dprintf("created thread %s, daemon=%d\n",
+			nameThread(jlThread), daemon);)
 
 		resume_thread(ntid);
         	return (tid);
 	}
 	else {
-DBG(JTHREAD,
-	dprintf("can't create thread: per-thread info table too small\n"); )
+		DBG(JTHREAD, dprintf("can't create thread: "
+			"per-thread info table too small\n");)
 
 		kill_thread(ntid);   /* stillborn */
 		deallocator(tid);
@@ -597,17 +610,13 @@ DBG(JTHREAD,
 void
 jthread_sleep(jlong time)
 {
-DBG(JTHREAD,
-        dprintf("thread %s: sleeping for %g second(s)...",
-                THREAD_NAME(GET_JTHREAD()), (double)time/1000.0);
-    )
+	DBG(JTHREAD, dprintf("thread %s: sleeping for %g second(s)...",
+                THREAD_NAME(GET_JTHREAD()), (double)time/1000.0);)
 
 	/* snooze_until takes an arg in usecs */
 	snooze_until(system_time() + (time*1000L), B_SYSTEM_TIMEBASE);
 
-DBG(JTHREAD,
-        dprintf("OK, I'm awake!\n");
-    )
+	DBG(JTHREAD, dprintf("OK, I'm awake!\n");)
 }
 
 /* 
@@ -629,7 +638,7 @@ jthread_alive(jthread_t tid)
 void
 jthread_setpriority(jthread_t jtid, int prio)
 {
-	set_thread_priority(jtid->native_thread, prio);
+	set_thread_priority(jtid->native_thread, map_Java_priority(prio));
 }
 
 /*
@@ -641,9 +650,7 @@ jthread_stop(jthread_t jtid)
 	/* can I cancel myself safely??? */
 	/* NB: jthread_stop should never be invoked on the current thread */
 
-DBG(JTHREAD,
-	dprintf("stopping %s...\n", THREAD_NAME(jtid));
-    )
+	DBG(JTHREAD, dprintf("stopping %s...\n", THREAD_NAME(jtid));)
 	send_signal(jtid->native_thread, STOP_SIGNAL);
 }
 
@@ -653,9 +660,8 @@ remove_thread(jthread_t tid)
 	jthread_t* ntid;
 	int found = 0;
 
-DBG(JTHREAD,
-	dprintf("Removing entry for thread %s\n", THREAD_NAME(tid));
-    )
+	DBG(JTHREAD, dprintf("Removing entry for thread %s\n",
+		THREAD_NAME(tid));)
 
 	acquire_sem(threadLock);
 
@@ -688,31 +694,25 @@ DBG(JTHREAD,
 		}
 		atomic_or(&isShuttingDown, 1);
 
-DBG(JTHREAD,
-		dprintf("%s: all done, closing shop\n", THREAD_NAME(tid));
-    )
+		DBG(JTHREAD, dprintf("%s: all done, closing shop\n",
+			THREAD_NAME(tid));)
+
 		if (runOnExit != 0) {
 		    runOnExit();
 		}
 
 		/* does that really make sense??? */
 		for (tid = liveThreads; tid != 0; tid = tid->nextlive) {
-			int32 tstat;
 			if (destructor1) {
 				(*destructor1)(tid->jlThread);
 			}
-			send_signal(tid->native_thread, STOP_SIGNAL);
-			wait_for_thread(tid->native_thread, &tstat);
-DBG(JTHREAD,
-			dprintf("waited for %s, which returned %d\n",
-				THREAD_NAME(tid), tstat);
-    )
+			kill_thread(tid->native_thread);
+			DBG(JTHREAD, dprintf("killed %s\n", THREAD_NAME(tid));)
 		}
 
 		/* Shut down this thread */
-DBG(JTHREAD,
-		dprintf("%s: Goodbye!\n", THREAD_NAME(GET_JTHREAD()));
-    )
+		DBG(JTHREAD, dprintf("%s: Goodbye!\n",
+			THREAD_NAME(GET_JTHREAD()));)
 		exit_thread(0);
 
 	} else {
@@ -747,27 +747,26 @@ jthread_exit(void)
 {
 	jthread_t currentJThread = GET_JTHREAD();
 
-DBG(JTHREAD,
-	dprintf("jthread_exit called by %s\n", THREAD_NAME(currentJThread));
-    )
+	DBG(JTHREAD, dprintf("jthread_exit called by %s\n",
+		THREAD_NAME(currentJThread));)
 
 	mark_thread_dead();
 
 	/*
-	 * If this is the main thread, wait for all daemons to finish.
+	 * If this is the main thread, wait for all other threads to finish.
 	 * At this point, the main jthread is no longer part of the
 	 * live list.
 	 */
 	if (the_main_thread == currentJThread->native_thread) {
 		while (talive > tdaemon) {
-		        jthread_t tid = NULL;
+	        jthread_t tid = NULL;
 			int32 rc;
 
-			jmutex_lock(&threadLock);
-		        for (tid = liveThreads; tid != NULL; tid = tid->nextlive) {
+			acquire_sem(threadLock);
+	        for (tid = liveThreads; tid != NULL; tid = tid->nextlive) {
 				if (tid->daemon) break;
 			}
-			jmutex_unlock(&threadLock);
+			release_sem(threadLock);
 			if (NULL != tid) {
 				wait_for_thread(tid->native_thread, &rc);
 			}
@@ -777,6 +776,9 @@ DBG(JTHREAD,
 	/*
 	 * OK, now it's safe to exit
 	 */
+	DBG(JTHREAD, dprintf("%s: at the point of no return in jthread_exit\n",
+		THREAD_NAME(currentJThread));)
+
 	exit_thread(0);
 	while (1)
 		assert(!"This better not return.");
@@ -788,7 +790,7 @@ DBG(JTHREAD,
 void 
 jthread_dumpthreadinfo(jthread_t tid)
 {
-	fprintf(stderr, "jthread %p native %ld status %s\n", 
+	dprintf("jthread %p native %ld status %s\n", 
 		tid, tid->native_thread,
 		tid->status == THREAD_NEWBORN ? "NEWBORN" :
 		tid->status == THREAD_RUNNING ? "RUNNING" :
@@ -818,105 +820,3 @@ void jthread_exit_when_done(void)
 		jthread_yield();
 	jthread_exit();
 }
-
-/*============================================================================
- * 
- * locking subsystem
- *
- */
-void 
-jmutex_initialise(jmutex *lock)
-{
-	*lock = create_sem(1, "Kaffe sem");
-	assert(*lock >= B_NO_ERROR);
-}
-
-void
-jmutex_lock(jmutex *lock)
-{
-	status_t rc;
-
-	rc = acquire_sem(*lock);
-	assert(B_NO_ERROR == rc);
-}
-
-void
-jmutex_unlock(jmutex *lock)
-{
-	status_t rc;
-
-	rc = release_sem(*lock);
-	assert(B_NO_ERROR == rc);
-}
-
-void
-jcondvar_initialise(jcondvar *cv)
-{
-	cv->mutex = create_sem(1, "Kaffe cv - mutex");
-	cv->cond = create_sem(0, "Kaffe cv - cond");
-	cv->num_sleeping = 0;
-	assert(cv->mutex >= B_NO_ERROR && cv->cond >= B_NO_ERROR);
-}
-
-void
-jcondvar_wait(jcondvar *cv, jmutex *lock, jlong timeout)
-{
-	status_t rc;
-
-	if (0 == timeout) {
-		rc = acquire_sem(cv->mutex);
-		if (B_OK != rc) return;
-
-		atomic_add(&cv->num_sleeping, 1);
-		jmutex_unlock(lock);
-		release_sem(cv->mutex);
-
-		rc = acquire_sem(cv->cond);
-		if (B_OK != rc) {
-			atomic_add(&cv->num_sleeping, -1);
-		}
-		jmutex_lock(lock);
-	}
-	else   /* non-zero timeout */
-	{
-		bigtime_t timeoutUSecs = timeout * 1000L;
-		bigtime_t startTime = system_time();
-
-		rc = acquire_sem_etc(cv->mutex, 1, B_TIMEOUT, timeoutUSecs);
-		if (B_OK != rc) return;
-
-		atomic_add(&cv->num_sleeping, 1);
-		jmutex_unlock(lock);
-		release_sem(cv->mutex);
-
-		timeoutUSecs -= (system_time() - startTime);
-		rc = acquire_sem_etc(cv->cond, 1, B_TIMEOUT, timeoutUSecs);
-		if (B_OK != rc) {
-			atomic_add(&cv->num_sleeping, -1);
-		}
-		jmutex_lock(lock);
-	}
-}
-
-void
-jcondvar_signal(jcondvar *cv, jmutex *lock)
-{
-	acquire_sem(cv->mutex);
-	if (cv->num_sleeping > 0) {
-		release_sem(cv->cond);
-		atomic_add(&cv->num_sleeping, -1);
-	}
-	release_sem(cv->mutex);
-}
-
-void
-jcondvar_broadcast(jcondvar *cv, jmutex *lock)
-{
-	acquire_sem(cv->mutex);
-	while (cv->num_sleeping > 0) {
-		release_sem(cv->cond);
-		atomic_add(&cv->num_sleeping, -1);
-	}
-	release_sem(cv->mutex);
-}
-
