@@ -16,6 +16,7 @@
 
 #include "jthread.h"
 #include "jsignal.h"
+#include "xprofiler.h"
 
 
 /* Flags used for threading I/O calls */
@@ -122,12 +123,32 @@ static int  min_priority;		/* minimum supported priority */
 
 jthread* currentJThread;
 
+/* The arguments to a signal handler */
+#ifndef SIGNAL_ARGS
+#define SIGNAL_ARGS(sig, sc) int sig
+#endif
+
+/* Get a signal context pointer from signal arguments */
+#ifndef GET_SIGNAL_CONTEXT_POINTER
+#define GET_SIGNAL_CONTEXT_POINTER(x) 0
+#endif
+
+/* A signal context pointer type, used in parameter lists/declarations */
+#ifndef SIGNAL_CONTEXT_POINTER
+#define SIGNAL_CONTEXT_POINTER(x) void *##x
+#endif
+
+/* Get the PC from a signal context pointer */
+#ifndef SIGNAL_PC
+#define SIGNAL_PC(scp) 0
+#endif
+
 /*
  * Function declarations.
  * Again, keep these static to ensure encapsulation.
  */
-static void handleInterrupt(int sig);
-static void interrupt(int sig);
+static void handleInterrupt(int sig, SIGNAL_CONTEXT_POINTER(sc));
+static void interrupt(SIGNAL_ARGS(sig, sc));
 static void childDeath(void);
 static void handleIO(int);
 static void killThread(jthread *jtid);
@@ -149,6 +170,11 @@ static void intsRestore(void);
 #define GET_FP(E)       (((void**)(E))[FP_OFFSET])
 #define SET_FP(E, V)    ((void**)(E))[FP_OFFSET] = (V)
 
+/* Set the base pointer in a jmp_buf if we can (only a convenience) */
+#if defined(BP_OFFSET)
+#define SET_BP(E, V)    ((void**)(E))[BP_OFFSET] = (V)
+#endif
+
 /* amount of stack space to be duplicated at stack creation time */
 #if !defined(STACK_COPY)
 #define STACK_COPY      (32*4)
@@ -193,40 +219,6 @@ isOnList(jthread *list, jthread *t)
 	return (0);
 }
 
-/*
- * Transform list of threads in linear array
- */
-static int
-makeListFromQueue(jthread *first, jthread_t **threads)
-{
-	int n = 0;
-	jthread *t;
-	jthread_t *list;
-
-	/* count them */
-	for (t = first; t; t = t->nextQ) {
-		n++;
-	}
-
-	/* list is empty */
-	if (n == 0) {
-		*threads = 0;
-		return (0);
-	}
-
-	/* allocate an array */
-	/* XXX: We use the real malloc here since we don't want to get
-	 * stuck in a gc */
-	list = malloc(n * sizeof(jthread_t *));
-	assert(list != 0);
-
-	/* and copy them in */
-	for (n = 0, t = first; t; t = t->nextQ) {
-		list[n++] = t;
-	}
-	*threads = list;
-	return (n);
-}
 
 /*
  * yield to another thread
@@ -326,7 +318,7 @@ processSignals(void)
 	for (i = 1; i < NSIG; i++) {
 		if (pendingSig[i]) {
 			pendingSig[i] = 0;
-			handleInterrupt(i);
+			handleInterrupt(i, 0);
 		}
 	}
 	sigPending = 0;
@@ -391,7 +383,7 @@ jthread_unsuspendall(void)
  * be running with all signals blocked.
  */
 static void
-interrupt(int sig)
+interrupt(SIGNAL_ARGS(sig, sc))
 {
 	/*
 	 * If ints are blocked, this might indicate an inconsistent state of
@@ -409,6 +401,20 @@ interrupt(int sig)
 		char c;
 		pendingSig[sig] = 1;
 		sigPending = 1;
+		
+#if defined(KAFFE_XPROFILER)
+		/*
+		 * Since the regular handler won't run with the sig context we
+		 * need to do the hit here
+		 */
+		if( sig == SIGVTALRM )
+		{
+			SIGNAL_CONTEXT_POINTER(scp) =
+				GET_SIGNAL_CONTEXT_POINTER(sc);
+				
+			profileHit((char *)SIGNAL_PC(scp));
+		}
+#endif
 		/*
 		 * There is a race condition in handleIO() between
 		 * zeroing blockints and going into select().
@@ -477,7 +483,7 @@ interrupt(int sig)
 	/*
 	 * Handle the signal.
 	 */
-	handleInterrupt(sig);
+	handleInterrupt(sig, GET_SIGNAL_CONTEXT_POINTER(sc));
 
 	/*
 	 * Leave the critical section.  This may or may not cause a
@@ -495,10 +501,14 @@ interrupt(int sig)
  * priority.
  */
 static void 
-handleVtAlarm(void)
+handleVtAlarm(int sig, SIGNAL_CONTEXT_POINTER(sc))
 {
 	static int c;
 
+#if defined(KAFFE_XPROFILER)
+	if( sc )
+		profileHit((char *)SIGNAL_PC(sc));
+#endif
 	if (preemptive) {
 		internalYield();
 	}
@@ -533,6 +543,7 @@ alarmException(void)
 		 */
 		jtid = alarmList;
 		alarmList = alarmList->nextalarm;
+		jtid->flags |= THREAD_FLAGS_INTERRUPTED;
 		resumeThread(jtid);
 	}
 
@@ -582,7 +593,7 @@ printflags(unsigned i)
 void
 jthread_dumpthreadinfo(jthread_t tid)
 {
-	fprintf(stderr, "tid %p, status %s flags %s\n", tid, 
+	dprintf("tid %p, status %s flags %s\n", tid, 
 		tid->status == THREAD_SUSPENDED ? "SUSPENDED" :
 		tid->status == THREAD_RUNNING ? "RUNNING" :
 		tid->status == THREAD_DEAD ? "DEAD" : "UNKNOWN!!!", 
@@ -591,31 +602,36 @@ jthread_dumpthreadinfo(jthread_t tid)
 		jthread *t;
 		int i;
 
-		fprintf(stderr, " blocked");
+		dprintf(" blocked");
 		if (isOnList(waitForList, tid)) {
-			fprintf(stderr, ": waiting for children");
+			dprintf(": waiting for children");
 		}
+#if 0
+		/* XXX FIXME: alarmList uses nextalarm, but isOnList iterates
+		 * using nextQ
+		 */
 		if (isOnList(alarmList, tid)) {
-			fprintf(stderr, ": sleeping");
+			dprintf(": sleeping");
 		}
+#endif
 		for (i = 0; i < FD_SETSIZE; i++) {
 			if (isOnList(readQ[i], tid)) {
-				fprintf(stderr, ": reading from fd %d ", i);
+				dprintf(": reading from fd %d ", i);
 				break;
 			}
 			if (isOnList(writeQ[i], tid)) {
-				fprintf(stderr, ": writing to fd %d ", i);
+				dprintf(": writing to fd %d ", i);
 				break;
 			}
 		}
 
-		fprintf(stderr, "@%p (%p->", tid->blockqueue,
+		dprintf("@%p (%p->", tid->blockqueue,
 					     t = *tid->blockqueue);
 		while (t && t->nextQ) {
 			t = t->nextQ; 
-			fprintf(stderr, "%p->", t);
+			dprintf("%p->", t);
 		}
-		fprintf(stderr, "|) ");
+		dprintf("|) ");
 	}
 }
 
@@ -626,7 +642,7 @@ jthread_dumpthreadinfo(jthread_t tid)
  * result of intsRestore.
  */
 static void 
-handleInterrupt(int sig)
+handleInterrupt(int sig, SIGNAL_CONTEXT_POINTER(sc))
 {
 	switch(sig) {
 	case SIGALRM:
@@ -639,7 +655,7 @@ handleInterrupt(int sig)
 
 #if defined(SIGVTALRM)
 	case SIGVTALRM:
-		handleVtAlarm();
+		handleVtAlarm(sig, sc);
 		break;
 #endif
 
@@ -652,7 +668,7 @@ handleInterrupt(int sig)
 		break;
 
 	default:
-		printf("unknown signal %d\n", sig);
+		dprintf("unknown signal %d\n", sig);
 		exit(-1);
 	}
 }
@@ -674,7 +690,7 @@ resumeThread(jthread* jtid)
 {
 	jthread** ntid;
 
-DBG(JTHREAD,	dprintf("resumeThread %x\n", jtid);	)
+DBG(JTHREAD,	dprintf("resumeThread %p\n", jtid);	)
 	intsDisable();
 
 	if (jtid->status != THREAD_RUNNING) {
@@ -715,7 +731,7 @@ DBG(JTHREAD,	dprintf("resumeThread %x\n", jtid);	)
 		}
 		jtid->nextQ = 0;
 	} else {
-DBG(JTHREAD,		dprintf("Re-resuming 0x%x\n", jtid); )
+DBG(JTHREAD,		dprintf("Re-resuming %p\n", jtid); )
 	}
 	intsRestore();
 }
@@ -724,7 +740,7 @@ DBG(JTHREAD,		dprintf("Re-resuming 0x%x\n", jtid); )
  * Suspend a thread on a queue.
  * Return true if thread was interrupted.
  */
-int
+static int
 suspendOnQThread(jthread* jtid, jthread** queue, jlong timeout)
 {
 	int rc = false;
@@ -777,7 +793,7 @@ DBG(JTHREAD,	dprintf("suspendOnQThread %p %p (%qd) bI %d\n",
 		}
 	} else {
 	DBG(JTHREAD,	
-		dprintf("Re-suspending 0x%x on %x\n", jtid, *queue); )
+		dprintf("Re-suspending %p on %p\n", jtid, *queue); )
 	}
 	return (rc);
 }
@@ -800,7 +816,7 @@ killThread(jthread *tid)
 		(*destructor1)(tid->jlThread);
 
 DBG(JTHREAD,	
-	dprintf("killThread %x kills %x\n", currentJThread, tid); )
+	dprintf("killThread %p kills %p\n", currentJThread, tid); )
 
 	if (tid->status != THREAD_DEAD) {
 
@@ -829,6 +845,18 @@ DBG(JTHREAD,
 			blockInts = 1;
 		}
 
+		/* Now that we are off the runQ, it is safe to leave
+		 * the list of live threads and be GCed.
+		 */
+		/* Remove thread from live list so it can be garbaged */
+		for (ntid = &liveThreads; *ntid != 0; ntid =
+			     &(*ntid)->nextlive) { 
+			if (tid == (*ntid)) {
+				(*ntid) = tid->nextlive;
+				break;
+			}
+		}
+
 		/* Dead Jim - let the GC pick up the remains */
 		tid->status = THREAD_DEAD;
 	}
@@ -874,6 +902,12 @@ DBG(JTHREAD,
 void    
 jthread_destroy(jthread *jtid)
 {
+	jthread *x;
+
+	if (DBGEXPR(JTHREAD, true, false)) {
+		for (x = liveThreads; x; x = x->nextlive)
+			assert(x != jtid);
+	}
 	deallocator(jtid);
 }
 
@@ -896,7 +930,7 @@ jthread_walkLiveThreads(void (*func)(void *jlThread))
 int
 jthread_extract_stack(jthread *jtid, void **from, unsigned *len)
 {
-	assert(jtid);
+	assert(jtid != NULL);
 #if defined(STACK_GROWS_UP)
 	*from = jtid->stackBase;
 	*len = jtid->restorePoint - jtid->stackBase;
@@ -954,10 +988,9 @@ static void deactivate_time_slicing(void) { }
 /*
  * Initialize the threading system. 
  */
-jthread * 
+void
 jthread_init(int pre,
-	int maxpr, int minpr, int mainthreadpr, 
-	size_t mainThreadStackSize,
+	int maxpr, int minpr,
 	void *(*_allocator)(size_t), 
 	void (*_deallocator)(void*),
 	void (*_destructor1)(void*),
@@ -997,13 +1030,15 @@ jthread_init(int pre,
 	 * mode.  So by default, when debugging, we want stdio be synchronous.
 	 * To override this, give the ASYNCSTDIO flag.
 	 */
+#ifndef _HURD_ASYNC_QUICKFIX_
 	if (DBGEXPR(ANY, DBGEXPR(ASYNCSTDIO, true, false), true)) {
 		for (i = 0; i < 3; i++) {
 			if (i != jthreadedFileDescriptor(i)) {
-				return (0);
+				return;
 			}
 		}
 	}
+#endif
 
 	/*
 	 * On some systems, it is essential that we put the fds back
@@ -1026,7 +1061,7 @@ jthread_init(int pre,
 
 	/* create the helper pipe for lost wakeup problem */
 	if (pipe(sigPipe) != 0) {
-		return (0);
+		return;
 	}
 	if (maxFd == -1) {
 		maxFd = sigPipe[0] > sigPipe[1] ? sigPipe[0] : sigPipe[1];
@@ -1034,10 +1069,20 @@ jthread_init(int pre,
 
 	jtid = newThreadCtx(0);
 	if (!jtid) {
-		return (0);
+		return;
 	}
 
-        jtid->priority = mainthreadpr;
+	/* Fake up some stack bounds until we get the real ones later - we're
+	 * the only thread running so this isn't a problem.
+	 */
+	jtid->stackBase = 0;
+	jtid->stackEnd = (char*)0 - 1;
+#if defined(STACK_GROWS_UP)
+        jtid->restorePoint = jtid->stackEnd;
+#else
+        jtid->restorePoint = jtid->stackBase;
+#endif
+        jtid->priority = maxpr;
         jtid->status = THREAD_SUSPENDED;
         jtid->flags = THREAD_FLAGS_NOSTACKALLOC;
         jtid->func = (void (*)(void*))jthread_init;
@@ -1045,6 +1090,31 @@ jthread_init(int pre,
         jtid->time = 0;
         liveThreads = jtid;
 
+        talive++;
+        currentJThread = jtid;
+        resumeThread(jtid);
+#if defined(KAFFE_XPROFILER)
+	/*
+	 * The profiler is started at program startup, we take over from here
+	 * on out so we disable whatever one was installed
+	 */
+	disableProfileTimer();
+#endif
+	/* Because of the handleVtAlarm hack (poll every 20 SIGVTALRMs) 
+	 * we turn on the delivery of SIGVTALRM even if no actual time
+	 * slicing is possible because only one Java thread is running.
+	 * XXX We should be smarter about that.
+	 */
+	activate_time_slicing();
+}
+
+jthread_t
+jthread_createfirst(size_t mainThreadStackSize, unsigned char prio, void* jlThread)
+{
+        jthread *jtid; 
+
+	jtid = currentJThread;
+
 	/*
 	 * Note: the stackBase and stackEnd values are used for two purposes:
 	 * - to report to the gc what area to scan (extract_stack)
@@ -1052,24 +1122,20 @@ jthread_init(int pre,
 	 *   of frames is valid.  This is done by checking its range.
 	 */
 #if defined(STACK_GROWS_UP)
-	jtid->stackBase = ((void *)&jtid);
+	jtid->stackBase = (void*)((uintp)&jtid & -0x1000);
 	jtid->stackEnd = jtid->stackBase + mainThreadStackSize;
         jtid->restorePoint = jtid->stackEnd;
 #else
-        jtid->stackEnd = ((void *)&jtid);
+	jtid->stackEnd = (void*)(((uintp)&jtid + 0x1000 - 1) & -0x1000);
         jtid->stackBase = jtid->stackEnd - mainThreadStackSize;
         jtid->restorePoint = jtid->stackBase;
 #endif
-        talive++;
-        currentJThread = jtid;
-        resumeThread(jtid);
-	/* Because of the handleVtAlarm hack (poll every 20 SIGVTALRMs) 
-	 * we turn on the delivery of SIGVTALRM even if no actual time
-	 * slicing is possible because only one Java thread is running.
-	 * XXX We should be smarter about that.
-	 */
-	activate_time_slicing();
-        return jtid;
+
+	jtid->jlThread = jlThread;
+
+	jthread_setpriority(jtid, prio);
+
+	return (jtid);
 }
 
 /*
@@ -1087,9 +1153,17 @@ jthread_atexit(void (*f)(void))
 void 
 jthread_disable_stop(void)
 {
-	intsDisable();
-	currentJThread->flags |= THREAD_FLAGS_DONTSTOP;
-	intsRestore();
+	if (currentJThread) {
+		intsDisable();
+		currentJThread->flags |= THREAD_FLAGS_DONTSTOP;
+		currentJThread->stopCounter++;
+		assert(currentJThread->stopCounter > 0);
+
+		/* XXX Shouldn't recurse that much... ever... hopefully. */
+		assert(currentJThread->stopCounter < 10);  
+
+		intsRestore();
+	}
 }
 
 /*
@@ -1098,14 +1172,20 @@ jthread_disable_stop(void)
 void 
 jthread_enable_stop(void)
 {
-	intsDisable();
-	currentJThread->flags &= ~THREAD_FLAGS_DONTSTOP;
-	if ((currentJThread->flags & THREAD_FLAGS_KILLED) != 0 && 
-		blockInts == 1) 
-	{
-		die();
+	if (currentJThread) {
+		intsDisable();
+		if (--currentJThread->stopCounter == 0) {
+			currentJThread->flags &= ~THREAD_FLAGS_DONTSTOP;
+			if ((currentJThread->flags & THREAD_FLAGS_KILLED) != 0
+			   && ((currentJThread->flags & THREAD_FLAGS_EXITING) 
+			   	== 0))
+			{
+				die();
+			}
+		}
+		assert(currentJThread->stopCounter >= 0);
+		intsRestore();
 	}
-	intsRestore();
 }
 
 /*
@@ -1141,13 +1221,16 @@ die(void)
 static void
 start_this_sucker_on_a_new_frame(void)
 {
+	/* all threads start with interrupts turned off */
+	blockInts = 1;
+
 	/* I might be dying already */
 	if ((currentJThread->flags & THREAD_FLAGS_KILLED) != 0) {
 		die();
 	}
 
-	/* all threads start with interrupts turned off */
 	intsRestore();
+	assert(currentJThread->stopCounter == 0);
 	currentJThread->func(currentJThread->jlThread);
 	jthread_exit(); 
 }
@@ -1163,9 +1246,20 @@ jthread_create(unsigned char pri, void (*func)(void *), int daemon,
 	jthread *jtid; 
 	void	*oldstack, *newstack;
 
+	/*
+	 * Disable stop to protect the threadLock lock, and prevent
+	 * the system from losing a new thread context (before the new
+	 * thread is queued up).
+	 */
+	jthread_disable_stop();
+
 	jmutex_lock(&threadLock);
         jtid = newThreadCtx(threadStackSize);
-        assert(jtid != 0);      /* XXX */
+	if (!jtid) {
+		jmutex_unlock(&threadLock);
+		jthread_enable_stop();
+		return 0;
+	}
         jtid->priority = pri;
         jtid->jlThread = jlThread;
         jtid->status = THREAD_SUSPENDED;
@@ -1216,14 +1310,29 @@ DBG(JTHREAD,
 	memcpy(newstack, oldstack, STACK_COPY);
 #endif /* !STACK_GROWS_UP */
 
+#if defined(NEED_STACK_ALIGN)
+        newstack = (void *) STACK_ALIGN(newstack);
+#endif
+
 	SET_SP(jtid->env, newstack);
 
+#if defined(SET_BP)
+	/*
+	 * Clear the base pointer in the new thread's stack.
+	 * Nice for debugging, but not strictly necessary.
+	 */
+	SET_BP(jtid->env, 0);
+#endif
+
+
 #if defined(FP_OFFSET)
 	/* needed for: IRIX */
 	SET_FP(jtid->env, newstack + ((void *)GET_FP(jtid->env) - oldstack));
 #endif
 
         resumeThread(jtid);
+	jthread_enable_stop();
+
         return jtid;
 }
 
@@ -1269,9 +1378,23 @@ jthread_alive(jthread *jtid)
 {
         int status = true;
         intsDisable();
-        if (jtid == 0 || 
-		(jtid->flags & (THREAD_FLAGS_KILLED | THREAD_FLAGS_DYING)) != 0 
-		|| jtid->status == THREAD_DEAD)
+        if (jtid == 0
+#if 0
+	    /* this code makes kaffe behave like sun, but it means
+	     * that Thread.join() after Thread.stop() is useless.
+	     */
+	    || (jtid->flags & (THREAD_FLAGS_KILLED | THREAD_FLAGS_DYING)) != 0 
+#else
+	    /* There seems to be a window in which death can be
+	     * broadcast before it is waited for.  Basically,
+	     * jthread_alive will be false immediately after
+	     * Thread.stop(), unless stopping the thread was
+	     * disabled.  Thread.alive() will become false as soon as
+	     * the thread is on its way out.
+	     */
+	    || (jtid->flags & (THREAD_FLAGS_DYING | THREAD_FLAGS_EXITING))
+#endif
+	    || jtid->status == THREAD_DEAD)
                 status = false;
         intsRestore();
         return status;
@@ -1353,18 +1476,17 @@ jthread_stop(jthread *jtid)
 }
 
 /*
- * Have a thread exit.
+ * Have a thread exit.  This function does not return.
  */
 void
 jthread_exit(void)
 {
-	jthread** ntid;
 	jthread* tid;
-	int found = 0;
 
 DBG(JTHREAD,
-	dprintf("jthread_exit %x\n", currentJThread);		)
+	dprintf("jthread_exit %p\n", currentJThread);		)
 
+	jthread_disable_stop();
 	jmutex_lock(&threadLock);
 
 	talive--;
@@ -1372,18 +1494,12 @@ DBG(JTHREAD,
 		tdaemon--;
 	}
 
-	/* Remove thread from live list so it can be garbaged */
-	for (ntid = &liveThreads; *ntid != 0; ntid = &(*ntid)->nextlive) {
-		if (currentJThread == (*ntid)) {
-			(*ntid) = currentJThread->nextlive;
-			found = 1;
-			break;
-		}
-	}
+	assert(!(currentJThread->flags & THREAD_FLAGS_EXITING));
 	currentJThread->flags |= THREAD_FLAGS_EXITING;
-	assert(found || !!!"Attempt to exit a thread twice");
 
 	jmutex_unlock(&threadLock);
+	jthread_enable_stop();
+
 	/* we disable interrupts while we go out to prevent a reschedule
 	 * in killThread()
 	 */
@@ -1398,7 +1514,12 @@ DBG(JTHREAD,
 		}
 
 		for (tid = liveThreads; tid != 0; tid = tid->nextlive) {
-		    killThread(tid);
+			/* The current thread is still on the live
+			 * list, and we don't want to recursively
+			 * suicide.
+			 */
+			if (!(tid->flags & THREAD_FLAGS_EXITING))
+				killThread(tid);
 		}
 		EXIT(0);
 	}
@@ -1496,6 +1617,7 @@ dprintf("switch from %p to %p\n", lastThread, currentJThread); )
 		 */
 		wouldlosewakeup = 1;
 		if (sigPending) {
+			wouldlosewakeup = 0;
 			processSignals();
 			continue;
 		}
@@ -1611,12 +1733,9 @@ retry:
 	r = poll(pollarray, nfd, sleep ? -1 : 0);
 #else
 	r = select(maxFd+1, &rd, &wr, 0, sleep ? 0 : &zero);
-
-	/* Reset wouldlosewakeup here */
-	if (wouldlosewakeup) {
-		wouldlosewakeup = 0; 
-	}
 #endif
+	/* Reset wouldlosewakeup here */
+	wouldlosewakeup = 0; 
 
 	if (sleep) {
 		int can_read_from_pipe = 0;
@@ -1776,22 +1895,25 @@ jmutex_unlock(jmutex *lock)
 	intsRestore();
 }
 
-int
-jmutex_blocked(jmutex *lock, jthread_t **threads)
+void
+jmutex_destroy(jmutex *lock)
 {
-	return (makeListFromQueue(lock->waiting, threads));
+	assert(lock->holder == NULL);
+	assert(lock->waiting == NULL);
 }
 
 void
 jcondvar_initialise(jcondvar *cv)
 {
-	cv = NULL;
+	*cv = NULL;
 }
 
-void
+jbool
 jcondvar_wait(jcondvar *cv, jmutex *lock, jlong timeout)
 {
 	jthread *current = jthread_current();
+	jbool r;
+
 	intsDisable();
 
 	/* give up mutex */
@@ -1813,13 +1935,15 @@ jcondvar_wait(jcondvar *cv, jmutex *lock, jlong timeout)
 #endif
 
 	/* wait to be signaled */
-	suspendOnQThread(current, cv, timeout);
+	r = suspendOnQThread(current, cv, timeout);
 	/* reacquire mutex */
 	while (lock->holder != NULL) {
 		suspendOnQThread(current, &lock->waiting, NOTIMEOUT);
 	}
 	lock->holder = current;
 	intsRestore();
+
+	return (r);
 }
 
 void
@@ -1856,12 +1980,13 @@ jcondvar_broadcast(jcondvar *cv, jmutex *lock)
 	intsRestore();
 }
 
-int  
-jcondvar_waiting(jcondvar *cv, jthread_t **threads)
+void
+jcondvar_destroy(jcondvar* cv)
 {
-	return (makeListFromQueue(*cv, threads));
+	assert(*cv == NULL);
 }
 
+
 /*============================================================================
  * 
  * I/O routines that are exported to the user
@@ -1877,7 +2002,11 @@ jcondvar_waiting(jcondvar *cv, jthread_t **threads)
 static int
 jthreadedFileDescriptor(int fd)
 {
-	int r, on = 1;
+	int r;
+#if (defined(FIOSSAIOSTAT) && !(defined(hpux) && defined(FIOASYNC))) || \
+    (defined(FIOASYNC) && !defined(linux))
+	int on = 1;
+#endif
 #if (defined(FIOSSAIOOWN) && !(defined(hpux) && defined(F_SETOWN))) || \
     defined(F_SETOWN)
 	/* cache pid to accommodate antique C libraries */
@@ -2085,7 +2214,7 @@ jthreadedOpen(const char* path, int flags, int mode, int *out)
  * Threaded socket connect.
  */
 int
-jthreadedConnect(int fd, struct sockaddr* addr, size_t len, int timeout)
+jthreadedConnect(int fd, struct sockaddr* addr, int len, int timeout)
 {
 	int r;
 	jlong deadline = 0;
@@ -2135,8 +2264,8 @@ jthreadedConnect(int fd, struct sockaddr* addr, size_t len, int timeout)
  * Threaded socket accept.
  */
 int
-jthreadedAccept(int fd, struct sockaddr* addr, size_t* len, 
-		int timeout, ssize_t* out)
+jthreadedAccept(int fd, struct sockaddr* addr, int* len, 
+		int timeout, int* out)
 {
 	/* absolute time at which time out is reached */
 	jlong deadline = 0;
@@ -2294,24 +2423,37 @@ jthreadedWaitpid(int wpid, int* status, int options, int *outpid)
 {
 #if defined(HAVE_WAITPID)
 	int npid;
+	int ret = 0;
 
 DBG(JTHREAD,
 	dprintf("waitpid %d current=%p\n", wpid, currentJThread); )
 
 	intsDisable();
 	for (;;) {
+	        int procStatus;
+
 		wouldlosewakeup = 1;
-		npid = waitpid(wpid, status, options|WNOHANG);
-		/* XXX what return codes should cause us to return an error? */
+		npid = waitpid(wpid, &procStatus, options|WNOHANG);
 		if (npid > 0) {
 			*outpid = npid;
+			if (WIFEXITED(procStatus))
+			  *status = WEXITSTATUS(procStatus);
+			else if (WIFSIGNALED(procStatus))
+			  *status=128+WTERMSIG(procStatus);
+			else
+			  *status = -1;		/* shouldn't happen */
+			break;
+		}
+		if ((npid == -1) && (errno == ECHILD)) {
+			/* child does not exist */
+			ret = -1;
 			break;
 		}
 		BLOCKED_ON_EXTERNAL(currentJThread);
 		suspendOnQThread(currentJThread, &waitForList, NOTIMEOUT);
 	}
 	intsRestore();
-	return (0);
+	return (ret);
 #else
 	return (EOPNOTSUPPORT);
 #endif
@@ -2327,7 +2469,8 @@ close_fds(int fd[], int n)
 }
 
 int 
-jthreadedForkExec(char **argv, char **arge, int ioes[4], int *outpid)
+jthreadedForkExec(char **argv, char **arge,
+	int ioes[4], int *outpid, const char *dir)
 {
 /* these defines are indices in ioes */
 #define IN_IN		0
@@ -2418,6 +2561,11 @@ DBG(JTHREAD,
 		/* now close all pipe fds */
 		close_fds(fds, 8);
 
+		/* change working directory */
+#if defined(HAVE_CHDIR)
+		(void)chdir(dir);
+#endif
+ 
 		/*
 		 * If no environment was given and we have execvp, we use it.
 		 * If an environment was given, we use execve.
@@ -2435,9 +2583,11 @@ DBG(JTHREAD,
 
 	case -1:
 		/* Error */
+		err = errno;
 		/* Close all pipe fds */
 		close_fds(fds, 8);
-		return (errno);		/* XXX unprotected! */
+		sigprocmask(SIG_UNBLOCK, &nsig, 0);
+		return (err);
 
 	default:
 		/* Parent */
