@@ -19,7 +19,7 @@
  * auxiliary functions
  */
 
-__inline__ int
+static __inline__ int
 xclrEquals ( XColor* pc, int r, int g, int b )
 {
   return ((pc->red == r) && (pc->green == g) && (pc->blue == b));
@@ -30,7 +30,7 @@ xclrEquals ( XColor* pc, int r, int g, int b )
  * this is a rather crude color difference - a HSV difference might be
  * more ergonomical, but also much more expensive
  */
-__inline__ double
+static __inline__ double
 colorDiff ( int r1, int g1, int b1, int r2, int g2, int b2 )
 {
   int dr,dg, db;
@@ -92,7 +92,7 @@ unsigned char rgb8 [] = { 0, 36, 73, 109, 146, 182, 219, 255 };
  * its result to set cells in m->pix while other empty cells are still waiting to
  * be served.
  */
-unsigned char
+static unsigned char
 nearestColor ( Rgb2Pseudo* m, int pi, int pj, int pk, int dist )
 {
   int i, j, k, i0, i1, j0, j1, k0, k1;
@@ -140,7 +140,7 @@ typedef struct _Mismatch {
 } Mismatch;
 
 
-void
+static void
 fillUpColorCube ( Rgb2Pseudo* map, Colormap cm, int nAvail, unsigned long *pixels,
 				  unsigned char (*mp) [8][8][8] )
 {
@@ -222,12 +222,12 @@ fillUpColorCube ( Rgb2Pseudo* map, Colormap cm, int nAvail, unsigned long *pixel
 
 #define MAX_REQUESTS 16
 
-void
+static void
 initColormap ( JNIEnv* env, Toolkit* X, Colormap cm, Rgb2Pseudo* map )
 {
   jclass   clazz;
   jfieldID fid;
-  jarray   rgbRequests = 0;
+  jintArray rgbRequests = 0;
   jboolean isCopy;
   jint     *jrgbs = 0;
   int      nReq = 0;
@@ -354,8 +354,8 @@ initColormap ( JNIEnv* env, Toolkit* X, Colormap cm, Rgb2Pseudo* map )
 
 
 
-Rgb2Pseudo*
-initRgb2Pseudo ( JNIEnv* env, Toolkit* X )
+static Rgb2Pseudo*
+initRgb2Pseudo ( JNIEnv* env, jclass clazz, Toolkit* X )
 {
   Colormap dcm;
   int i, j, k;
@@ -384,8 +384,8 @@ initRgb2Pseudo ( JNIEnv* env, Toolkit* X )
  * TrueColor (RGB) visual
  */
 
-Rgb2True*
-initRgb2True ( Toolkit* X )
+static Rgb2True*
+initRgb2True (JNIEnv* env, jclass clazz,  Toolkit* X )
 {
   Visual *v = DefaultVisualOfScreen( DefaultScreenOfDisplay( X->dsp));
   unsigned int m;
@@ -400,7 +400,7 @@ initRgb2True ( Toolkit* X )
 	 * a 0,0 TrueColor conversion, but (esp. for image manipulations) we can save a lot
 	 * of computation by a special TrueColor mode
 	 */
-	DBG( awt_clr, ("AWT color mode: CM_TRUE_888\n"));
+	DBG( AWT_CLR, printf("AWT color mode: CM_TRUE_888\n"));
 	X->colorMode = CM_TRUE_888;
   }
   else {
@@ -450,10 +450,10 @@ initRgb2True ( Toolkit* X )
 
 	X->colorMode = CM_TRUE;
 
-	DBG( awt_clr, ("AWT color mode: CM_TRUE\n"));
-	DBG( awt_clr, ("    red:   %8x, %d\n", map->redMask, map->redShift));
-	DBG( awt_clr, ("    green: %8x, %d\n", map->greenMask, map->greenShift));
-	DBG( awt_clr, ("    blue:  %8x, %d\n", map->blueMask, map->blueShift));
+	DBG( AWT_CLR, printf("AWT color mode: CM_TRUE\n"));
+	DBG( AWT_CLR, printf("    red:   %8x, %d\n", map->redMask, map->redShift));
+	DBG( AWT_CLR, printf("    green: %8x, %d\n", map->greenMask, map->greenShift));
+	DBG( AWT_CLR, printf("    blue:  %8x, %d\n", map->blueMask, map->blueShift));
   }
 
   return map;
@@ -476,7 +476,7 @@ initRgb2True ( Toolkit* X )
  * with a 0-value)
  */
 
-void fillUpPartMap ( unsigned char* pix, unsigned char* val )
+static void fillUpPartMap ( unsigned char* pix, unsigned char* val )
 {
   int i, j, k, i2;
 
@@ -508,7 +508,7 @@ void fillUpPartMap ( unsigned char* pix, unsigned char* val )
   }
 }
 
-void setPartMapFromDMap ( Toolkit *X, Colormap dcm,
+static void setPartMapFromDMap ( Toolkit *X, Colormap dcm,
 						  int component, int nIdx, int idxShift,
 						  unsigned char* pix, unsigned char* val )
 {
@@ -536,8 +536,8 @@ void setPartMapFromDMap ( Toolkit *X, Colormap dcm,
   }
 }
 
-Rgb2Direct*
-initRgb2Direct ( Toolkit* X )
+static Rgb2Direct*
+initRgb2Direct ( JNIEnv* env, jclass clazz, Toolkit* X )
 {
   Visual      *v = DefaultVisualOfScreen( DefaultScreenOfDisplay( X->dsp));
   Rgb2Direct  *map = (Rgb2Direct*) AWT_MALLOC( sizeof( Rgb2Direct));
@@ -603,35 +603,35 @@ initRgb2Direct ( Toolkit* X )
  */
 
 void
-initColorMapping ( JNIEnv* env, Toolkit* X )
+initColorMapping ( JNIEnv* env, jclass clazz,  Toolkit* X )
 {
   Visual *v = DefaultVisualOfScreen( DefaultScreenOfDisplay( X->dsp));
 
-  DBG( awt_clr, ("X visual:\n"));
-  DBG( awt_clr, ("  id:          %d\n", v->visualid));
-  DBG( awt_clr, ("  class:       %d\n", v->class));
-  DBG( awt_clr, ("  red_mask     %x\n", v->red_mask));
-  DBG( awt_clr, ("  green_mask   %x\n", v->green_mask));
-  DBG( awt_clr, ("  blue_mask    %x\n", v->blue_mask));
-  DBG( awt_clr, ("  bits_per_rgb %x\n", v->bits_per_rgb));
-  DBG( awt_clr, ("  map_entries  %d\n", v->map_entries));
+  DBG( AWT_CLR, printf("X visual:\n"));
+  DBG( AWT_CLR, printf("  id:          %ld\n", v->visualid));
+  DBG( AWT_CLR, printf("  class:       %d\n", v->class));
+  DBG( AWT_CLR, printf("  red_mask     %lx\n", v->red_mask));
+  DBG( AWT_CLR, printf("  green_mask   %lx\n", v->green_mask));
+  DBG( AWT_CLR, printf("  blue_mask    %lx\n", v->blue_mask));
+  DBG( AWT_CLR, printf("  bits_per_rgb %x\n", v->bits_per_rgb));
+  DBG( AWT_CLR, printf("  map_entries  %d\n", v->map_entries));
 
   /* check for directly supported color modes / visuals */
   switch ( v->class ) {
   case DirectColor:
-	X->dclr = initRgb2Direct( X);
+	X->dclr = initRgb2Direct( env, clazz, X);
 	break;
   case TrueColor:
-	X->tclr = initRgb2True( X);
+	X->tclr = initRgb2True( env, clazz, X);
 	break;
   case PseudoColor:  
-	X->pclr = initRgb2Pseudo( env, X); 
+	X->pclr = initRgb2Pseudo( env, clazz, X); 
 	break;
   default:
 	X->colorMode = CM_GENERIC;
   }
 
-  DBG( awt_clr, ("colorMode: %d\n", X->colorMode));
+  DBG( AWT_CLR, printf("colorMode: %d\n", X->colorMode));
 }
 
 
@@ -649,16 +649,16 @@ Java_java_awt_Toolkit_clrGetPixelValue ( JNIEnv* env, jclass clazz, jint rgb )
    * (the notorious DefaultsRGB workaround)
    */
   if ( !X->colorMode )
-	initColorMapping( env, X);
+	initColorMapping( env, clazz, X);
 
   pix = pixelValue( X, rgb);
-  DBG( awt_clr, ("clrGetPixelValue: %8x -> %x (%d)\n", rgb, pix, pix));
+  DBG( AWT_CLR, printf("clrGetPixelValue: %8x -> %x (%d)\n", rgb, pix, pix));
 
   return pix;
 }
 
 void
-Java_java_awt_Toolkit_clrSetSystemColors ( JNIEnv* env, jclass clazz, jarray sysClrs )
+Java_java_awt_Toolkit_clrSetSystemColors ( JNIEnv* env, jclass clazz, jintArray sysClrs )
 {
 #ifdef NEVER /* maybe this could be initialized via X resources */
   jboolean isCopy;
@@ -745,3 +745,39 @@ Java_java_awt_Toolkit_clrDark ( JNIEnv* env, jclass clazz, jint rgb )
 
   return (((jlong)modPix << 32) | modRgb);
 }
+
+jobject
+Java_java_awt_Toolkit_clrGetColorModel ( JNIEnv* env, jclass clazz )
+{
+  jobject    cm = 0;
+  jclass     cmClazz;
+  jmethodID  cmCtorId;
+  Visual     *v = DefaultVisualOfScreen( DefaultScreenOfDisplay( X->dsp));
+
+  if ( !X->colorMode )
+	initColorMapping( env, clazz, X);
+
+  switch ( v->class ) {
+  case DirectColor:
+	break;
+
+  case TrueColor:
+	cmClazz = (*env)->FindClass( env, "java/awt/image/DirectColorModel");
+	cmCtorId = (*env)->GetMethodID( env, cmClazz, "<init>", "(IIIII)V");
+	cm = (*env)->NewObject( env, cmClazz, cmCtorId,
+							v->bits_per_rgb, v->red_mask, v->green_mask, v->blue_mask, 0);
+	break;
+
+  case PseudoColor:  
+	cmClazz = (*env)->FindClass( env, "java/awt/IndexColorModel");
+	cmCtorId = (*env)->GetMethodID( env, cmClazz, "<init>", "(I[II)V");
+	//rgbs = (*env)->NewIntArray( env, 256, 0);
+	//cm = (*env)->NewObject( env, cmClazz, cmCtorId, 8, rgbs, 0);
+	break;
+
+  default:
+
+  }
+
+  return cm;
+}
