@@ -42,7 +42,7 @@ Java_java_awt_Toolkit_graInitGraphics ( JNIEnv* env, jclass clazz,
   XRectangle     rect;
   unsigned long  valueMask = GCForeground | GCBackground | GCFont | GCFunction;
 
-  DBG( awt_gra, ("initGraphics: %x (%x, %d) %d,%d  %d,%d - %d,%d  %x %x %x\n",
+  DBG( AWT_GRA, printf("initGraphics: %p (%p, %d) %d,%d  %d,%d - %d,%d  %p %x %x\n",
 				 gr, tgt,tgtType, xOff,yOff, xClip,yClip,wClip,hClip, fnt,fg,bg));
 
   if ( tgt ) {
@@ -75,7 +75,7 @@ Java_java_awt_Toolkit_graInitGraphics ( JNIEnv* env, jclass clazz,
 	gr = (Graphics*) AWT_MALLOC( sizeof(Graphics));
 	gr->gc = XCreateGC( X->dsp, drw, valueMask, &values);
 
-	DBG( awt_gra, (" ->gr: %x (%x)\n", gr, gr->gc));
+	DBG( AWT_GRA, printf(" ->gr: %p (%p)\n", gr, gr->gc));
   }
 
   rect.x      = xClip; 
@@ -104,7 +104,7 @@ Java_java_awt_Toolkit_graInitGraphics ( JNIEnv* env, jclass clazz,
 void
 Java_java_awt_Toolkit_graFreeGraphics ( JNIEnv* env, jclass clazz, Graphics* gr )
 {
-  DBG( awt_gra, ("freeGraphics: %x\n", gr));
+  DBG( AWT_GRA, printf("freeGraphics: %p\n", gr));
 
   XFreeGC( X->dsp, gr->gc);
   AWT_FREE( gr);
@@ -116,7 +116,11 @@ Java_java_awt_Toolkit_graCopyArea ( JNIEnv* env, jclass clazz, Graphics* gr,
 									jint x, jint y, jint width, jint height,
 									jint xDelta, jint yDelta )
 {
-  DBG( awt_gra, ("copyArea: %x, %d,%d-%d,%d  %d,%d\n", gr, x,y,width,height, xDelta,yDelta));
+  DBG( AWT_GRA, printf("copyArea: %p, %d,%d-%d,%d  %d,%d\n", gr, x,y,width,height, xDelta,yDelta));
+
+  /* some X servers act alleric on that (careless bitblt, again) */
+  if ( (width < 0) || (height < 0) )
+	return;
 
   x += gr->x0; y += gr->y0;
   XCopyArea( X->dsp, gr->drw, gr->drw, gr->gc, x, y, width, height, x + xDelta, y + yDelta);
@@ -128,7 +132,7 @@ void
 Java_java_awt_Toolkit_graClearRect ( JNIEnv* env, jclass clazz, Graphics* gr,
 									 jint x, jint y, jint width, jint height )
 {
-  DBG( awt_gra, ("clearRect: %x, %d,%d-%d,%d\n", gr, x,y,width,height));
+  DBG( AWT_GRA, printf("clearRect: %p, %d,%d-%d,%d\n", gr, x,y,width,height));
 
   XSetForeground( X->dsp, gr->gc, gr->bg);
   XFillRectangle( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, width, height);
@@ -138,8 +142,8 @@ Java_java_awt_Toolkit_graClearRect ( JNIEnv* env, jclass clazz, Graphics* gr,
 
 
 void
-Java_java_awt_Toolkit_graDrawBytes ( JNIEnv* env, jclass clazz, Graphics* gr,
-									 jarray jBytes, jint offset, jint len, jint x, jint y )
+Java_java_awt_Toolkit_graDrawBytes ( JNIEnv* env, jclass clazz,
+	Graphics* gr, jbyteArray jBytes, jint offset, jint len, jint x, jint y )
 {
   jboolean isCopy;
   int      n;
@@ -151,7 +155,7 @@ Java_java_awt_Toolkit_graDrawBytes ( JNIEnv* env, jclass clazz, Graphics* gr,
   jb = (*env)->GetByteArrayElements( env, jBytes, &isCopy);
   jbo = jb + offset;
 
-  DBG( awt_gra, ("drawBytes: %x, %x,%d,%d  \"%s\"  %d,%d\n", gr,
+  DBG( AWT_GRA, printf("drawBytes: %p, %p,%d,%d  \"%s\"  %d,%d\n", gr,
 				 jBytes,offset,len, jb, x,y));
 
   if ( offset+len > n )
@@ -165,8 +169,8 @@ Java_java_awt_Toolkit_graDrawBytes ( JNIEnv* env, jclass clazz, Graphics* gr,
 
 
 void
-Java_java_awt_Toolkit_graDrawChars ( JNIEnv* env, jclass clazz, Graphics* gr,
-									 jarray jChars, jint offset, jint len, jint x, jint y )
+Java_java_awt_Toolkit_graDrawChars ( JNIEnv* env, jclass clazz,
+	Graphics* gr, jcharArray jChars, jint offset, jint len, jint x, jint y )
 {
   jboolean isCopy;
   int      n;
@@ -179,7 +183,7 @@ Java_java_awt_Toolkit_graDrawChars ( JNIEnv* env, jclass clazz, Graphics* gr,
   jc = (*env)->GetCharArrayElements( env, jChars, &isCopy);
   jco = jc + offset;
 
-  DBG( awt_gra, ("drawChars: %x, %x,%d,%d  \"%s\"  %d,%d\n", gr, jChars,offset,len,
+  DBG( AWT_GRA, printf("drawChars: %p, %p,%d,%d  \"%s\"  %d,%d\n", gr, jChars,offset,len,
 				 jchar2CString(X,jco,len), x,y));
 
   if ( offset+len > n )
@@ -188,7 +192,7 @@ Java_java_awt_Toolkit_graDrawChars ( JNIEnv* env, jclass clazz, Graphics* gr,
 #ifndef WORDS_BIGENDIAN
   n = sizeof(XChar2b)*len;
   b = (XChar2b*) getBuffer( X, n);
-  swab( jco, b, n);
+  swab( (void*)jco, (void*)b, n);
 #else
   b = (XChar2b*) jco;
 #endif
@@ -209,7 +213,7 @@ Java_java_awt_Toolkit_graDrawString ( JNIEnv* env, jclass clazz,
   const jchar  *jc;
   XChar2b      *b;
 
-  DBG( awt_gra, ("drawString: %x  \"%s\"  %d,%d\n", gr, java2CString(env,X,str), x,y));
+  DBG( AWT_GRA, printf("drawString: %p  \"%s\"  %d,%d\n", gr, java2CString(env,X,str), x,y));
 
   if ( !str ) return;
 
@@ -235,7 +239,7 @@ void
 Java_java_awt_Toolkit_graDrawLine ( JNIEnv* env, jclass clazz, Graphics* gr,
 									jint x1, jint y1, jint x2, jint y2 )
 {
-  DBG( awt_gra, ("drawLine: %x, %d,%d - %d,%d\n", gr, x1,y1, x2,y2));
+  DBG( AWT_GRA, printf("drawLine: %p, %d,%d - %d,%d\n", gr, x1,y1, x2,y2));
 
   if ( (x1==x2) && (y1==y2) ) {  /* swing gimmick */
 	XDrawPoint( X->dsp, gr->drw, gr->gc, x1 +gr->x0, y1 +gr->y0);
@@ -252,7 +256,7 @@ Java_java_awt_Toolkit_graDrawArc ( JNIEnv* env, jclass clazz, Graphics* gr,
 								   jint x, jint y, jint width, jint height,
 								   jint startAngle, jint arcAngle )
 {
-  DBG( awt_gra, ("drawArc: %x, %d,%d-%d,%d  %d,%d\n", gr,
+  DBG( AWT_GRA, printf("drawArc: %p, %d,%d-%d,%d  %d,%d\n", gr,
 				 x,y,width,height, startAngle,arcAngle));
 
   XDrawArc( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, width, height,
@@ -266,7 +270,7 @@ Java_java_awt_Toolkit_graFillArc ( JNIEnv* env, jclass clazz, Graphics* gr,
 								   jint x, jint y, jint width, jint height,
 								   jint startAngle, jint arcAngle )
 {
-  DBG( awt_gra, ("fillArc: %x, %d,%d-%d,%d  %d,%d\n", gr,
+  DBG( AWT_GRA, printf("fillArc: %p, %d,%d-%d,%d  %d,%d\n", gr,
 				 x,y,width,height, startAngle, arcAngle));
 
   XFillArc( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, width, height,
@@ -279,7 +283,7 @@ void
 Java_java_awt_Toolkit_graDrawOval ( JNIEnv* env, jclass clazz, Graphics* gr,
 									jint x, jint y, jint width, jint height )
 {
-  DBG( awt_gra, ("drawOval: %x, %d,%d - %d,%d\n", gr, x,y,width,height));
+  DBG( AWT_GRA, printf("drawOval: %p, %d,%d - %d,%d\n", gr, x,y,width,height));
 
   XDrawArc( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, width, height, 0, 23040);
   XFLUSH( X, False);
@@ -290,7 +294,7 @@ void
 Java_java_awt_Toolkit_graFillOval ( JNIEnv* env, jclass clazz, Graphics* gr,
 									jint x, jint y, jint width, jint height )
 {
-  DBG( awt_gra, ("fillOval: %x, %d,%d - %d,%d\n", gr, x,y,width,height));
+  DBG( AWT_GRA, printf("fillOval: %p, %d,%d - %d,%d\n", gr, x,y,width,height));
 
   XFillArc( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, width, height, 0, 23040);
   XFLUSH( X, False);
@@ -298,7 +302,7 @@ Java_java_awt_Toolkit_graFillOval ( JNIEnv* env, jclass clazz, Graphics* gr,
 
 
 int jarray2Points ( JNIEnv* env, Toolkit* X, XPoint** pp, int x0, int y0,
-					jarray xPoints, jarray yPoints, int nPoints )
+			jintArray xPoints, jintArray yPoints, int nPoints )
 {
   register int i;
   int      n;
@@ -324,13 +328,13 @@ int jarray2Points ( JNIEnv* env, Toolkit* X, XPoint** pp, int x0, int y0,
 
 
 void
-Java_java_awt_Toolkit_graDrawPolygon ( JNIEnv* env, jclass clazz, Graphics* gr,
-									   jarray xPoints, jarray yPoints, jint nPoints )
+Java_java_awt_Toolkit_graDrawPolygon ( JNIEnv* env, jclass clazz,
+	Graphics* gr, jintArray xPoints, jintArray yPoints, jint nPoints )
 {
   int n;
   XPoint   *p;
 
-  DBG( awt_gra, ("drawPolygon: %x, %x,%x  %d\n", gr, xPoints, yPoints, nPoints));
+  DBG( AWT_GRA, printf("drawPolygon: %p, %p,%p  %d\n", gr, xPoints, yPoints, nPoints));
 
   if ( !xPoints || !yPoints ) return;
 
@@ -350,11 +354,11 @@ Java_java_awt_Toolkit_graDrawPolygon ( JNIEnv* env, jclass clazz, Graphics* gr,
 
 void
 Java_java_awt_Toolkit_graDrawPolyline ( JNIEnv* env, jclass clazz, Graphics* gr,
-										jarray xPoints, jarray yPoints, jint nPoints )
+										jintArray xPoints, jintArray yPoints, jint nPoints )
 {
   XPoint   *p;
 
-  DBG( awt_gra, ("drawPolyline: %x, %x,%x  %d\n", gr, xPoints, yPoints, nPoints));
+  DBG( AWT_GRA, printf("drawPolyline: %p, %p,%p  %d\n", gr, xPoints, yPoints, nPoints));
 
   if ( !xPoints || !yPoints ) return;
 
@@ -365,12 +369,12 @@ Java_java_awt_Toolkit_graDrawPolyline ( JNIEnv* env, jclass clazz, Graphics* gr,
 
 
 void
-Java_java_awt_Toolkit_graFillPolygon ( JNIEnv* env, jclass clazz, Graphics* gr,
-									   jarray xPoints, jarray yPoints, jint nPoints )
+Java_java_awt_Toolkit_graFillPolygon ( JNIEnv* env, jclass clazz,
+	Graphics* gr, jintArray xPoints, jintArray yPoints, jint nPoints )
 {
   XPoint   *p;
 
-  DBG( awt_gra, ("fillPolygon: %x, %x,%x  %d\n", gr, xPoints, yPoints, nPoints));
+  DBG( AWT_GRA, printf("fillPolygon: %p, %p,%p  %d\n", gr, xPoints, yPoints, nPoints));
 
   if ( !xPoints || !yPoints ) return;
 
@@ -384,7 +388,7 @@ void
 Java_java_awt_Toolkit_graDrawRect ( JNIEnv* env, jclass clazz, Graphics* gr,
 									jint x, jint y, jint width, jint height )
 {
-  DBG( awt_gra, ("drawRect: %x, %d,%d - %d,%d\n", gr, x,y,width,height));
+  DBG( AWT_GRA, printf("drawRect: %p, %d,%d - %d,%d\n", gr, x,y,width,height));
 
   if ( (width >= 0) && (height >= 0) )
 	XDrawRectangle( X->dsp, gr->drw, gr->gc, x+gr->x0, y+gr->y0, width, height);
@@ -396,7 +400,7 @@ void
 Java_java_awt_Toolkit_graFillRect ( JNIEnv* env, jclass clazz, Graphics* gr,
 									jint x, jint y, jint width, jint height )
 {
-  DBG( awt_gra, ("fillRect: %x, %d,%d - %d,%d\n", gr, x,y,width,height));
+  DBG( AWT_GRA, printf("fillRect: %p, %d,%d - %d,%d\n", gr, x,y,width,height));
 
   if ( (width >= 0) && (height >= 0) ) {
 	if ( width == 1 ) {   /* some swing gimmicks */
@@ -424,7 +428,7 @@ Java_java_awt_Toolkit_graDrawRoundRect ( JNIEnv* env, jclass clazz, Graphics* gr
 {
   int x1, x2, y1, y2, a, b;
 
-  DBG( awt_gra, ("drawRoundRect: %x, %d,%d - %d,%d  %d,%d\n", gr,
+  DBG( AWT_GRA, printf("drawRoundRect: %p, %d,%d - %d,%d  %d,%d\n", gr,
 				 x,y,width,height, wArc, hArc));
   
   x += gr->x0;  y += gr->y0;
@@ -458,7 +462,7 @@ Java_java_awt_Toolkit_graFillRoundRect ( JNIEnv* env, jclass clazz, Graphics* gr
 {
   int x1, x2, y1, y2, a, b;
 
-  DBG( awt_gra, ("fillRoundRect: %x, %d,%d - %d,%d  %d,%d\n", gr,
+  DBG( AWT_GRA, printf("fillRoundRect: %p, %d,%d - %d,%d  %d,%d\n", gr,
 				 x,y,width,height, wArc, hArc));
   
   x += gr->x0;  y += gr->y0;
@@ -496,7 +500,7 @@ Java_java_awt_Toolkit_graDraw3DRect ( JNIEnv* env, jclass clazz, Graphics* gr,
   int      bright, dark;
   int      xw, yh;
 
-  DBG( awt_gra, ("draw3DRect: %x, %d,%d - %d,%d  %d %x\n", gr,
+  DBG( AWT_GRA, printf("draw3DRect: %p, %d,%d - %d,%d  %d %x\n", gr,
 				 x,y,width,height, raised, rgb));
 
   /* we should use the same mechanism like ordinary Graphics brighter()/darker() here */
@@ -525,7 +529,7 @@ Java_java_awt_Toolkit_graFill3DRect ( JNIEnv* env, jclass clazz, Graphics* gr,
 									  jint x, jint y, jint width, jint height,
 									  jboolean raised, jint rgb )
 {
-  DBG( awt_gra, ("fill3DRect: %x, %d,%d - %d,%d  %d %x\n", gr,
+  DBG( AWT_GRA, printf("fill3DRect: %p, %d,%d - %d,%d  %d %x\n", gr,
 				 x,y,width,height, raised, rgb));
 
   XFillRectangle( X->dsp, gr->drw, gr->gc, x+gr->x0+1, y+gr->y0+1, width-2, height-2);
@@ -548,7 +552,7 @@ Java_java_awt_Toolkit_graSetClip ( JNIEnv* env, jclass clazz, Graphics* gr,
 {
   XRectangle rect;
 
-  DBG( awt_gra, ("setClip: %x, %d,%d - %d,%d\n", gr, xClip, yClip, wClip, hClip));
+  DBG( AWT_GRA, printf("setClip: %p, %d,%d - %d,%d\n", gr, xClip, yClip, wClip, hClip));
 
   rect.x      = xClip;
   rect.y      = yClip;
@@ -560,7 +564,7 @@ Java_java_awt_Toolkit_graSetClip ( JNIEnv* env, jclass clazz, Graphics* gr,
 void
 Java_java_awt_Toolkit_graSetColor ( JNIEnv* env, jclass clazz, Graphics* gr, jint clr )
 {
-  DBG( awt_gra, ("setColor: %x, %x\n", gr, clr));
+  DBG( AWT_GRA, printf("setColor: %p, %x\n", gr, clr));
 
   gr->fg = clr;
 
@@ -573,7 +577,7 @@ Java_java_awt_Toolkit_graSetColor ( JNIEnv* env, jclass clazz, Graphics* gr, jin
 void
 Java_java_awt_Toolkit_graSetBackColor ( JNIEnv* env, jclass clazz, Graphics* gr, jint clr )
 {
-  DBG( awt_gra, ("setBackColor: %x, %x\n", gr, clr));
+  DBG( AWT_GRA, printf("setBackColor: %p, %x\n", gr, clr));
 
   gr->bg = clr;
   XSetBackground( X->dsp, gr->gc, clr);
@@ -582,7 +586,7 @@ Java_java_awt_Toolkit_graSetBackColor ( JNIEnv* env, jclass clazz, Graphics* gr,
 void
 Java_java_awt_Toolkit_graSetFont ( JNIEnv* env, jclass clazz, Graphics* gr, jobject fnt )
 {
-  DBG( awt_gra, ("setFont: %x, %x\n", gr, fnt));
+  DBG( AWT_GRA, printf("setFont: %p, %p\n", gr, fnt));
 
   XSetFont( X->dsp, gr->gc, ((XFontStruct*)fnt)->fid);
 }
@@ -590,7 +594,7 @@ Java_java_awt_Toolkit_graSetFont ( JNIEnv* env, jclass clazz, Graphics* gr, jobj
 void
 Java_java_awt_Toolkit_graSetOffset ( JNIEnv* env, jclass clazz, Graphics* gr, jint xOff, jint yOff )
 {
-  DBG( awt_gra, ("setOffset: %x, %d,%d\n", gr, xOff, yOff));
+  DBG( AWT_GRA, printf("setOffset: %p, %d,%d\n", gr, xOff, yOff));
 
   gr->x0 = xOff;
   gr->y0 = yOff;
@@ -599,7 +603,7 @@ Java_java_awt_Toolkit_graSetOffset ( JNIEnv* env, jclass clazz, Graphics* gr, ji
 void
 Java_java_awt_Toolkit_graSetPaintMode ( JNIEnv* env, jclass clazz, Graphics* gr )
 {
-  DBG( awt_gra, ("setPaintMode: %x\n", gr));
+  DBG( AWT_GRA, printf("setPaintMode: %p\n", gr));
 
   gr->xor = 0;
   XSetForeground( X->dsp, gr->gc, gr->fg);
@@ -609,7 +613,7 @@ Java_java_awt_Toolkit_graSetPaintMode ( JNIEnv* env, jclass clazz, Graphics* gr
 void
 Java_java_awt_Toolkit_graSetXORMode ( JNIEnv* env, jclass clazz, Graphics* gr, jint xorClr )
 {
-  DBG( awt_gra, ("setXORMode: %x, %x\n", gr, xorClr));
+  DBG( AWT_GRA, printf("setXORMode: %p, %x\n", gr, xorClr));
 
   gr->xor = 1;
   gr->xclr = xorClr;
@@ -620,7 +624,7 @@ Java_java_awt_Toolkit_graSetXORMode ( JNIEnv* env, jclass clazz, Graphics* gr, j
 void
 Java_java_awt_Toolkit_graSetVisible ( JNIEnv* env, jclass clazz, Graphics* gr, jint isVisible )
 {
-  DBG( awt_gra, ("setVisble: %x  %d\n", gr, isVisible));
+  DBG( AWT_GRA, printf("setVisble: %p  %d\n", gr, isVisible));
 
   /*
    * This is rather a hack to "defuse" a Graphics object, but we don't want to
@@ -641,7 +645,7 @@ Java_java_awt_Toolkit_graSetVisible ( JNIEnv* env, jclass clazz, Graphics* gr, j
  * image rendering
  */
 
-void
+static void
 drawAlphaImage ( Graphics* gr, Image* img,
 				 jint srcX, jint srcY, jint dstX, jint dstY,
 				 jint width, jint height, jint bgval )
@@ -652,10 +656,11 @@ drawAlphaImage ( Graphics* gr, Image* img,
   unsigned long dpix, spix, bgpix = 0;
   int    sr, sg, sb, dr, dg, db;
 
-  DBG( awt_gra, ("drawAlphaImage: %x %x (%x, %x),  %d,%d  %d,%d  %d,%d  %x\n",
+  DBG( AWT_GRA, printf("drawAlphaImage: %p %p (%p, %p),  %d,%d  %d,%d  %d,%d  %x\n",
 				 gr, img, img->xImg, img->alpha, srcX, srcY, dstX, dstY, width, height, bgval));
 
-#ifdef HAVE_LIBXEXT
+  if ( !img ) return;
+
   if ( X->shm == USE_SHM ){
 	dstImg = createImage( width, height);
 	createXImage( X, dstImg);
@@ -664,13 +669,10 @@ drawAlphaImage ( Graphics* gr, Image* img,
 	  XShmGetImage( X->dsp, gr->drw, dstXim, dstX, dstY, 0xffffffff);
 	}
 	else {  /* Shm failed, backup to normal XImage */
-#endif
 	  Java_java_awt_Toolkit_imgFreeImage( 0, 0, dstImg);
 	  dstImg = 0;
-#ifdef HAVE_LIBXEXT
 	}
   }
-#endif
 
   if ( !dstXim ) {
 	dstXim = XGetImage( X->dsp, gr->drw, dstX, dstY, width, height, 0xffffffff, ZPixmap);
@@ -707,20 +709,16 @@ drawAlphaImage ( Graphics* gr, Image* img,
 	  }
 	}
 
-#ifdef HAVE_LIBXEXT
 	if ( dstImg != 0 ){
 	  XShmPutImage( X->dsp, gr->drw, gr->gc, dstXim, 0, 0, dstX, dstY, width, height, False);
 	  XSync( X->dsp, False);
 	  Java_java_awt_Toolkit_imgFreeImage( 0, 0, dstImg);
 	}
 	else {
-#endif
 	  XPutImage( X->dsp, gr->drw, gr->gc, dstXim, 0, 0, dstX, dstY, width, height);
 	  XFlush( X->dsp);
 	  XDestroyImage( dstXim);
-#ifdef HAVE_LIBXEXT
 	}
-#endif
 
 	XFLUSH( X, True);
   }
@@ -735,13 +733,18 @@ Java_java_awt_Toolkit_graDrawImage ( JNIEnv* env, jclass clazz, Graphics* gr, Im
 {
   XGCValues values;
 
-  DBG( awt_gra, ("drawImage: %x %x (%x,%x,%x %d,%d) %d,%d, %d,%d, %d,%d, %x\n",
+  DBG( AWT_GRA, printf("drawImage: %p %p (%p,%p,%p %d,%d) %d,%d, %d,%d, %d,%d, %x\n",
 				 gr, img,  img->xImg,img->xMask,img->alpha,  img->width,img->height,
 				 srcX,srcY,  dstX,dstY,	 width,height, bgval));
 
+  if ( !img ) return;
+
   dstX += gr->x0;
   dstY += gr->y0;
 
+  dstX += img->left;
+  dstY += img->top;
+
   if ( img->pix ) {
 	XCopyArea( X->dsp, img->pix, gr->drw, gr->gc, srcX, srcY, width, height, dstX, dstY);
   }
@@ -763,18 +766,14 @@ Java_java_awt_Toolkit_graDrawImage ( JNIEnv* env, jclass clazz, Graphics* gr, Im
 	  XSetBackground( X->dsp, gr->gc, 0);
 
 	  /* draw the mask bitmap */
-#ifdef HAVE_LIBXEXT
 	  if ( img->shmiMask ){
 		XShmPutImage( X->dsp, gr->drw, gr->gc, img->xMask,
 					  srcX, srcY, dstX, dstY, width, height, False);
 	  }
 	  else {
-#endif
 		XPutImage( X->dsp, gr->drw, gr->gc, img->xMask,
 				   srcX, srcY, dstX, dstY, width, height);
-#ifdef HAVE_LIBXEXT
 	  }
-#endif
 
 	  /* restore gc values except of the function */
 	  XChangeGC( X->dsp, gr->gc, GCFunction|GCPlaneMask|GCForeground|GCBackground, &values);
@@ -786,18 +785,14 @@ Java_java_awt_Toolkit_graDrawImage ( JNIEnv* env, jclass clazz, Graphics* gr, Im
 	}
 
 	/* this is the real image drawing */
-#ifdef HAVE_LIBXEXT
 	if ( img->shmiImg ){
 	  XShmPutImage( X->dsp, gr->drw, gr->gc, img->xImg,
 					srcX, srcY, dstX, dstY, width, height, False);
 	}
 	else {
-#endif
 	  XPutImage( X->dsp, gr->drw, gr->gc, img->xImg,
 				 srcX, srcY, dstX, dstY, width, height);
-#ifdef HAVE_LIBXEXT
 	}
-#endif
 
 	/* in case we had a mask, restore the original function */
 	if ( img->xMask )
@@ -817,9 +812,11 @@ Java_java_awt_Toolkit_graDrawImageScaled ( JNIEnv* env, jclass clazz, Graphics*
   Image      *tgt;
   int        tmpXImg = (img->xImg == NULL);
 
-  DBG( awt_gra, ("drawImageScaled: %x %x (%x), %d,%d,%d,%d, %d,%d,%d,%d, %x\n",
+  DBG( AWT_GRA, printf("drawImageScaled: %p %p (%p), %d,%d,%d,%d, %d,%d,%d,%d, %x\n",
 				 gr, img, img->xImg, dx0, dy0, dx1, dy1, sx0, sy0, sx1, sy1, bgval));
 
+  if ( !img ) return;
+
   /* screen images don't have an XImage, we have to get the data first */
   if ( tmpXImg ) {
   	img->xImg = XGetImage( X->dsp, img->pix, 0, 0, img->width, img->height,
@@ -857,11 +854,9 @@ Java_java_awt_Toolkit_graDrawImageScaled ( JNIEnv* env, jclass clazz, Graphics*
   initScaledImage( X, tgt, img, dx0-x0, dy0-y0, dx1-x0, dy1-y0, sx0, sy0, sx1, sy1);
   Java_java_awt_Toolkit_graDrawImage ( env, clazz, gr, tgt, 0, 0,
 									   x0, y0, tgt->width, tgt->height, bgval);
-#ifdef HAVE_LIBXEXT
   if ( tgt->shmiImg ) {
 	XSync( X->dsp, False); /* since we're going to destroy tgt, process its drawing first */
   }
-#endif
   Java_java_awt_Toolkit_imgFreeImage( env, clazz, tgt);
 
   if ( tmpXImg ) {
