@@ -13,11 +13,11 @@
 
 
 /* interfaces of image conversion functions */
-Image* readGifFile ( FILE* );
+Image* readGifFile ( int fd );
 Image* readGifData ( unsigned char*, long len );
-Image* readJpegFile ( FILE* );
+Image* readJpegFile ( int fd );
 Image* readJpegData ( unsigned char*, long len );
-Image* readPngFile ( FILE* );
+Image* readPngFile ( int fd );
 Image* readPngData ( unsigned char*, long len );
 
 /************************************************************************************
@@ -37,10 +37,10 @@ createImage ( int width, int height )
   return img;
 }
 
-#ifdef HAVE_LIBXEXT
-int
+static int
 createShmXImage ( Toolkit* X, Image* img, int depth, int isMask )
 {
+#if defined(HAVE_LIBXEXT)
   Visual  *vis = DefaultVisual( X->dsp, DefaultScreen( X->dsp));
   XShmSegmentInfo* shmi = (XShmSegmentInfo*) AWT_MALLOC( sizeof(XShmSegmentInfo));
   XImage *xim;
@@ -90,12 +90,16 @@ createShmXImage ( Toolkit* X, Image* img, int depth, int isMask )
   }
 
   return 1;
+#else
+  return 0;
+#endif
 }
 
 
-void
+static void
 destroyShmXImage ( Toolkit* X, Image* img, int isMask )
 {
+#if defined(HAVE_LIBXEXT)
   XShmSegmentInfo *shmi;
   XImage          *xim;
 
@@ -120,8 +124,8 @@ destroyShmXImage ( Toolkit* X, Image* img, int isMask )
   /* if we have suspended shm, give it a try again */
   if ( X->shm == SUSPEND_SHM )
 	X->shm = USE_SHM;
+#endif
 }
-#endif /* HAVE_LIBXEXT */
 
 
 
@@ -144,20 +148,18 @@ createXImage ( Toolkit* X, Image* img )
   bitmap_pad = bytes_per_pix * 8;
   nPix = img->width * img->height;
 
-#ifdef HAVE_LIBXEXT
   if ( (X->shm == USE_SHM) && (nPix > X->shmThreshold) && (img->alpha == 0) ) {
 	if ( createShmXImage( X, img, depth, False) ){
-	  DBG( awt_img, ( "alloc Shm: %x %x %x (%dx%d) \n", img, img->xImg, img->shmiImg,
+	  DBG( AWT_IMG, printf("alloc Shm: %p %p %p (%dx%d) \n", img, img->xImg, img->shmiImg,
 					  img->width, img->height));
 	  return;
 	}
   }
-#endif
 
   data = AWT_CALLOC( nPix, bytes_per_pix);
   img->xImg = XCreateImage( X->dsp, vis, depth, ZPixmap, 0,
 							data, img->width, img->height, bitmap_pad, bytes_per_line);
-  DBG( awt_img, ( "alloc: %x %x (%dx%d)\n", img, img->xImg, img->width, img->height));
+  DBG( AWT_IMG, printf( "alloc: %p %p (%dx%d)\n", img, img->xImg, img->width, img->height));
 }
 
 void
@@ -172,22 +174,20 @@ createXMaskImage ( Toolkit* X, Image* img )
   nPix   = img->width * img->height;
   nBytes = bytes_per_line * img->height;
 
-#ifdef HAVE_LIBXEXT
   if ( (X->shm == USE_SHM) && (nPix > X->shmThreshold) ) {
 	if ( createShmXImage( X, img, 1, True) ){
-	  DBG( awt_img, ( "alloc Shm mask: %x %x %x (%dx%d) \n", img, img->xMask, img->shmiMask,
+	  DBG( AWT_IMG, printf( "alloc Shm mask: %p %p %p (%dx%d) \n", img, img->xMask, img->shmiMask,
 					  img->width, img->height));
 	  return;
 	}
   }
-#endif
 
   data = AWT_MALLOC( nBytes);
   memset( data, 0xff, nBytes);
 
   img->xMask = XCreateImage( X->dsp, vis, 1, XYBitmap, 0,
 							 data, img->width, img->height, 8, bytes_per_line );
-  DBG( awt_img, ( "alloc mask: %x %x (%dx%d)\n", img, img->xMask, img->width, img->height));
+  DBG( AWT_IMG, printf( "alloc mask: %p %p (%dx%d)\n", img, img->xMask, img->width, img->height));
 }
 
 
@@ -231,33 +231,11 @@ needsFullAlpha ( Toolkit* X, Image *img, double threshold )
   return 0;
 }
 
-void
-countAlphas ( Image *img, int* noAlpha, int* partAlpha, int* fullAlpha )
-{
-  int i, j, a;
-
-  if ( !img->alpha ) return;
-
-  for ( i=0; i<img->height; i++ ) {
-	for ( j=0; j<img->width; j++ ) {
-	  a = GetAlpha( img->alpha, j, i);
-	  if ( a == 0 ) {
-		(*noAlpha)++;
-	  }
-	  else if ( a == 0xff ) {
-		(*fullAlpha)++;
-	  }
-	  else {
-		(*partAlpha)++;
-	  }
-	}
-  }
-}
-
 /*
  * A full alpha image channel is way slower than using a mask bitmap (= 0 / 0xff alpha).
  * This function provides a simple alpha-to-mask translation
  */
+/* also used in imgpng */
 void
 reduceAlpha ( Toolkit* X, Image* img, int threshold )
 {
@@ -284,7 +262,7 @@ reduceAlpha ( Toolkit* X, Image* img, int threshold )
 }
 
 
-__inline__ int
+static __inline__ int
 interpolate ( int ul, int ur, int ll, int lr, double dx, double dy )
 {
   double u = ul + (double)(ur - ul) * dx;
@@ -293,7 +271,7 @@ interpolate ( int ul, int ur, int ll, int lr, double dx, double dy )
   return (int) (u + (l - u) * dy  + 0.5);
 }
 
-unsigned int
+static unsigned int
 getScaledAlpha ( Toolkit* X, Image* img, int x, int y, double dx, double dy )
 {
   int   ul, ur, ll, lr, a;
@@ -312,7 +290,7 @@ getScaledAlpha ( Toolkit* X, Image* img, int x, int y, double dx, double dy )
   return 0xff;
 }
 
-long
+static long
 getScaledPixel ( Toolkit* X, Image* img, int x, int y, double dx, double dy )
 {
   unsigned long  ul, ur, ll, lr;
@@ -426,9 +404,9 @@ Java_java_awt_Toolkit_imgCreateScreenImage ( JNIEnv* env, jclass clazz, jint wid
 
 void
 Java_java_awt_Toolkit_imgSetIdxPels ( JNIEnv* env, jclass clazz, Image * img,
-									  jint x, jint y, jint w, jint h,
-									  jarray clrMap, jarray idxPels, jint trans,
-									  jint off, jint scan)
+		jint x, jint y, jint w, jint h,
+		jintArray clrMap, jbyteArray idxPels, jint trans,
+		jint off, jint scan)
 {
   register int    row, col;
   unsigned long   pix;
@@ -467,7 +445,7 @@ Java_java_awt_Toolkit_imgSetIdxPels ( JNIEnv* env, jclass clazz, Image * img,
 void
 Java_java_awt_Toolkit_imgSetRGBPels ( JNIEnv* env, jclass clazz, Image * img,
 									  jint x, jint y, jint w, jint h,
-									  jarray rgbPels, jint off, jint scan)
+									  jintArray rgbPels, jint off, jint scan)
 {
   register int    row, col;
   unsigned long   pix = 0;
@@ -541,41 +519,32 @@ Java_java_awt_Toolkit_imgFreeImage( JNIEnv* env, jclass clazz, Image * img)
 	 */
 
 	if ( img->xImg ){
-#ifdef HAVE_LIBXEXT
 	  if ( img->shmiImg ) {
-		DBG( awt_img, ( "free Shm: %x %x %x (%dx%d)\n", img, img->xImg, img->shmiImg,
+		DBG( AWT_IMG, printf( "free Shm: %p %p %p (%dx%d)\n", img, img->xImg, img->shmiImg,
 						img->width, img->height));
 		destroyShmXImage( X, img, False);
 	  }
 	  else {
-#endif
-		DBG( awt_img, ( "free: %x %x (%dx%d)\n", img, img->xImg, img->width, img->height));
+		DBG( AWT_IMG, printf( "free: %p %p (%dx%d)\n", img, img->xImg, img->width, img->height));
 		AWT_FREE( img->xImg->data);
 		img->xImg->data = 0;
 		XDestroyImage( img->xImg);
-#ifdef HAVE_LIBXEXT
 	  }
-#endif
 	  img->xImg = 0;
 	}
 
 	if ( img->xMask ){
-#ifdef HAVE_LIBXEXT
 	  if ( img->shmiMask ) {
-		DBG( awt_img, ( "free Shm mask: %x %x %x (%dx%d)\n", img, img->xMask, img->shmiMask,
+		DBG( AWT_IMG, printf( "free Shm mask: %p %p %p (%dx%d)\n", img, img->xMask, img->shmiMask,
 						img->width, img->height));
 		destroyShmXImage( X, img, True);
 	  }
 	  else {
-#endif
-		DBG( awt_img, ( "free mask: %x %x (%dx%d)\n", img, img->xMask,
-						img->width, img->height));
+		DBG( AWT_IMG, printf( "free mask: %p %p (%dx%d)\n", img, img->xMask, img->width, img->height));
 		AWT_FREE( img->xMask->data);
 		img->xMask->data = 0;
 		XDestroyImage( img->xMask);
-#ifdef HAVE_LIBXEXT
 	  }
-#endif
 	  img->xMask = 0;
 	}
 
@@ -621,42 +590,60 @@ Java_java_awt_Toolkit_imgCreateScaledImage ( JNIEnv* env, jclass clazz,
 void
 Java_java_awt_Toolkit_imgProduceImage ( JNIEnv* env, jclass clazz, jobject producer, Image* img )
 {
-  int       i, j;
-  int       r, g, b;
-  unsigned long pix;
-  jboolean  isCopy;
-  jclass    prodClazz  = (*env)->GetObjectClass( env, producer);
-  jclass    modelClazz = (*env)->FindClass( env, "java/awt/image/ColorModel");
+  int            i, j;
+  int            r, g, b;
+  unsigned long  pix;
+  jclass         prodClazz, modelClazz;
+  jmethodID      modelCtor, setDim, setCM, setHints, setPix, imgCompl;
+  jobject        model;
+  jintArray      pelArray;
+  jint           *pels;
+  jboolean       isCopy;
 
-  jmethodID modelCtor = (*env)->GetStaticMethodID( env, modelClazz, "getRGBdefault", "()Ljava/awt/image/ColorModel;");
-  jmethodID setDim    = (*env)->GetMethodID( env, prodClazz, "setDimensions", "(II)V");
-  jmethodID setCM     = (*env)->GetMethodID( env, prodClazz, "setColorModel", "(Ljava/awt/image/ColorModel;)V");
-  jmethodID setHints  = (*env)->GetMethodID( env, prodClazz, "setHints", "(I)V");
-  jmethodID setPix    = (*env)->GetMethodID( env, prodClazz, "setPixels", "(IIIILjava/awt/image/ColorModel;[III)V");
-  jmethodID imgCompl  = (*env)->GetMethodID( env, prodClazz, "imageComplete", "(I)V");
+  prodClazz  = (*env)->GetObjectClass( env, producer);
+  modelClazz = (*env)->FindClass( env, "kaffe/awt/JavaColorModel");
 
-  jobject   model     = (*env)->CallStaticObjectMethod( env, modelClazz, modelCtor);
+  modelCtor = (*env)->GetStaticMethodID( env, modelClazz, "getSingleton", "()Lkaffe/awt/JavaColorModel;");
+  setDim    = (*env)->GetMethodID( env, prodClazz, "setDimensions", "(II)V");
+  setCM     = (*env)->GetMethodID( env, prodClazz, "setColorModel", "(Ljava/awt/image/ColorModel;)V");
+  setHints  = (*env)->GetMethodID( env, prodClazz, "setHints", "(I)V");
+  setPix    = (*env)->GetMethodID( env, prodClazz, "setPixels", "(IIIILjava/awt/image/ColorModel;[III)V");
+  imgCompl  = (*env)->GetMethodID( env, prodClazz, "imageComplete", "(I)V");
+
+  model     = (*env)->CallStaticObjectMethod( env, modelClazz, modelCtor);
 
   /* for JDK compat, the pixel buffer has to be large enough to hold the *complete* image */
-  jarray    pelArray  = (*env)->NewIntArray( env, img->width * img->height);
-  jint*     pels = (*env)->GetIntArrayElements( env, pelArray, &isCopy);
+  pelArray  = (*env)->NewIntArray( env, img->width * img->height);
+  pels = (*env)->GetIntArrayElements( env, pelArray, &isCopy);
 
   (*env)->CallVoidMethod( env, producer, setDim, img->width, img->height);
   (*env)->CallVoidMethod( env, producer, setCM, model);
   (*env)->CallVoidMethod( env, producer, setHints, 6); /* TOPDOWNLEFTRIGHT | COMPLETESCANLINES */
 
-  for ( j=0; j<img->height; j++ ) {
-	for ( i=0; i<img->width; i++ ) {
-	  if ( (img->xMask == 0) || XGetPixel( img->xMask, i, j) ) {
-		pix = XGetPixel( img->xImg, i, j);
-		rgbValues( X, pix, &r, &g, &b);
-		pels[j*img->width+i] = (0xff000000 | (r << 16) | (g << 8) | b);
-	  }
-	  else {
-		pels[j*img->width+i] = 0;
+  if ( img->pix && !img->xImg ) {
+	img->xImg = XGetImage( X->dsp, img->pix, 0, 0, img->width, img->height, 0xffffffff, ZPixmap);
+  }
+
+  if ( img->xImg ) {
+	for ( j=0; j<img->height; j++ ) {
+	  for ( i=0; i<img->width; i++ ) {
+		if ( (img->xMask == 0) || XGetPixel( img->xMask, i, j) ) {
+		  pix = XGetPixel( img->xImg, i, j);
+		  rgbValues( X, pix, &r, &g, &b);
+		  pels[j*img->width+i] = (0xff000000 | (r << 16) | (g << 8) | b);
+		}
+		else {
+		  pels[j*img->width+i] = 0;
+		}
 	  }
 	}
   }
+
+  if ( img->pix && img->xImg ) {
+	XDestroyImage( img->xImg);
+	img->xImg = 0;
+  }
+
   if ( isCopy ) {
     (*env)->ReleaseIntArrayElements( env, pelArray, pels, JNI_COMMIT);
   }
@@ -678,7 +665,7 @@ Java_java_awt_Toolkit_imgProduceImage ( JNIEnv* env, jclass clazz, jobject produ
 
 Image *unknownImage = 0;  /* fill in some default image here */
 
-int imageFormat ( unsigned char* sig ) {
+static int imageFormat ( unsigned char* sig ) {
   if ( (sig[0] == 'G') && (sig[1] == 'I') && (sig[2] == 'F') )
 	return SIG_GIF;
 
@@ -695,13 +682,13 @@ void*
 Java_java_awt_Toolkit_imgCreateFromFile ( JNIEnv* env, jclass clazz, jstring fileName )
 {
   Image *img = 0;
-  FILE  *infile;
+  int   infile;
   char  *fn = java2CString( env, X, fileName);
   unsigned char  sig[SIG_LENGTH];
 
-  if ((infile = fopen( fn, "rb")) != NULL) {
-	if ( fread( sig, 1, sizeof(sig), infile) == sizeof(sig) ) {
-	  rewind( infile);  /* some native converters can't skip the signature read */
+  if ( (infile = AWT_OPEN( fn)) >= 0 ) {
+	if ( AWT_READ( infile, sig, sizeof(sig)) == sizeof(sig) ) {
+	  AWT_REWIND( infile);  /* some native converters can't skip the signature read */
 
 	  switch ( imageFormat( sig) ) {
 	  case SIG_GIF:
@@ -717,7 +704,7 @@ Java_java_awt_Toolkit_imgCreateFromFile ( JNIEnv* env, jclass clazz, jstring fil
 		img = unknownImage;
 	  }
 	}
-	fclose( infile);
+	AWT_CLOSE( infile);
   }
 
   return img;
@@ -725,7 +712,7 @@ Java_java_awt_Toolkit_imgCreateFromFile ( JNIEnv* env, jclass clazz, jstring fil
 
 void*
 Java_java_awt_Toolkit_imgCreateFromData ( JNIEnv* env, jclass clazz,
-										  jarray jbuffer, jint off, jint len )
+										  jbyteArray jbuffer, jint off, jint len )
 {
   Image *img = 0;
   jboolean isCopy;
@@ -754,6 +741,24 @@ Java_java_awt_Toolkit_imgCreateFromData ( JNIEnv* env, jclass clazz,
   return img;  
 }
 
+void*
+Java_java_awt_Toolkit_imgSetFrame ( JNIEnv* env, jclass clazz, Image* img, int frameNo )
+{
+  Image *imgCur = img;
+
+  if ( !img->next )
+	return img;
+
+  while ( img->frame != frameNo ) {
+	img = img->next;
+	if ( img == imgCur ) {
+		break;
+	}
+  }
+
+  return img;
+}
+
 
 /************************************************************************************
  * field access
