@@ -14,17 +14,25 @@
 #include "files.h"
 #include "defs.h"
 #include "jsyscall.h"
+#include "kaffe_applet_AudioPlayer.h"
 #include "../../../kaffe/kaffevm/support.h"
 #include "../../../kaffe/kaffevm/stringSupport.h"
 
 void
-kaffe_applet_AudioPlayer_playFile( jstring jstr ) {
+kaffe_applet_AudioPlayer_playFile(struct Hjava_lang_String* jstr)
+{
 	char    fName[MAXPATHLEN];
 	int     bLen = 1024;
-	int     bRead, fin, dev, rc;
+	int     fin, dev, rc;
+	size_t	bRead;
 	void	*buf;
 
 	buf = KMALLOC( bLen);
+	if (!buf) {
+		errorInfo info;
+		postOutOfMemory(&info);
+		throwError(&info);
+	}
 
 	stringJava2CBuf( jstr, fName, sizeof(fName));
 
@@ -38,7 +46,7 @@ kaffe_applet_AudioPlayer_playFile( jstring jstr ) {
 		SignalError("java.io.IOException", SYS_ERROR(rc));
 	}
 
-	while ( (KREAD( fin, buf, bLen, &bRead ) == 0) && (bRead > 0)) {
+	while ( (KREAD( fin, buf, bLen, &bRead ) == 0) && ((ssize_t)bRead > 0)) {
 		ssize_t bWritten;
 		KWRITE( dev, buf, bRead, &bWritten );	/* XXX check error */
 	}
