@@ -16,6 +16,7 @@
 #include "../../../kaffe/kaffevm/baseClasses.h"
 #include "../../../kaffe/kaffevm/itypes.h"
 #include "../../../kaffe/kaffevm/soft.h"
+#include "../../../kaffe/kaffevm/exception.h"
 #include "java_lang_reflect_Array.h"
 #include "java_lang_Boolean.h"
 #include "java_lang_Byte.h"
@@ -60,139 +61,153 @@ java_lang_reflect_Array_get(struct Hjava_lang_Object* obj, jint elem)
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_intClass ) {
+	if ( clazz == intClass ) {
 		HArrayOfInt *arr = (HArrayOfInt *)obj;
 
 #if GUARANA
 		if (mo) {
 			jvalue res;
 			guarana_reify_array_load(mo, obj, elem, &res);
-			return execute_java_constructor(0,javaLangIntegerClass,"(I)V",
+			return execute_java_constructor(0, 0,
+							javaLangIntegerClass,
+							"(I)V",
 							res.i);
 		}
 #endif
 
 		if (elem < 0 || elem >= obj_length(arr))
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
-		
-		return execute_java_constructor(0,javaLangIntegerClass,"(I)V",
-						(jint)(unhand_array(arr)->body[elem]));
+
+		return execute_java_constructor(0, 0, javaLangIntegerClass,
+		    "(I)V", (jint)unhand_array(arr)->body[elem]);
 	}
-	else if ( clazz == _Jv_shortClass ) {
+	else if ( clazz == shortClass ) {
 		HArrayOfShort *arr = (HArrayOfShort *)obj;
 
 #if GUARANA
 		if (mo) {
 			jvalue res;
 			guarana_reify_array_load(mo, obj, elem, &res);
-			return execute_java_constructor(0,javaLangShortClass,"(I)V",
+			return execute_java_constructor(0, 0,
+							javaLangShortClass,
+							"(I)V",
 							res.i);
 		}
 #endif
 
 		if (elem < 0 || elem >= obj_length(arr))
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
-		
-		return execute_java_constructor(0,javaLangShortClass,"(S)V",
-						(jint)(unhand_array(arr)->body[elem]));
+
+		return execute_java_constructor(0, 0, javaLangShortClass,
+		    "(S)V", (jint)unhand_array(arr)->body[elem]);
 	}
-	else if ( clazz == _Jv_longClass ) {
+	else if ( clazz == longClass ) {
 		HArrayOfLong *arr = (HArrayOfLong *)obj;
 
 #if GUARANA
 		if (mo) {
 			jvalue res;
 			guarana_reify_array_load(mo, obj, elem, &res);
-			return execute_java_constructor(0,javaLangLongClass,"(J)V",
+			return execute_java_constructor(0, 0,
+							javaLangLongClass,
+							"(J)V",
 							res.j);
 		}
 #endif
 
 		if (elem < 0 || elem >= obj_length(arr))
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
-		
-		return execute_java_constructor(0,javaLangLongClass,"(J)V",
-						(jlong)(unhand_array(arr)->body[elem]));
+
+		return execute_java_constructor(0, 0, javaLangLongClass,
+		    "(J)V", (jlong)unhand_array(arr)->body[elem]);
 	}
-	else if ( clazz == _Jv_byteClass ) {
+	else if ( clazz == byteClass ) {
 		HArrayOfByte *arr = (HArrayOfByte *)obj;
 
 #if GUARANA
 		if (mo) {
 			jvalue res;
 			guarana_reify_array_load(mo, obj, elem, &res);
-			return execute_java_constructor(0,javaLangByteClass,"(B)V",
+			return execute_java_constructor(0, 0,
+							javaLangByteClass,
+							"(B)V",
 							res.i);
 		}
 #endif
 
 		if (elem < 0 || elem >= obj_length(arr))
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
-		
-		return execute_java_constructor(0,javaLangByteClass,"(B)V",
-						(jint)(unhand_array(arr)->body[elem]));
+
+		return execute_java_constructor(0, 0, javaLangByteClass,
+		    "(B)V", (jint)unhand_array(arr)->body[elem]);
 	}
-	else if ( clazz == _Jv_booleanClass ) {
+	else if ( clazz == booleanClass ) {
 		HArrayOfBoolean *arr = (HArrayOfBoolean *)obj;
 		if (elem < 0 || elem >= obj_length(arr))
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
-		
-		return execute_java_constructor(0,javaLangBooleanClass,"(Z)V",
-						(jint)(unhand_array(arr)->body[elem]));
+
+		return execute_java_constructor(0, 0, javaLangBooleanClass,
+		   "(Z)V", (jint)unhand_array(arr)->body[elem]);
 	}
-	else if ( clazz == _Jv_charClass ) {
+	else if ( clazz == charClass ) {
 		HArrayOfChar *arr = (HArrayOfChar *)obj;
 
 #if GUARANA
 		if (mo) {
 			jvalue res;
 			guarana_reify_array_load(mo, obj, elem, &res);
-			return execute_java_constructor(0,javaLangCharacterClass,"(C)V",
+			return execute_java_constructor(0, 0,
+							javaLangCharacterClass,
+							"(C)V",
 							res.i);
 		}
 #endif
 
 		if (elem < 0 || elem >= obj_length(arr))
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
-		
-		return execute_java_constructor(0,javaLangCharacterClass,"(C)V",
-						(jint)(unhand_array(arr)->body[elem]));
+
+		return execute_java_constructor(0, 0, javaLangCharacterClass,
+		    "(C)V", (jint)unhand_array(arr)->body[elem]);
 	}
-	else if ( clazz == _Jv_floatClass ) {
+	else if ( clazz == floatClass ) {
 		HArrayOfFloat *arr = (HArrayOfFloat *)obj;
 
 #if GUARANA
 		if (mo) {
 			jvalue res;
 			guarana_reify_array_load(mo, obj, elem, &res);
-			return execute_java_constructor(0,javaLangFloatClass,"(F)V",
+			return execute_java_constructor(0, 0,
+							javaLangFloatClass,
+							"(F)V",
 							res.f);
 		}
 #endif
 
 		if (elem < 0 || elem >= obj_length(arr))
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
-		
-		return execute_java_constructor(0,javaLangFloatClass,"(F)V",
-						(jfloat)(unhand_array(arr)->body[elem]));
+
+		return execute_java_constructor(0, 0, javaLangFloatClass,
+		    "(F)V", (jfloat)unhand_array(arr)->body[elem]);
 	}
-	else if ( clazz == _Jv_doubleClass ) {
+	else if ( clazz == doubleClass ) {
 		HArrayOfDouble *arr = (HArrayOfDouble *)obj;
 
 #if GUARANA
 		if (mo) {
 			jvalue res;
 			guarana_reify_array_load(mo, obj, elem, &res);
-			return execute_java_constructor(0,javaLangDoubleClass,"(D)V",
+			return execute_java_constructor(0, 0,
+							javaLangDoubleClass,
+							"(D)V",
 							res.d);
 		}
 #endif
 
 		if (elem < 0 || elem >= obj_length(arr))
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
-		
-		return execute_java_constructor(0,javaLangDoubleClass,"(D)V",
-						(jdouble)(unhand_array(arr)->body[elem]));
+
+		return execute_java_constructor(0, 0, javaLangDoubleClass,
+		    "(D)V", (jdouble)unhand_array(arr)->body[elem]);
 	}
 	else {
 		/* If clazz isn't one of the above then it's either a
@@ -211,7 +226,7 @@ java_lang_reflect_Array_get(struct Hjava_lang_Object* obj, jint elem)
 
 		if (elem < 0 || elem >= obj_length(arr))
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
-		
+
 		return unhand_array(arr)->body[elem];
 	}
 }
@@ -229,7 +244,7 @@ java_lang_reflect_Array_getBoolean(struct Hjava_lang_Object* obj, jint elem)
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_booleanClass ) {
+	if ( clazz == booleanClass ) {
 		HArrayOfBoolean *arr = (HArrayOfBoolean *)obj;
 
 #if GUARANA
@@ -262,7 +277,7 @@ java_lang_reflect_Array_getByte(struct Hjava_lang_Object* obj, jint elem)
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_byteClass ) {
+	if ( clazz == byteClass ) {
 		HArrayOfByte *arr = (HArrayOfByte *)obj;
 
 #if GUARANA
@@ -295,7 +310,7 @@ java_lang_reflect_Array_getChar(struct Hjava_lang_Object* obj, jint elem)
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_charClass ) {
+	if ( clazz == charClass ) {
 		HArrayOfChar *arr = (HArrayOfChar *)obj;
 
 #if GUARANA
@@ -310,7 +325,7 @@ java_lang_reflect_Array_getChar(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_byteClass ) {
+	} else if ( clazz == byteClass ) {
 		HArrayOfByte *arr = (HArrayOfByte *)obj;
 
 #if GUARANA
@@ -343,7 +358,7 @@ java_lang_reflect_Array_getShort(struct Hjava_lang_Object* obj, jint elem)
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_shortClass ) {
+	if ( clazz == shortClass ) {
 		HArrayOfShort *arr = (HArrayOfShort *)obj;
 
 #if GUARANA
@@ -358,7 +373,7 @@ java_lang_reflect_Array_getShort(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_byteClass ) {
+	} else if ( clazz == byteClass ) {
 		HArrayOfByte *arr = (HArrayOfByte *)obj;
 
 #if GUARANA
@@ -391,7 +406,7 @@ java_lang_reflect_Array_getInt(struct Hjava_lang_Object* obj, jint elem)
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_intClass ) {
+	if ( clazz == intClass ) {
 		HArrayOfInt *arr = (HArrayOfInt *)obj;
 
 #if GUARANA
@@ -406,7 +421,7 @@ java_lang_reflect_Array_getInt(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_shortClass ) {
+	} else if ( clazz == shortClass ) {
 		HArrayOfShort *arr = (HArrayOfShort *)obj;
 
 #if GUARANA
@@ -421,7 +436,7 @@ java_lang_reflect_Array_getInt(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_charClass ) {
+	} else if ( clazz == charClass ) {
 		HArrayOfChar *arr = (HArrayOfChar *)obj;
 
 #if GUARANA
@@ -436,7 +451,7 @@ java_lang_reflect_Array_getInt(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_byteClass ) {
+	} else if ( clazz == byteClass ) {
 		HArrayOfByte *arr = (HArrayOfByte *)obj;
 
 #if GUARANA
@@ -469,7 +484,7 @@ java_lang_reflect_Array_getLong(struct Hjava_lang_Object* obj, jint elem)
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_longClass ) {
+	if ( clazz == longClass ) {
 		HArrayOfLong *arr = (HArrayOfLong *)obj;
 
 #if GUARANA
@@ -484,7 +499,7 @@ java_lang_reflect_Array_getLong(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_intClass ) {
+	} else if ( clazz == intClass ) {
 		HArrayOfInt *arr = (HArrayOfInt *)obj;
 
 #if GUARANA
@@ -499,7 +514,7 @@ java_lang_reflect_Array_getLong(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_shortClass ) {
+	} else if ( clazz == shortClass ) {
 		HArrayOfShort *arr = (HArrayOfShort *)obj;
 
 #if GUARANA
@@ -514,7 +529,7 @@ java_lang_reflect_Array_getLong(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_charClass ) {
+	} else if ( clazz == charClass ) {
 		HArrayOfChar *arr = (HArrayOfChar *)obj;
 
 #if GUARANA
@@ -529,7 +544,7 @@ java_lang_reflect_Array_getLong(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_byteClass ) {
+	} else if ( clazz == byteClass ) {
 		HArrayOfByte *arr = (HArrayOfByte *)obj;
 
 #if GUARANA
@@ -562,7 +577,7 @@ java_lang_reflect_Array_getFloat(struct Hjava_lang_Object* obj, jint elem)
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_floatClass ) {
+	if ( clazz == floatClass ) {
 		HArrayOfFloat *arr = (HArrayOfFloat *)obj;
 
 #if GUARANA
@@ -577,7 +592,7 @@ java_lang_reflect_Array_getFloat(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_longClass ) {
+	} else if ( clazz == longClass ) {
 		HArrayOfLong *arr = (HArrayOfLong *)obj;
 
 #if GUARANA
@@ -592,7 +607,7 @@ java_lang_reflect_Array_getFloat(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_intClass ) {
+	} else if ( clazz == intClass ) {
 		HArrayOfInt *arr = (HArrayOfInt *)obj;
 
 #if GUARANA
@@ -607,7 +622,7 @@ java_lang_reflect_Array_getFloat(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_shortClass ) {
+	} else if ( clazz == shortClass ) {
 		HArrayOfShort *arr = (HArrayOfShort *)obj;
 
 #if GUARANA
@@ -622,7 +637,7 @@ java_lang_reflect_Array_getFloat(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_charClass ) {
+	} else if ( clazz == charClass ) {
 		HArrayOfChar *arr = (HArrayOfChar *)obj;
 
 #if GUARANA
@@ -637,7 +652,7 @@ java_lang_reflect_Array_getFloat(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_byteClass ) {
+	} else if ( clazz == byteClass ) {
 		HArrayOfByte *arr = (HArrayOfByte *)obj;
 
 #if GUARANA
@@ -670,7 +685,7 @@ java_lang_reflect_Array_getDouble(struct Hjava_lang_Object* obj, jint elem)
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_doubleClass ) {
+	if ( clazz == doubleClass ) {
 		HArrayOfDouble *arr = (HArrayOfDouble *)obj;
 
 #if GUARANA
@@ -685,7 +700,7 @@ java_lang_reflect_Array_getDouble(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_floatClass ) {
+	} else if ( clazz == floatClass ) {
 		HArrayOfFloat *arr = (HArrayOfFloat *)obj;
 
 #if GUARANA
@@ -700,7 +715,7 @@ java_lang_reflect_Array_getDouble(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_longClass ) {
+	} else if ( clazz == longClass ) {
 		HArrayOfLong *arr = (HArrayOfLong *)obj;
 
 #if GUARANA
@@ -715,7 +730,7 @@ java_lang_reflect_Array_getDouble(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_intClass ) {
+	} else if ( clazz == intClass ) {
 		HArrayOfInt *arr = (HArrayOfInt *)obj;
 
 #if GUARANA
@@ -730,7 +745,7 @@ java_lang_reflect_Array_getDouble(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_shortClass ) {
+	} else if ( clazz == shortClass ) {
 		HArrayOfShort *arr = (HArrayOfShort *)obj;
 
 #if GUARANA
@@ -745,7 +760,7 @@ java_lang_reflect_Array_getDouble(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_charClass ) {
+	} else if ( clazz == charClass ) {
 		HArrayOfChar *arr = (HArrayOfChar *)obj;
 
 #if GUARANA
@@ -760,7 +775,7 @@ java_lang_reflect_Array_getDouble(struct Hjava_lang_Object* obj, jint elem)
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		return unhand_array(arr)->body[elem];
-	} else if ( clazz == _Jv_byteClass ) {
+	} else if ( clazz == byteClass ) {
 		HArrayOfByte *arr = (HArrayOfByte *)obj;
 
 #if GUARANA
@@ -856,7 +871,7 @@ java_lang_reflect_Array_setBoolean(struct Hjava_lang_Object* obj, jint elem, jbo
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_booleanClass ) {
+	if ( clazz == booleanClass ) {
 		HArrayOfBoolean *arr = (HArrayOfBoolean *)obj;
 
 #if GUARANA
@@ -890,7 +905,7 @@ java_lang_reflect_Array_setByte(struct Hjava_lang_Object* obj, jint elem, jbyte
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_byteClass ) {
+	if ( clazz == byteClass ) {
 		HArrayOfByte *arr = (HArrayOfByte *)obj;
 
 #if GUARANA
@@ -906,7 +921,7 @@ java_lang_reflect_Array_setByte(struct Hjava_lang_Object* obj, jint elem, jbyte
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_shortClass ) {
+	} else if ( clazz == shortClass ) {
 		HArrayOfShort *arr = (HArrayOfShort *)obj;
 
 #if GUARANA
@@ -922,7 +937,7 @@ java_lang_reflect_Array_setByte(struct Hjava_lang_Object* obj, jint elem, jbyte
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_charClass ) {
+	} else if ( clazz == charClass ) {
 		HArrayOfChar *arr = (HArrayOfChar *)obj;
 
 #if GUARANA
@@ -938,7 +953,7 @@ java_lang_reflect_Array_setByte(struct Hjava_lang_Object* obj, jint elem, jbyte
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_intClass ) {
+	} else if ( clazz == intClass ) {
 		HArrayOfInt *arr = (HArrayOfInt *)obj;
 
 #if GUARANA
@@ -954,7 +969,7 @@ java_lang_reflect_Array_setByte(struct Hjava_lang_Object* obj, jint elem, jbyte
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_longClass ) {
+	} else if ( clazz == longClass ) {
 		HArrayOfLong *arr = (HArrayOfLong *)obj;
 
 #if GUARANA
@@ -970,7 +985,7 @@ java_lang_reflect_Array_setByte(struct Hjava_lang_Object* obj, jint elem, jbyte
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_floatClass ) {
+	} else if ( clazz == floatClass ) {
 		HArrayOfFloat *arr = (HArrayOfFloat *)obj;
 
 #if GUARANA
@@ -986,7 +1001,7 @@ java_lang_reflect_Array_setByte(struct Hjava_lang_Object* obj, jint elem, jbyte
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_doubleClass ) {
+	} else if ( clazz == doubleClass ) {
 		HArrayOfDouble *arr = (HArrayOfDouble *)obj;
 
 #if GUARANA
@@ -1020,7 +1035,7 @@ java_lang_reflect_Array_setChar(struct Hjava_lang_Object* obj, jint elem, jchar
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_charClass ) {
+	if ( clazz == charClass ) {
 		HArrayOfChar *arr = (HArrayOfChar *)obj;
 
 #if GUARANA
@@ -1036,7 +1051,7 @@ java_lang_reflect_Array_setChar(struct Hjava_lang_Object* obj, jint elem, jchar
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_intClass ) {
+	} else if ( clazz == intClass ) {
 		HArrayOfInt *arr = (HArrayOfInt *)obj;
 
 #if GUARANA
@@ -1052,7 +1067,7 @@ java_lang_reflect_Array_setChar(struct Hjava_lang_Object* obj, jint elem, jchar
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_longClass ) {
+	} else if ( clazz == longClass ) {
 		HArrayOfLong *arr = (HArrayOfLong *)obj;
 
 #if GUARANA
@@ -1068,7 +1083,7 @@ java_lang_reflect_Array_setChar(struct Hjava_lang_Object* obj, jint elem, jchar
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_floatClass ) {
+	} else if ( clazz == floatClass ) {
 		HArrayOfFloat *arr = (HArrayOfFloat *)obj;
 
 #if GUARANA
@@ -1084,7 +1099,7 @@ java_lang_reflect_Array_setChar(struct Hjava_lang_Object* obj, jint elem, jchar
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_doubleClass ) {
+	} else if ( clazz == doubleClass ) {
 		HArrayOfDouble *arr = (HArrayOfDouble *)obj;
 
 #if GUARANA
@@ -1118,7 +1133,7 @@ java_lang_reflect_Array_setShort(struct Hjava_lang_Object* obj, jint elem, jshor
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_shortClass ) {
+	if ( clazz == shortClass ) {
 		HArrayOfShort *arr = (HArrayOfShort *)obj;
 
 #if GUARANA
@@ -1134,7 +1149,7 @@ java_lang_reflect_Array_setShort(struct Hjava_lang_Object* obj, jint elem, jshor
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_intClass ) {
+	} else if ( clazz == intClass ) {
 		HArrayOfInt *arr = (HArrayOfInt *)obj;
 
 #if GUARANA
@@ -1150,7 +1165,7 @@ java_lang_reflect_Array_setShort(struct Hjava_lang_Object* obj, jint elem, jshor
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_longClass ) {
+	} else if ( clazz == longClass ) {
 		HArrayOfLong *arr = (HArrayOfLong *)obj;
 
 #if GUARANA
@@ -1166,7 +1181,7 @@ java_lang_reflect_Array_setShort(struct Hjava_lang_Object* obj, jint elem, jshor
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_floatClass ) {
+	} else if ( clazz == floatClass ) {
 		HArrayOfFloat *arr = (HArrayOfFloat *)obj;
 
 #if GUARANA
@@ -1182,7 +1197,7 @@ java_lang_reflect_Array_setShort(struct Hjava_lang_Object* obj, jint elem, jshor
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_doubleClass ) {
+	} else if ( clazz == doubleClass ) {
 		HArrayOfDouble *arr = (HArrayOfDouble *)obj;
 
 #if GUARANA
@@ -1216,7 +1231,7 @@ java_lang_reflect_Array_setInt(struct Hjava_lang_Object* obj, jint elem, jint va
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_intClass ) {
+	if ( clazz == intClass ) {
 		HArrayOfInt *arr = (HArrayOfInt *)obj;
 
 #if GUARANA
@@ -1232,7 +1247,7 @@ java_lang_reflect_Array_setInt(struct Hjava_lang_Object* obj, jint elem, jint va
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_longClass ) {
+	} else if ( clazz == longClass ) {
 		HArrayOfLong *arr = (HArrayOfLong *)obj;
 
 #if GUARANA
@@ -1248,7 +1263,7 @@ java_lang_reflect_Array_setInt(struct Hjava_lang_Object* obj, jint elem, jint va
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_floatClass ) {
+	} else if ( clazz == floatClass ) {
 		HArrayOfFloat *arr = (HArrayOfFloat *)obj;
 
 #if GUARANA
@@ -1264,7 +1279,7 @@ java_lang_reflect_Array_setInt(struct Hjava_lang_Object* obj, jint elem, jint va
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_doubleClass ) {
+	} else if ( clazz == doubleClass ) {
 		HArrayOfDouble *arr = (HArrayOfDouble *)obj;
 
 #if GUARANA
@@ -1298,7 +1313,7 @@ java_lang_reflect_Array_setLong(struct Hjava_lang_Object* obj, jint elem, jlong
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_longClass ) {
+	if ( clazz == longClass ) {
 		HArrayOfLong *arr = (HArrayOfLong *)obj;
 
 #if GUARANA
@@ -1314,7 +1329,7 @@ java_lang_reflect_Array_setLong(struct Hjava_lang_Object* obj, jint elem, jlong
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_floatClass ) {
+	} else if ( clazz == floatClass ) {
 		HArrayOfFloat *arr = (HArrayOfFloat *)obj;
 
 #if GUARANA
@@ -1330,7 +1345,7 @@ java_lang_reflect_Array_setLong(struct Hjava_lang_Object* obj, jint elem, jlong
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_doubleClass ) {
+	} else if ( clazz == doubleClass ) {
 		HArrayOfDouble *arr = (HArrayOfDouble *)obj;
 
 #if GUARANA
@@ -1364,7 +1379,7 @@ java_lang_reflect_Array_setFloat(struct Hjava_lang_Object* obj, jint elem, jfloa
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_floatClass ) {
+	if ( clazz == floatClass ) {
 		HArrayOfFloat *arr = (HArrayOfFloat *)obj;
 
 #if GUARANA
@@ -1380,7 +1395,7 @@ java_lang_reflect_Array_setFloat(struct Hjava_lang_Object* obj, jint elem, jfloa
 			SignalError("java.lang.ArrayIndexOutOfBoundsException", "");
 		
 		unhand_array(arr)->body[elem] = val;
-	} else if ( clazz == _Jv_doubleClass ) {
+	} else if ( clazz == doubleClass ) {
 		HArrayOfDouble *arr = (HArrayOfDouble *)obj;
 
 #if GUARANA
@@ -1414,7 +1429,7 @@ java_lang_reflect_Array_setDouble(struct Hjava_lang_Object* obj, jint elem, jdou
 
 	clazz = CLASS_ELEMENT_TYPE(OBJECT_CLASS(obj));
 	
-	if ( clazz == _Jv_doubleClass ) {
+	if ( clazz == doubleClass ) {
 		HArrayOfDouble *arr = (HArrayOfDouble *)obj;
 
 #if GUARANA
@@ -1452,6 +1467,7 @@ java_lang_reflect_Array_multiNewArray(struct Hjava_lang_Class* clazz, HArrayOfIn
 	int i;
 	int s;
 	Hjava_lang_Object* array;
+	errorInfo info;
 
 	s = obj_length(sizes);
 
@@ -1460,6 +1476,10 @@ java_lang_reflect_Array_multiNewArray(struct Hjava_lang_Class* clazz, HArrayOfIn
 	}
 
 	dims = KCALLOC(s+1, sizeof(int));
+	if (!dims) {
+		postOutOfMemory(&info);
+		throwError(&info);
+	}
 
 	/* Copy dimentions into array */
 	for( i = 0; i < s; i++ ) {
@@ -1467,7 +1487,11 @@ java_lang_reflect_Array_multiNewArray(struct Hjava_lang_Class* clazz, HArrayOfIn
 		if (dims[i] < 0) {
 			SignalError("java.lang.NegativeArraySizeException", "");
 		}
-		clazz = lookupArray(clazz);
+		clazz = lookupArray(clazz, &info);
+		if (clazz == 0) {
+			KFREE(dims);
+			throwError(&info);
+		}
 	}
 	dims[i] = -1;
 
