@@ -19,7 +19,6 @@
 #include "../../../kaffe/kaffevm/itypes.h"
 #include "../../../kaffe/kaffevm/support.h"
 #include "../../../kaffe/kaffevm/stringSupport.h"
-#include "../../../kaffe/kaffevm/errors.h"
 #include "../../../kaffe/kaffevm/soft.h"
 #include "../../../kaffe/kaffevm/stackTrace.h"
 #include "../../../kaffe/kaffevm/baseClasses.h"
@@ -30,7 +29,7 @@
 #include "java_lang_reflect_Constructor.h"
 #include "java_lang_reflect_Method.h"
 #include "java_lang_reflect_Field.h"
-#include <native.h>
+#include "java_lang_Class.h"
 #include "defs.h"
 
 extern Hjava_lang_Object* buildStackTrace(struct _exceptionFrame*);
@@ -45,11 +44,27 @@ java_lang_Class_forName(struct Hjava_lang_String* str, jbool doinit,
 	errorInfo einfo;
 	Hjava_lang_Class* clazz;
 	Utf8Const *utf8buf;
-	char buf[MAXNAMELEN];
+	const char *buf;
+	int jlen;
+	jchar *js;
 
-	/* Get string and convert '.' to '/' */
-	stringJava2CBuf(str, buf, sizeof(buf));
-	classname2pathname(buf, buf);
+	/*
+	 * NB: internally, we store class names as path names (with slashes
+	 *     instead of dots.  However, we must also prevent calls to
+	 *     "java/lang/Object" or "[[Ljava/lang/Object;" from succeeding.
+	 *	Since class names cannot have slashes, we reject all attempts
+	 *	to look up names that do.  Awkward.  Inefficient.
+	 */
+	js = STRING_DATA(str);
+	jlen = STRING_SIZE(str);
+	while (--jlen > 0) {
+		if (*js++ == '/') {
+			postExceptionMessage(&einfo, 
+				JAVA_LANG(ClassNotFoundException), 
+				"Cannot have slashes - use dots instead.");
+			throwError(&einfo);
+		}
+	}
 
 	/*
 	 * Note the following oddity: 
@@ -68,7 +83,11 @@ java_lang_Class_forName(struct Hjava_lang_String* str, jbool doinit,
 	 * array names (those starting with an [), and this is what calling
 	 * loadArray does.
 	 */
-	utf8buf = utf8ConstNew(buf, -1);
+
+	/* Convert string to utf8, converting '.' to '/' */
+	utf8buf = checkPtr(stringJava2Utf8ConstReplace(str, '.', '/'));
+	buf = utf8buf->data;
+
 	if (buf[0] == '[') {
 		clazz = loadArray(utf8buf, loader, &einfo);
 	}
@@ -93,7 +112,7 @@ java_lang_Class_forName(struct Hjava_lang_String* str, jbool doinit,
 		 * NB: 1.2 seems to be more consistent and throws 
 		 * ClassNotFoundException in most cases.
 		 */
-		if (!strcmp(einfo.classname, "java.lang.VerifyError")) 
+		if ((einfo.type & KERR_EXCEPTION) && !strcmp(einfo.classname, "java.lang.VerifyError")) 
 		{
 			errorInfo einfo_copy = einfo;
 			postExceptionMessage(&einfo, 
@@ -101,16 +120,16 @@ java_lang_Class_forName(struct Hjava_lang_String* str, jbool doinit,
 				einfo.mess);
 			discardErrorInfo(&einfo_copy);
 		} else
-		if (!strcmp(einfo.classname, "java.lang.NoClassDefFoundError"))
+		if ((einfo.type & KERR_EXCEPTION) && !strcmp(einfo.classname, "java.lang.NoClassDefFoundError"))
 		{
 			/*
 			 * However, we don't upgrade if it is a second attempt
 			 * to load a class whose loading has already failed.
 			 */
 			classEntry* centry;
-			centry = lookupClassEntry(utf8buf, loader);
-			if (centry->class &&
-			    centry->class->state == CSTATE_FAILED) {
+			centry = lookupClassEntry(utf8buf, loader, &einfo);
+			if (centry == 0 || (centry->class &&
+			    centry->class->state == CSTATE_FAILED)) {
 				utf8ConstRelease(utf8buf);
 				throwError(&einfo);
 			}
@@ -148,57 +167,13 @@ java_lang_Class_forName(struct Hjava_lang_String* str, jbool doinit,
 	return (clazz);
 }
 
-struct Hjava_lang_Class*
-java_lang_Class_forName0(struct Hjava_lang_String* str)
-{
-        int i;
-	Hjava_lang_ClassLoader* loader;
-	stackTraceInfo* info;
-	static Method * cfnmeth;
-
-	if (!cfnmeth) {
-		Utf8Const *fname = utf8ConstNew("forName", -1);
-		Utf8Const *fsig = utf8ConstNew("(Ljava/lang/String;)Ljava/lang/Class;", -1);
-		cfnmeth = findMethodLocal(ClassClass, fname, fsig);
-		utf8ConstRelease(fname);
-		utf8ConstRelease(fsig);
-	}
-
-	if (cfnmeth == 0) {
-		abort();
-	}
-
-        /*
-         * If the calling method is in a class that was loaded by a class
-         * loader, use that class loader to find the class corresponding to
-         * the name.  Otherwise, use the system class loader.
-         */
-        loader = 0;
-        info = (stackTraceInfo*)buildStackTrace(0);
-        for (i = 0; info[i].meth != ENDOFSTACK; i++) {
-                info[i].meth = stacktraceFindMethod(&info[i]);
-		/* skip java.lang.Class.forName(String) cause that's
-		 * who called us
-		 */
-		if (info[i].meth == cfnmeth) {
-			continue;
-		}
-
-                if (info[i].meth != 0 && info[i].meth->class != 0) {
-                        loader = info[i].meth->class->loader;
-                        break;
-                }
-        }
-	return (java_lang_Class_forName(str, true, loader));
-}
-
 /*
  * Convert class to string name.
  */
 struct Hjava_lang_String*
 java_lang_Class_getName(struct Hjava_lang_Class* c)
 {
-	return(utf8Const2JavaReplace(c->name, '/', '.'));
+	return(checkPtr(utf8Const2JavaReplace(c->name, '/', '.')));
 }
 
 /*
@@ -211,7 +186,7 @@ java_lang_Class_newInstance(struct Hjava_lang_Class* this)
 		SignalError("java.lang.InstantiationException", 
 			    CLASS_CNAME(this));
 	}
-	return (execute_java_constructor(0, this, "()V"));
+	return (execute_java_constructor(0, 0, this, "()V"));
 }
 
 /*
@@ -254,7 +229,7 @@ java_lang_Class_getInterfaces(struct Hjava_lang_Class* this)
 	}
 #endif
 
-	obj = (HArrayOfObject*)AllocObjectArray(nr, "Ljava/lang/Class");
+	obj = (HArrayOfObject*)AllocObjectArray(nr, "Ljava/lang/Class;", 0);
 	ifaces = (struct Hjava_lang_Class**)unhand_array(obj)->body;
 	for (i = 0; i < nr; i++) {
 		ifaces[i] = this->interfaces[i];
@@ -322,26 +297,26 @@ java_lang_Class_getPrimitiveClass(struct Hjava_lang_String* name)
 	switch (chrs[0]) {
 	case 'b':
 		if (chrs[1] == 'y') {
-			return (_Jv_byteClass);
+			return (byteClass);
 		}
 		if (chrs[1] == 'o') {
-			return (_Jv_booleanClass);
+			return (booleanClass);
 		}
 		break;
 	case 'c':
-		return (_Jv_charClass);
+		return (charClass);
 	case 'd':
-		return (_Jv_doubleClass);
+		return (doubleClass);
 	case 'f':
-		return (_Jv_floatClass);
+		return (floatClass);
 	case 'i':
-		return (_Jv_intClass);
+		return (intClass);
 	case 'l':
-		return (_Jv_longClass);
+		return (longClass);
 	case 's':
-		return (_Jv_shortClass);
+		return (shortClass);
 	case 'v':
-		return (_Jv_voidClass);
+		return (voidClass);
 	}
 	return(NULL);
 }
@@ -373,34 +348,19 @@ java_lang_Class_setSigners(struct Hjava_lang_Class* this, HArrayOfObject* sigs)
 	unimp("java.lang.Class:setSigners unimplemented");
 }
 
-static
-HArrayOfObject*
+static HArrayOfObject*
 makeParameters(Method* meth)
 {
 	int i;
-	int len;
-	const char* sig;
 	HArrayOfObject* array;
-	Hjava_lang_Class* clazz;
 	errorInfo info;
+	Hjava_lang_Class* clazz;
 
-	sig = meth->signature->data;
-	len = 0;
-	for (sig++; *sig != ')'; sig++, len++) {
-		while (*sig == '[') {
-			sig++;
-		}
-		if (*sig == 'L') {
-			while (*sig != ';') {
-				sig++;
-			}
-		}
-	}
-
-	array = (HArrayOfObject*)AllocObjectArray(len, "Ljava/lang/Class;");
-	sig = meth->signature->data + 1;	/* Skip leading '(' */
-	for (i = 0; i < len; i++) {
-		clazz = classFromSig(&sig, meth->class->loader, &info);
+	array = (HArrayOfObject*)AllocObjectArray(METHOD_NARGS(meth),
+	    "Ljava/lang/Class;", 0);
+	for (i = 0; i < METHOD_NARGS(meth); ++i) {
+		clazz = getClassFromSignaturePart(METHOD_ARG_TYPE(meth, i),
+					      meth->class->loader, &info);
 		if (clazz == 0) {
 			throwError(&info);
 		}
@@ -410,18 +370,13 @@ makeParameters(Method* meth)
         return (array);
 }
 
-static
-Hjava_lang_Class*
+static Hjava_lang_Class*
 makeReturn(Method* meth)
 {
-	const char* sig;
 	errorInfo info;
 	Hjava_lang_Class* clazz;
 
-	/* Skip to end of signature to find return type */
-	sig = strchr(meth->signature->data, ')') + 1;
-
-	clazz = classFromSig(&sig, meth->class->loader, &info);
+	clazz = getClassFromSignaturePart(METHOD_RET_TYPE(meth), meth->class->loader, &info);
 	if (clazz == 0) {
 		throwError(&info);
 	}
@@ -435,8 +390,7 @@ makeReturn(Method* meth)
  *
  * We do not bother to cache the resolved types here.
  */
-static
-HArrayOfObject*
+static HArrayOfObject*
 makeExceptions(Method* meth)
 {
 	int nr;
@@ -445,7 +399,7 @@ makeExceptions(Method* meth)
 	Hjava_lang_Class** ptr;
 
 	nr = meth->ndeclared_exceptions;
-	array = (HArrayOfObject*)AllocObjectArray(nr, "Ljava/lang/Class;");
+	array = (HArrayOfObject*)AllocObjectArray(nr, "Ljava/lang/Class;", 0);
 	ptr = (Hjava_lang_Class**)&unhand_array(array)->body[0];
 	for (i = 0; i < nr; i++) {
 		errorInfo info;
@@ -468,7 +422,8 @@ makeConstructor(struct Hjava_lang_Class* clazz, int slot)
 	Method* mth;
 
 	mth = CLASS_METHODS(clazz) + slot;
-	meth = (Hjava_lang_reflect_Constructor*)AllocObject("java/lang/reflect/Constructor");      
+	meth = (Hjava_lang_reflect_Constructor*)
+	    AllocObject("java/lang/reflect/Constructor", 0);      
 
 	unhand(meth)->clazz = clazz;
 	unhand(meth)->slot = slot;
@@ -486,11 +441,12 @@ makeMethod(struct Hjava_lang_Class* clazz, int slot)
 	Method* mth;
 
 	mth = CLASS_METHODS(clazz) + slot;
-	meth = (Hjava_lang_reflect_Method*)AllocObject("java/lang/reflect/Method");      
+	meth = (Hjava_lang_reflect_Method*)
+	    AllocObject("java/lang/reflect/Method", 0);      
 
 	unhand(meth)->clazz = clazz;
 	unhand(meth)->slot = slot;
-	unhand(meth)->name = utf8Const2Java(mth->name);
+	unhand(meth)->name = checkPtr(utf8Const2Java(mth->name));
 	unhand(meth)->parameterTypes = makeParameters(mth);
 	unhand(meth)->exceptionTypes = makeExceptions(mth);
 	unhand(meth)->returnType = makeReturn(mth);
@@ -507,22 +463,19 @@ makeField(struct Hjava_lang_Class* clazz, int slot)
 	errorInfo info;
 
 	fld = CLASS_FIELDS(clazz) + slot;
-	field = (Hjava_lang_reflect_Field*)AllocObject("java/lang/reflect/Field");
+	field = (Hjava_lang_reflect_Field*)
+	    AllocObject("java/lang/reflect/Field", 0);
 	unhand(field)->clazz = clazz;
 	unhand(field)->slot = slot;
 	unhand(field)->type = resolveFieldType(fld, clazz, &info);
 	if (unhand(field)->type == 0) {
 		throwError(&info);
 	}
-	unhand(field)->name = utf8Const2Java(fld->name);
+	unhand(field)->name = checkPtr(utf8Const2Java(fld->name));
 	return (field);
 }
  
 #if GUARANA
-extern HArrayOfObject* (*p_makeParameters)(Method *meth);
-
-extern Hjava_lang_Class* (*p_makeReturn)(Method *meth);
-
 extern Hjava_lang_reflect_Constructor* (*p_makeConstructor)(struct Hjava_lang_Class* clazz, int slot);
 
 extern Hjava_lang_reflect_Method* (*p_makeMethod)(struct Hjava_lang_Class* clazz, int slot);
@@ -537,8 +490,6 @@ extern Hjava_lang_String* (*p_getClassName)(struct Hjava_lang_Class *clazz);
    Guarana.  */
 void 
 kaffe_guarana_init_function_pointers(void) {
-	p_makeParameters = makeParameters;
-	p_makeReturn = makeReturn;
 	p_makeConstructor = makeConstructor;
 	p_makeMethod = makeMethod;
 	p_makeField = makeField;
@@ -546,19 +497,49 @@ kaffe_guarana_init_function_pointers(void) {
 }
 #endif
 
+/* 
+ * Return true if there is method defined in any subclass of 'cls'
+ * that overrides 'meth'.
+ *
+ * Assumes that base is a subclass of cls
+ */
+static int
+isOverridden(Hjava_lang_Class *base, Hjava_lang_Class *cls, Method *meth)
+{
+	/* XXX for interfaces for now */
+	if (base == 0) 
+		return (false);
+
+	/* Search superclasses for equivalent method name.
+	 * If found extract its index nr.
+	 */
+	for (; base != cls;  base = base->superclass) {
+		int j = CLASS_NMETHODS(base);
+		Method* mt = CLASS_METHODS(base);
+		for (; --j >= 0;  ++mt) {
+			if (utf8ConstEqual (mt->name, meth->name) &&
+			    utf8ConstEqual (METHOD_SIG(mt), METHOD_SIG(meth)))
+			{
+				return (true);
+			}
+		}
+	}
+	return (false);
+}
+
 /*
  * count the number of methods in a class that are not constructors.
  * If declared is not set, count only public methods.
  */
 static int
-countMethods(struct Hjava_lang_Class* clas, jint declared)
+countMethods(Hjava_lang_Class* base, Hjava_lang_Class* clas, jint declared)
 {
 	Method* mth = CLASS_METHODS(clas);
 	int i;
 	int count = 0;
 
 	for (i = CLASS_NMETHODS(clas)-1 ; i >= 0; i--) {
-		if (((mth[i].accflags & ACC_PUBLIC) || declared) && !(mth[i].accflags & ACC_CONSTRUCTOR)) {
+		if (((mth[i].accflags & ACC_PUBLIC) || declared) && !(mth[i].accflags & ACC_CONSTRUCTOR) && !isOverridden(base, clas, mth + i)) {
 			count++;
 		}
 	}
@@ -570,14 +551,14 @@ countMethods(struct Hjava_lang_Class* clas, jint declared)
  * not constructors.  If declared is not set, include only public methods.
  */
 static void
-addMethods(struct Hjava_lang_Class* clas, jint declared, 
+addMethods(Hjava_lang_Class* base, Hjava_lang_Class* clas, jint declared, 
 	Hjava_lang_reflect_Method*** ptr)
 {
 	Method* mth = CLASS_METHODS(clas);
 	int i;
 
 	for (i = CLASS_NMETHODS(clas)-1; i >= 0; i--) {
-		if (((mth[i].accflags & ACC_PUBLIC) || declared) && !(mth[i].accflags & ACC_CONSTRUCTOR)) {
+		if (((mth[i].accflags & ACC_PUBLIC) || declared) && !(mth[i].accflags & ACC_CONSTRUCTOR) && !isOverridden(base, clas, mth + i)) {
 			**ptr = makeMethod(clas, i);
 			(*ptr)++;
 		}
@@ -607,20 +588,21 @@ getInterfaceMethods0(struct Hjava_lang_Class* this, jint declared)
 	int i;
 
 	count = 0;
-	count += countMethods(this, declared);
+	count += countMethods(0, this, declared);
 	if (!declared) {
 		for (i = 0; i < this->total_interface_len; i++) {
-			count += countMethods(this->interfaces[i], declared);
+			count += countMethods(0, this->interfaces[i], declared);
 		}
 	}
 
-	array = (HArrayOfObject*)AllocObjectArray(count, "Ljava/lang/reflect/Method;");
+	array = (HArrayOfObject*)
+	    AllocObjectArray(count, "Ljava/lang/reflect/Method;", 0);
 	ptr = (Hjava_lang_reflect_Method**)&unhand_array(array)->body[0];
 
-	addMethods(this, declared, &ptr);
+	addMethods(0, this, declared, &ptr);
 	if (!declared) {
 		for (i = 0; i < this->total_interface_len; i++) {
-			addMethods(this->interfaces[i], declared, &ptr);
+			addMethods(0, this->interfaces[i], declared, &ptr);
 		}
 	}
 
@@ -628,7 +610,7 @@ getInterfaceMethods0(struct Hjava_lang_Class* this, jint declared)
 }
 
 HArrayOfObject*
-java_lang_Class_getMethods0(struct Hjava_lang_Class* this, jint declared)
+java_lang_Class_getMethods0(struct Hjava_lang_Class* this, jboolean declared)
 {
 	int count;
 	Hjava_lang_Class* clas;
@@ -648,17 +630,18 @@ java_lang_Class_getMethods0(struct Hjava_lang_Class* this, jint declared)
 	count = 0;
 	for (clas = this; clas != NULL; clas = clas->superclass) {
 
-		count += countMethods(clas, declared);
+		count += countMethods(this, clas, declared);
 
 		if (declared) {
 			break;
 		}
 	}
-	array = (HArrayOfObject*)AllocObjectArray(count, "Ljava/lang/reflect/Method;");
+	array = (HArrayOfObject*)
+	    AllocObjectArray(count, "Ljava/lang/reflect/Method;", 0);
 	ptr = (Hjava_lang_reflect_Method**)&unhand_array(array)->body[0];
 	for (clas = this; clas != NULL; clas = clas->superclass) {
 
-		addMethods(clas, declared, &ptr);
+		addMethods(this, clas, declared, &ptr);
 
 		if (declared) {
 			break;
@@ -669,7 +652,7 @@ java_lang_Class_getMethods0(struct Hjava_lang_Class* this, jint declared)
 }
 
 HArrayOfObject*
-java_lang_Class_getConstructors0(struct Hjava_lang_Class* this, jint declared)
+java_lang_Class_getConstructors0(struct Hjava_lang_Class* this, jboolean declared)
 {
 	int count;
 	Hjava_lang_Class* clas;
@@ -686,7 +669,8 @@ java_lang_Class_getConstructors0(struct Hjava_lang_Class* this, jint declared)
 			count++;
 		}
 	}
-	array = (HArrayOfObject*)AllocObjectArray(count, "Ljava/lang/reflect/Constructor;");
+	array = (HArrayOfObject*)
+	   AllocObjectArray(count, "Ljava/lang/reflect/Constructor;", 0);
 	ptr = (Hjava_lang_reflect_Constructor**)&unhand_array(array)->body[0];
 	clas = this;
 	mth = CLASS_METHODS(clas);
@@ -699,34 +683,41 @@ java_lang_Class_getConstructors0(struct Hjava_lang_Class* this, jint declared)
 	return (array);
 }
 
-HArrayOfObject*
-java_lang_Class_getFields0(struct Hjava_lang_Class* clazz, int declared)
+/*
+ * Count all public fields of a class, including inherited fields and
+ * fields from implemented interfaces
+ */
+static int
+countPublicFields(Hjava_lang_Class* clazz) 
 {
-	int count;
 	Hjava_lang_Class* clas;
-	Field* fld;
-	Hjava_lang_reflect_Field** ptr;
-	HArrayOfObject* array;
-	int i;
+	int i, count;
 
-	if (declared) {
-		count = CLASS_NFIELDS((Hjava_lang_Class*)clazz);
-	}
-	else {
-		count = 0;
-		for (clas = clazz; clas != NULL; clas = clas->superclass) {
-			fld = CLASS_FIELDS(clas);
-			for (i = CLASS_NFIELDS(clas)-1; i >= 0; i--) {
-				if (fld[i].accflags & ACC_PUBLIC) {
-					count++;
-				}
+	count = 0;
+	for (clas = clazz; clas != NULL; clas = clas->superclass) {
+		Field *fld = CLASS_FIELDS(clas);
+		for (i = CLASS_NFIELDS(clas)-1; i >= 0; i--) {
+			if (fld[i].accflags & ACC_PUBLIC) {
+				count++;
 			}
 		}
 	}
-	array = (HArrayOfObject*)AllocObjectArray(count, "Ljava/lang/reflect/Field;");
-	ptr = (Hjava_lang_reflect_Field**)&unhand_array(array)->body[0];
+
+	for (i = 0; i < clazz->total_interface_len; i++) {
+		count += countPublicFields(clazz->interfaces[i]);
+	}
+	return (count);
+}
+
+static void
+makePublicFields(Hjava_lang_Class* clazz, jboolean declared, Hjava_lang_reflect_Field*** pptr)
+{
+	Hjava_lang_Class* clas;
+	Hjava_lang_reflect_Field** ptr = *pptr;
+	int i;
+
 	for (clas = clazz; clas != NULL; clas = clas->superclass) {
-		fld = CLASS_FIELDS(clas);
+		Field *fld = CLASS_FIELDS(clas);
 		for (i = CLASS_NFIELDS(clas)-1; i >= 0;  i--) {
 			if ((fld[i].accflags & ACC_PUBLIC) || declared) {
 				*ptr = makeField(clas, i);
@@ -737,6 +728,40 @@ java_lang_Class_getFields0(struct Hjava_lang_Class* clazz, int declared)
 			break;
 		}
 	}
+	if (!declared) {
+		for (i = 0; i < clazz->total_interface_len; i++) {
+			makePublicFields(clazz->interfaces[i], declared, &ptr);
+		}
+	}
+	*pptr = ptr;
+}
+
+/* 
+ * Below, "declared" means to include a field only if it is directly
+ * declared by that class (and not inherited from a superclass or defined
+ * by an interface the class implements.)  This applies to both private,
+ * protected, and public fields.
+ *
+ * On the other hand, if "declared" is false, we only include public 
+ * fields.  Weird semantics.
+ */
+HArrayOfObject*
+java_lang_Class_getFields0(struct Hjava_lang_Class* clazz, jboolean declared)
+{
+	int count;
+	Hjava_lang_reflect_Field** ptr;
+	HArrayOfObject* array;
+
+	if (declared) {
+		count = CLASS_NFIELDS((Hjava_lang_Class*)clazz);
+	}
+	else {
+		count = countPublicFields(clazz);
+	}
+	array = (HArrayOfObject*)
+	    AllocObjectArray(count, "Ljava/lang/reflect/Field;", 0);
+	ptr = (Hjava_lang_reflect_Field**)&unhand_array(array)->body[0];
+	makePublicFields(clazz, declared, &ptr);
 
 	return (array);
 }
@@ -757,71 +782,96 @@ static
 int
 checkParameters(Method* mth, HArrayOfObject* argtypes)
 {
-	const char *sig = mth->signature->data;
-	int   i;
+	int i;
+	errorInfo info;
 
 	/* The JDK doc says and experimentation shows that a null second 
 	 * parameter to all get(Declared){Method|Constructor} functions
 	 * is treated like passing an empty array "new Class [] {}"
 	 */
-	if ((argtypes == NULL) && sig[0] == '(' && sig[1] == ')') {
+	if ((argtypes == NULL) && METHOD_NARGS(mth) == 0) {
 		return (1);
 	}
 
-	sig++;	/* skip leading '(' */
+	if (ARRAY_SIZE(argtypes) != METHOD_NARGS(mth)) {
+		return (0);
+	}
+
 	for (i = 0; i < ARRAY_SIZE(argtypes); i++) {
 		Hjava_lang_Class* sigclass;
-		errorInfo info;
-
-		/* signature was too short or type doesn't match */
-		if (*sig == 0) {
-			return (0);
-		}
-
-		/* NB: if classFromSig fails, we pretend we don't care. */
-		sigclass = classFromSig(&sig, mth->class->loader, &info);
+		sigclass = getClassFromSignaturePart(METHOD_ARG_TYPE(mth, i), mth->class->loader, &info);
 		if (sigclass == 0) {
 			discardErrorInfo(&info);
+			return 0;
 		}
-		if (sigclass != 
+		if (sigclass !=
 			(struct Hjava_lang_Class *)OBJARRAY_DATA(argtypes)[i]) {
 			return (0);
 		}
 	}
-	/* return false if signature was too long */
-	return ((*sig == ')' ? 1 : 0));
+
+	return (1);
+}
+
+
+static
+Hjava_lang_reflect_Method*
+findMatchingMethod(struct Hjava_lang_Class* clas, 
+		   struct Hjava_lang_String* name, 
+		   HArrayOfObject* arr, jboolean declared)
+{
+	Method* mth = CLASS_METHODS(clas);
+	int n = CLASS_NMETHODS(clas);
+	int i;
+	for (i = 0;  i < n;  ++mth, ++i) {
+		if (((mth->accflags & ACC_PUBLIC) || declared)
+		    && utf8ConstEqualJavaString(mth->name, name)) {
+			if (checkParameters(mth, arr))
+				return (makeMethod(clas, i));
+		}
+	}
+	return (0);
 }
 
 Hjava_lang_reflect_Method*
-java_lang_Class_getMethod0(struct Hjava_lang_Class* this, struct Hjava_lang_String* name, HArrayOfObject* arr, jint declared)
+java_lang_Class_getMethod0(struct Hjava_lang_Class* this, struct Hjava_lang_String* name, HArrayOfObject* arr, jboolean declared)
 {
 	Hjava_lang_Class* clas;
+	Hjava_lang_reflect_Method *rmeth;
 
 	clas = this;
 	do {
-		Method* mth = CLASS_METHODS(clas);
-		int n = CLASS_NMETHODS(clas);
-		int i;
-		for (i = 0;  i < n;  ++mth, ++i) {
-			if (((mth->accflags & ACC_PUBLIC) || declared)
-			    && utf8ConstEqualJavaString(mth->name, name)) {
-				if (checkParameters(mth, arr))
-					return (makeMethod(clas, i));
-			}
+		rmeth = findMatchingMethod(clas, name, arr, declared);
+		if (rmeth != 0) {
+			return (rmeth);
 		}
 		clas = clas->superclass;
 	} while (!declared && clas != NULL);
 
+	/* If the class is an interface, check implemented interfaces as well.
+	 * Those are the interfaces this interface inherited
+	 */
+	if (CLASS_IS_INTERFACE(this)) {
+		int i;
+		for (i = 0; i < this->total_interface_len; i++) {
+			rmeth = findMatchingMethod(this->interfaces[i], 
+						   name, arr, declared);
+			if (rmeth != 0) {
+				return (rmeth);
+			}
+		}
+	}
+
 	/* like SignalError, except that the name of the class that is
 	 * not found becomes the error message 
 	 */
 	throwException((struct Hjava_lang_Throwable*)execute_java_constructor(
-		"java.lang.NoSuchMethodException", 0, 
+		"java.lang.NoSuchMethodException", 0, 0, 
 		"(Ljava/lang/String;)V", name));
 }
 
 struct Hjava_lang_reflect_Constructor*
-java_lang_Class_getConstructor0(struct Hjava_lang_Class* this, HArrayOfObject* arr, jint declared) 
+java_lang_Class_getConstructor0(struct Hjava_lang_Class* this, HArrayOfObject* arr, jboolean declared) 
 {
 	Hjava_lang_Class* clas = this;
 
@@ -838,11 +888,14 @@ java_lang_Class_getConstructor0(struct Hjava_lang_Class* this, HArrayOfObject* a
 	SignalError("java.lang.NoSuchMethodException", ""); /* FIXME */
 }
 
+static
 Hjava_lang_reflect_Field*
-java_lang_Class_getField0(struct Hjava_lang_Class* clazz, struct Hjava_lang_String* name, int declared)
+checkForField(struct Hjava_lang_Class* clazz, struct Hjava_lang_String* name, jboolean declared)
 {
+	int i;
 	Hjava_lang_Class* clas;
 
+	/* first try this class's own or inherited fields */
 	clas = (Hjava_lang_Class*) clazz;
 	do {
 		Field* fld = CLASS_FIELDS(clas);
@@ -856,5 +909,25 @@ java_lang_Class_getField0(struct Hjava_lang_Class* clazz, struct Hjava_lang_Stri
 		}
 		clas = clas->superclass;
 	} while (!declared && clas != NULL);
+
+	/* then try this class's interfaces fields using the wonders of
+	 * recursion.  */
+	for (i = 0; i < clazz->total_interface_len; i++) {
+		Hjava_lang_reflect_Field *f;
+		f = checkForField(clazz->interfaces[i], name, declared);
+		if (f != 0) {
+			return (f);
+		}
+	}
+	return (0);
+}
+
+Hjava_lang_reflect_Field*
+java_lang_Class_getField0(struct Hjava_lang_Class* clazz, struct Hjava_lang_String* name, jboolean declared)
+{
+	Hjava_lang_reflect_Field* f = checkForField(clazz, name, declared);
+	if (f != 0) {
+		return (f);
+	}
 	SignalError("java.lang.NoSuchFieldException", ""); /* FIXME */
 }
