@@ -10,7 +10,7 @@
 
 #include "config.h"
 #include "config-std.h"
-#include <native.h>
+#include "java_text_DateFormat.h"
 #include <time.h>
 
 #if defined(HAVE_STRFTIME)
@@ -42,5 +42,5 @@ java_text_DateFormat_format0(jlong time, int fmt)
 		GMTTIME(str, localtime(&date));
 		break;
 	}
-	return (stringC2Java(str));
+	return (checkPtr(stringC2Java(str)));
 }
