@@ -38,8 +38,7 @@
 #include "defs.h"
 
 #if GUARANA
-extern void guarana_reify_field_load(struct HBR_unicamp_Guarana_MetaObject *mo, struct Hjava_lang_Object *o, struct Hjava_lang_Class *declass, Field *field, jvalue *res);
-extern void guarana_reify_field_store(struct HBR_unicamp_Guarana_MetaObject *mo, struct Hjava_lang_Object *o, struct Hjava_lang_Class *declass, Field *field, jvalue *val);
+#include "../../extensions/guarana/clib/GuaraNative.h"
 #endif
 
 static
@@ -62,7 +61,7 @@ getFieldAddress(Hjava_lang_reflect_Field* this, struct Hjava_lang_Object* obj)
                 if  (!soft_instanceof(clas, obj)) {
                         SignalError("java.lang.IllegalArgumentException","");
                 }
-                return (((char*)(obj)) + FIELD_OFFSET(fld));
+                return (((char*)(obj)) + FIELD_BOFFSET(fld));
         }
 }
 
@@ -103,9 +102,9 @@ java_lang_reflect_Field_get(Hjava_lang_reflect_Field* this, struct Hjava_lang_Ob
 
 	if (mo) {
 	  base = (char*)&result;
-	  guarana_reify_field_load(mo, obj, clas, fld, &result);
+	  guarana_reify_field_load(obj, clas, fld, &result);
 
-	  if (FIELD_ISREF(fld))
+	  if (FIELD_ISPRIM(fld))
 	    switch (CLASS_PRIM_SIG(FIELD_TYPE(fld))) {
 	    case 'Z':
 	      result.z = result.i;
@@ -125,29 +124,37 @@ java_lang_reflect_Field_get(Hjava_lang_reflect_Field* this, struct Hjava_lang_Ob
 #endif
         base = getFieldAddress(this, obj);
 
-	if ( fld->type == _Jv_intClass ) {
-		return execute_java_constructor(0,javaLangIntegerClass,"(I)V",*((jint*)base));
+	if ( fld->type == intClass ) {
+		return execute_java_constructor(0, 0, javaLangIntegerClass,
+		    "(I)V", *((jint*)base));
 	}
-	else if ( fld->type == _Jv_shortClass ) {
-		return execute_java_constructor(0,javaLangShortClass,"(S)V",*(jshort*)base);
+	else if ( fld->type == shortClass ) {
+		return execute_java_constructor(0, 0, javaLangShortClass,
+		    "(S)V", *(jshort*)base);
 	}
-	else if ( fld->type == _Jv_longClass ) {
-		return execute_java_constructor(0,javaLangLongClass,"(J)V",*(jlong*)base);
+	else if ( fld->type == longClass ) {
+		return execute_java_constructor(0, 0, javaLangLongClass,
+		    "(J)V", *(jlong*)base);
 	}
-	else if ( fld->type == _Jv_byteClass ) {
-		return execute_java_constructor(0,javaLangByteClass,"(B)V",*(jbyte*)base);
+	else if ( fld->type == byteClass ) {
+		return execute_java_constructor(0, 0, javaLangByteClass,
+		    "(B)V", *(jbyte*)base);
 	}
-	else if ( fld->type == _Jv_booleanClass ) {
-		return execute_java_constructor(0,javaLangBooleanClass,"(Z)V",*(jbool*)base);
+	else if ( fld->type == booleanClass ) {
+		return execute_java_constructor(0, 0, javaLangBooleanClass,
+		    "(Z)V", *(jbool*)base);
 	}
-	else if ( fld->type == _Jv_charClass ) {
-		return execute_java_constructor(0,javaLangCharacterClass,"(C)V",*(jchar*)base);
+	else if ( fld->type == charClass ) {
+		return execute_java_constructor(0, 0, javaLangCharacterClass,
+		    "(C)V", *(jchar*)base);
 	}
-	else if ( fld->type == _Jv_floatClass ) {
-		return execute_java_constructor(0,javaLangFloatClass,"(F)V",*(jfloat*)base);
+	else if ( fld->type == floatClass ) {
+		return execute_java_constructor(0, 0, javaLangFloatClass,
+		    "(F)V", *(jfloat*)base);
 	}
-	else if ( fld->type == _Jv_doubleClass ) {
-		return execute_java_constructor(0,javaLangDoubleClass,"(D)V",*(jdouble*)base);
+	else if ( fld->type == doubleClass ) {
+		return execute_java_constructor(0, 0, javaLangDoubleClass,
+		    "(D)V", *(jdouble*)base);
 	}
 	else {
 		/* If fld->type isn't one of the above then it's either a
@@ -169,7 +176,7 @@ java_lang_reflect_Field_getBoolean(struct Hjava_lang_reflect_Field* this, struct
         fld = CLASS_FIELDS(clas) + unhand(this)->slot;
         base = getFieldAddress(this, obj);
 
-        if ( fld->type == _Jv_booleanClass ) {
+        if ( fld->type == booleanClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -180,7 +187,7 @@ java_lang_reflect_Field_getBoolean(struct Hjava_lang_reflect_Field* this, struct
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
@@ -203,7 +210,7 @@ java_lang_reflect_Field_getByte(struct Hjava_lang_reflect_Field* this, struct Hj
         fld = CLASS_FIELDS(clas) + unhand(this)->slot;
         base = getFieldAddress(this, obj);
 
-        if ( fld->type == _Jv_byteClass ) {
+        if ( fld->type == byteClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -214,7 +221,7 @@ java_lang_reflect_Field_getByte(struct Hjava_lang_reflect_Field* this, struct Hj
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
@@ -236,7 +243,7 @@ java_lang_reflect_Field_getChar(struct Hjava_lang_reflect_Field* this, struct Hj
         fld = CLASS_FIELDS(clas) + unhand(this)->slot;
         base = getFieldAddress(this, obj);
 
-        if ( fld->type == _Jv_charClass ) {
+        if ( fld->type == charClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -247,7 +254,7 @@ java_lang_reflect_Field_getChar(struct Hjava_lang_reflect_Field* this, struct Hj
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
@@ -270,7 +277,7 @@ java_lang_reflect_Field_getShort(struct Hjava_lang_reflect_Field* this, struct H
         fld = CLASS_FIELDS(clas) + unhand(this)->slot;
         base = getFieldAddress(this, obj);
 
-        if ( fld->type == _Jv_shortClass ) {
+        if ( fld->type == shortClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -281,13 +288,13 @@ java_lang_reflect_Field_getShort(struct Hjava_lang_reflect_Field* this, struct H
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
                 return *(jshort*)base;
 	}
-        else if ( fld->type == _Jv_byteClass ) {
+        else if ( fld->type == byteClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -298,7 +305,7 @@ java_lang_reflect_Field_getShort(struct Hjava_lang_reflect_Field* this, struct H
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
@@ -320,7 +327,7 @@ java_lang_reflect_Field_getInt(struct Hjava_lang_reflect_Field* this, struct Hja
         fld = CLASS_FIELDS(clas) + unhand(this)->slot;
         base = getFieldAddress(this, obj);
 
-        if ( fld->type == _Jv_intClass ) {
+        if ( fld->type == intClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -331,13 +338,13 @@ java_lang_reflect_Field_getInt(struct Hjava_lang_reflect_Field* this, struct Hja
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
                 return *(jint*)base;
 	}
-        else if ( fld->type == _Jv_shortClass ) {
+        else if ( fld->type == shortClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -348,13 +355,13 @@ java_lang_reflect_Field_getInt(struct Hjava_lang_reflect_Field* this, struct Hja
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
                 return (jint)(*(jshort*)base);
 	}
-        else if ( fld->type == _Jv_byteClass ) {
+        else if ( fld->type == byteClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -365,13 +372,13 @@ java_lang_reflect_Field_getInt(struct Hjava_lang_reflect_Field* this, struct Hja
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
                 return (jint)(*(jbyte*)base);
 	}
-        else if ( fld->type == _Jv_charClass ) {
+        else if ( fld->type == charClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -382,7 +389,7 @@ java_lang_reflect_Field_getInt(struct Hjava_lang_reflect_Field* this, struct Hja
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
@@ -404,7 +411,7 @@ java_lang_reflect_Field_getLong(struct Hjava_lang_reflect_Field* this, struct Hj
         fld = CLASS_FIELDS(clas) + unhand(this)->slot;
         base = getFieldAddress(this, obj);
 
-        if ( fld->type == _Jv_longClass ) {
+        if ( fld->type == longClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -415,13 +422,13 @@ java_lang_reflect_Field_getLong(struct Hjava_lang_reflect_Field* this, struct Hj
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.j;
 		} else
 #endif
                 return *(jlong*)base;
 	}
-        else if ( fld->type == _Jv_intClass ) {
+        else if ( fld->type == intClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -432,13 +439,13 @@ java_lang_reflect_Field_getLong(struct Hjava_lang_reflect_Field* this, struct Hj
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
                 return (jlong)(*(jint*)base);
 	}
-        else if ( fld->type == _Jv_shortClass ) {
+        else if ( fld->type == shortClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -449,13 +456,13 @@ java_lang_reflect_Field_getLong(struct Hjava_lang_reflect_Field* this, struct Hj
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
                 return (jlong)(*(jshort*)base);
 	}
-        else if ( fld->type == _Jv_byteClass ) {
+        else if ( fld->type == byteClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -466,13 +473,13 @@ java_lang_reflect_Field_getLong(struct Hjava_lang_reflect_Field* this, struct Hj
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
                 return (jlong)(*(jbyte*)base);
 	}
-        else if ( fld->type == _Jv_charClass ) {
+        else if ( fld->type == charClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -483,7 +490,7 @@ java_lang_reflect_Field_getLong(struct Hjava_lang_reflect_Field* this, struct Hj
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
@@ -505,7 +512,7 @@ java_lang_reflect_Field_getFloat(struct Hjava_lang_reflect_Field* this, struct H
         fld = CLASS_FIELDS(clas) + unhand(this)->slot;
         base = getFieldAddress(this, obj);
 
-        if ( fld->type == _Jv_floatClass ) {
+        if ( fld->type == floatClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -516,13 +523,13 @@ java_lang_reflect_Field_getFloat(struct Hjava_lang_reflect_Field* this, struct H
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.f;
 		} else
 #endif
                 return *(jfloat*)base;
 	}
-        else if ( fld->type == _Jv_longClass ) {
+        else if ( fld->type == longClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -533,13 +540,13 @@ java_lang_reflect_Field_getFloat(struct Hjava_lang_reflect_Field* this, struct H
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.j;
 		} else
 #endif
                 return (jfloat)(*(jlong*)base);
 	}
-        else if ( fld->type == _Jv_intClass ) {
+        else if ( fld->type == intClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -550,13 +557,13 @@ java_lang_reflect_Field_getFloat(struct Hjava_lang_reflect_Field* this, struct H
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
                 return (jfloat)(*(jint*)base);
 	}
-        else if ( fld->type == _Jv_shortClass ) {
+        else if ( fld->type == shortClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -567,13 +574,13 @@ java_lang_reflect_Field_getFloat(struct Hjava_lang_reflect_Field* this, struct H
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
                 return (jfloat)(*(jshort*)base);
 	}
-        else if ( fld->type == _Jv_byteClass ) {
+        else if ( fld->type == byteClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -584,13 +591,13 @@ java_lang_reflect_Field_getFloat(struct Hjava_lang_reflect_Field* this, struct H
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
                 return (jfloat)(*(jbyte*)base);
 	}
-        else if ( fld->type == _Jv_charClass ) {
+        else if ( fld->type == charClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -601,7 +608,7 @@ java_lang_reflect_Field_getFloat(struct Hjava_lang_reflect_Field* this, struct H
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
@@ -623,7 +630,7 @@ java_lang_reflect_Field_getDouble(struct Hjava_lang_reflect_Field* this, struct
         fld = CLASS_FIELDS(clas) + unhand(this)->slot;
         base = getFieldAddress(this, obj);
 
-        if ( fld->type == _Jv_doubleClass ) {
+        if ( fld->type == doubleClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -634,13 +641,13 @@ java_lang_reflect_Field_getDouble(struct Hjava_lang_reflect_Field* this, struct
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.d;
 		} else
 #endif
                 return *(jdouble*)base;
 	}
-        else if ( fld->type == _Jv_floatClass ) {
+        else if ( fld->type == floatClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -651,13 +658,13 @@ java_lang_reflect_Field_getDouble(struct Hjava_lang_reflect_Field* this, struct
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.f;
 		} else
 #endif
                 return (jdouble)(*(jfloat*)base);
 	}
-        else if ( fld->type == _Jv_longClass ) {
+        else if ( fld->type == longClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -668,13 +675,13 @@ java_lang_reflect_Field_getDouble(struct Hjava_lang_reflect_Field* this, struct
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.j;
 		} else
 #endif
                 return (jdouble)(*(jlong*)base);
 	}
-        else if ( fld->type == _Jv_intClass ) {
+        else if ( fld->type == intClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -685,13 +692,13 @@ java_lang_reflect_Field_getDouble(struct Hjava_lang_reflect_Field* this, struct
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
                 return (jdouble)(*(jint*)base);
 	}
-        else if ( fld->type == _Jv_shortClass ) {
+        else if ( fld->type == shortClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -702,13 +709,13 @@ java_lang_reflect_Field_getDouble(struct Hjava_lang_reflect_Field* this, struct
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
                 return (jdouble)(*(jshort*)base);
 	}
-        else if ( fld->type == _Jv_byteClass ) {
+        else if ( fld->type == byteClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -719,13 +726,13 @@ java_lang_reflect_Field_getDouble(struct Hjava_lang_reflect_Field* this, struct
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
                 return (jdouble)(*(jbyte*)base);
 	}
-        else if ( fld->type == _Jv_charClass ) {
+        else if ( fld->type == charClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -736,7 +743,7 @@ java_lang_reflect_Field_getDouble(struct Hjava_lang_reflect_Field* this, struct
 
 		if (mo) {
 		  jvalue result;
-		  guarana_reify_field_load(mo, obj, clas, fld, &result);
+		  guarana_reify_field_load(obj, clas, fld, &result);
 		  return result.i;
 		} else
 #endif
@@ -763,7 +770,7 @@ java_lang_reflect_Field_setBoolean(struct Hjava_lang_reflect_Field* this, struct
                 SignalError("java.lang.IllegalAccessException", "field is final");
         }
 
-        if ( fld->type == _Jv_booleanClass ) {
+        if ( fld->type == booleanClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -775,7 +782,7 @@ java_lang_reflect_Field_setBoolean(struct Hjava_lang_reflect_Field* this, struct
 		if (mo) {
 		  jvalue value;
 		  value.i = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jbool*)base = val;
@@ -800,7 +807,7 @@ java_lang_reflect_Field_setByte(struct Hjava_lang_reflect_Field* this, struct Hj
                 SignalError("java.lang.IllegalAccessException", "field is final");
         }
 
-        if ( fld->type == _Jv_byteClass ) {
+        if ( fld->type == byteClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -812,12 +819,12 @@ java_lang_reflect_Field_setByte(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.i = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jbyte*)base = val;
 	}
-        else if ( fld->type == _Jv_shortClass ) {
+        else if ( fld->type == shortClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -829,12 +836,12 @@ java_lang_reflect_Field_setByte(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.i = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jshort*)base = (jshort)val;
 	}
-        else if ( fld->type == _Jv_intClass ) {
+        else if ( fld->type == intClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -846,12 +853,12 @@ java_lang_reflect_Field_setByte(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.i = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jint*)base = (jint)val;
 	}
-        else if ( fld->type == _Jv_longClass ) {
+        else if ( fld->type == longClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -863,12 +870,12 @@ java_lang_reflect_Field_setByte(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.j = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jlong*)base = (jlong)val;
 	}
-        else if ( fld->type == _Jv_floatClass ) {
+        else if ( fld->type == floatClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -880,12 +887,12 @@ java_lang_reflect_Field_setByte(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.f = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jfloat*)base = (jfloat)val;
 	}
-        else if ( fld->type == _Jv_doubleClass ) {
+        else if ( fld->type == doubleClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -897,7 +904,7 @@ java_lang_reflect_Field_setByte(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.d = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jdouble*)base =(jdouble)val;
@@ -922,7 +929,7 @@ java_lang_reflect_Field_setChar(struct Hjava_lang_reflect_Field* this, struct Hj
                 SignalError("java.lang.IllegalAccessException", "field is final");
         }
 
-        if ( fld->type == _Jv_charClass ) {
+        if ( fld->type == charClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -934,12 +941,12 @@ java_lang_reflect_Field_setChar(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.i = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jchar*)base = val;
 	}
-        else if ( fld->type == _Jv_intClass ) {
+        else if ( fld->type == intClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -951,12 +958,12 @@ java_lang_reflect_Field_setChar(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.i = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jint*)base = (jint)val;
 	}
-        else if ( fld->type == _Jv_longClass ) {
+        else if ( fld->type == longClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -968,12 +975,12 @@ java_lang_reflect_Field_setChar(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.j = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jlong*)base = (jlong)val;
 	}
-        else if ( fld->type == _Jv_floatClass ) {
+        else if ( fld->type == floatClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -985,12 +992,12 @@ java_lang_reflect_Field_setChar(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.f = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jfloat*)base = (jfloat)val;
 	}
-        else if ( fld->type == _Jv_doubleClass ) {
+        else if ( fld->type == doubleClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1002,7 +1009,7 @@ java_lang_reflect_Field_setChar(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.d = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jdouble*)base =(jdouble)val;
@@ -1027,7 +1034,7 @@ java_lang_reflect_Field_setShort(struct Hjava_lang_reflect_Field* this, struct H
                 SignalError("java.lang.IllegalAccessException", "field is final");
         }
 
-        if ( fld->type == _Jv_shortClass ) {
+        if ( fld->type == shortClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1039,12 +1046,12 @@ java_lang_reflect_Field_setShort(struct Hjava_lang_reflect_Field* this, struct H
 		if (mo) {
 		  jvalue value;
 		  value.i = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jshort*)base = val;
 	}
-        else if ( fld->type == _Jv_intClass ) {
+        else if ( fld->type == intClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1056,12 +1063,12 @@ java_lang_reflect_Field_setShort(struct Hjava_lang_reflect_Field* this, struct H
 		if (mo) {
 		  jvalue value;
 		  value.i = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jint*)base = (jint)val;
 	}
-        else if ( fld->type == _Jv_longClass ) {
+        else if ( fld->type == longClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1073,12 +1080,12 @@ java_lang_reflect_Field_setShort(struct Hjava_lang_reflect_Field* this, struct H
 		if (mo) {
 		  jvalue value;
 		  value.j = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jlong*)base = (jlong)val;
 	}
-        else if ( fld->type == _Jv_floatClass ) {
+        else if ( fld->type == floatClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1090,12 +1097,12 @@ java_lang_reflect_Field_setShort(struct Hjava_lang_reflect_Field* this, struct H
 		if (mo) {
 		  jvalue value;
 		  value.f = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jfloat*)base = (jfloat)val;
 	}
-        else if ( fld->type == _Jv_doubleClass ) {
+        else if ( fld->type == doubleClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1107,7 +1114,7 @@ java_lang_reflect_Field_setShort(struct Hjava_lang_reflect_Field* this, struct H
 		if (mo) {
 		  jvalue value;
 		  value.d = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jdouble*)base =(jdouble)val;
@@ -1132,7 +1139,7 @@ java_lang_reflect_Field_setInt(struct Hjava_lang_reflect_Field* this, struct Hja
                 SignalError("java.lang.IllegalAccessException", "field is final");
         }
 
-        if ( fld->type == _Jv_intClass ) {
+        if ( fld->type == intClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1144,12 +1151,12 @@ java_lang_reflect_Field_setInt(struct Hjava_lang_reflect_Field* this, struct Hja
 		if (mo) {
 		  jvalue value;
 		  value.i = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jint*)base = val;
 	}
-        else if ( fld->type == _Jv_longClass ) {
+        else if ( fld->type == longClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1161,12 +1168,12 @@ java_lang_reflect_Field_setInt(struct Hjava_lang_reflect_Field* this, struct Hja
 		if (mo) {
 		  jvalue value;
 		  value.j = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jlong*)base = (jlong)val;
 	}
-        else if ( fld->type == _Jv_floatClass ) {
+        else if ( fld->type == floatClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1178,12 +1185,12 @@ java_lang_reflect_Field_setInt(struct Hjava_lang_reflect_Field* this, struct Hja
 		if (mo) {
 		  jvalue value;
 		  value.f = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jfloat*)base = (jfloat)val;
 	}
-        else if ( fld->type == _Jv_doubleClass ) {
+        else if ( fld->type == doubleClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1195,7 +1202,7 @@ java_lang_reflect_Field_setInt(struct Hjava_lang_reflect_Field* this, struct Hja
 		if (mo) {
 		  jvalue value;
 		  value.d = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jdouble*)base =(jdouble)val;
@@ -1220,7 +1227,7 @@ java_lang_reflect_Field_setLong(struct Hjava_lang_reflect_Field* this, struct Hj
                 SignalError("java.lang.IllegalAccessException", "field is final");
         }
 
-        if ( fld->type == _Jv_longClass ) {
+        if ( fld->type == longClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1232,12 +1239,12 @@ java_lang_reflect_Field_setLong(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.j = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jlong*)base = val;
 	}
-        else if ( fld->type == _Jv_floatClass ) {
+        else if ( fld->type == floatClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1249,12 +1256,12 @@ java_lang_reflect_Field_setLong(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.f = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jfloat*)base = (jfloat)val;
 	}
-        else if ( fld->type == _Jv_doubleClass ) {
+        else if ( fld->type == doubleClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1266,7 +1273,7 @@ java_lang_reflect_Field_setLong(struct Hjava_lang_reflect_Field* this, struct Hj
 		if (mo) {
 		  jvalue value;
 		  value.d = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jdouble*)base = (jdouble)val;
@@ -1291,7 +1298,7 @@ java_lang_reflect_Field_setFloat(struct Hjava_lang_reflect_Field* this, struct H
                 SignalError("java.lang.IllegalAccessException", "field is final");
         }
 
-        if ( fld->type == _Jv_floatClass ) {
+        if ( fld->type == floatClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1303,12 +1310,12 @@ java_lang_reflect_Field_setFloat(struct Hjava_lang_reflect_Field* this, struct H
 		if (mo) {
 		  jvalue value;
 		  value.f = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jfloat*)base = val;
 	}
-        else if ( fld->type == _Jv_doubleClass ) {
+        else if ( fld->type == doubleClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1320,7 +1327,7 @@ java_lang_reflect_Field_setFloat(struct Hjava_lang_reflect_Field* this, struct H
 		if (mo) {
 		  jvalue value;
 		  value.d = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jdouble*)base =(jdouble)val;
@@ -1345,7 +1352,7 @@ java_lang_reflect_Field_setDouble(struct Hjava_lang_reflect_Field* this, struct
                 SignalError("java.lang.IllegalAccessException", "field is final");
         }
 
-        if ( fld->type == _Jv_doubleClass ) {
+        if ( fld->type == doubleClass ) {
 #if GUARANA
 		struct HBR_unicamp_Guarana_MetaObject *mo;
 
@@ -1357,7 +1364,7 @@ java_lang_reflect_Field_setDouble(struct Hjava_lang_reflect_Field* this, struct
 		if (mo) {
 		  jvalue value;
 		  value.d = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		} else
 #endif
                 *(jdouble*)base = val;
@@ -1400,7 +1407,7 @@ java_lang_reflect_Field_set(Hjava_lang_reflect_Field* this, struct Hjava_lang_Ob
 		if (mo) {
 		  jvalue value;
 		  value.l = val;
-		  guarana_reify_field_store(mo, obj, clas, fld, &value);
+		  guarana_reify_field_store(obj, clas, fld, &value);
 		  return;
 		}
 
