@@ -18,13 +18,14 @@
 #include "../../../kaffe/kaffevm/stringSupport.h"
 #include "../../../kaffe/kaffevm/support.h"
 #include "../../../kaffe/kaffevm/exception.h"
+#include "../../../kaffe/kaffevm/baseClasses.h"
+#include "../../../kaffe/kaffevm/sha-1.h"
 #include <native.h>
 #include "java_io_ObjectInputStream.h"
 #include "java_io_ObjectOutputStream.h"
 #include "kaffe_io_ObjectStreamClassImpl.h"
-#include "sha-1.h"
 
-extern jobject Kaffe_NewObject(JNIEnv*, jclass, jmethodID, ...);   /* XXX */
+#include <jni.h>
 
 static Hjava_lang_Object* newSerialObject(Hjava_lang_Class*,Hjava_lang_Object*);
 static HArrayOfObject* getFields(struct Hkaffe_io_ObjectStreamClassImpl*);
@@ -53,13 +54,34 @@ kaffe_io_ObjectStreamClassImpl_init(void)
 {
 	errorInfo einfo;
 
-	serialVersionUIDName = utf8ConstNew("serialVersionUID" , -1);
+	serialVersionUIDName = checkPtr(utf8ConstNew("serialVersionUID" , -1));
 	writeObjectName = utf8ConstNew("writeObject", -1);
+	if (!writeObjectName) {
+		postOutOfMemory(&einfo);
+		goto svun;
+	}
 	readObjectName = utf8ConstNew("readObject", -1);
+	if (!readObjectName) {
+		postOutOfMemory(&einfo);
+		goto won;
+	}
 	ObjectOutputStreamSig = utf8ConstNew("(Ljava/io/ObjectOutputStream;)V", -1);
+	if (!ObjectOutputStreamSig) {
+		postOutOfMemory(&einfo);
+		goto ron;
+	}
 	ObjectInputStreamSig = utf8ConstNew("(Ljava/io/ObjectInputStream;)V", -1);
-	ptrType = lookupClass("kaffe/util/Ptr", &einfo);
+	if (!ObjectInputStreamSig) {
+		postOutOfMemory(&einfo);
+		goto oos;
+	}
+	ptrType = lookupClass("kaffe/util/Ptr", NULL, &einfo);
 	if (ptrType == 0) {
+		utf8ConstRelease(ObjectInputStreamSig);
+oos: 		utf8ConstRelease(ObjectOutputStreamSig);
+ron: 		utf8ConstRelease(readObjectName);
+won: 		utf8ConstRelease(writeObjectName);
+svun: 		utf8ConstRelease(serialVersionUIDName);
 		throwError(&einfo);
 	}
 }
@@ -76,6 +98,24 @@ kaffe_io_ObjectStreamClassImpl_allocateNewArray(struct Hkaffe_io_ObjectStreamCla
 	return (newArray(CLASS_ELEMENT_TYPE(unhand(cls)->clazz), sz));
 }
 
+void
+kaffe_io_ObjectStreamClassImpl_invokeSuperclassInitV(struct Hkaffe_io_ObjectStreamClassImpl* cls, struct Hjava_lang_Object* obj, struct Hjava_lang_Class* sc)
+{
+	Method* meth;
+
+	if (sc == NULL) {
+		return;
+	}
+
+	meth = findMethodLocal(sc, constructor_name, void_signature);
+	if (meth == NULL) {
+		SignalErrorf("java.io.InvalidClassException",
+			     "%s; <init>",
+			     sc->name->data);
+	}
+	do_execute_java_method(obj, 0, 0, meth, 0);
+}
+
 void
 kaffe_io_ObjectStreamClassImpl_inputClassFields(struct Hkaffe_io_ObjectStreamClassImpl* cls, struct Hjava_lang_Object* obj, struct Hjava_io_ObjectInputStream* in)
 {
@@ -94,7 +134,7 @@ kaffe_io_ObjectStreamClassImpl_inputClassFields(struct Hkaffe_io_ObjectStreamCla
 	}
 
 #define READ(FUNC,SIG,TYPE) \
-	((jvalue*)(((uint8*)obj) + FIELD_OFFSET(*fld)))->TYPE = \
+	((jvalue*)(((uint8*)obj) + FIELD_BOFFSET(*fld)))->TYPE = \
 		do_execute_java_method(in, #FUNC, #SIG, 0, 0).TYPE
 
 	for (i = 0; i < len; i++, fld++) {
@@ -158,7 +198,7 @@ kaffe_io_ObjectStreamClassImpl_outputClassFields(struct Hkaffe_io_ObjectStreamCl
 
 #define WRITE(FUNC,SIG,TYPE) \
 	do_execute_java_method(out, #FUNC, #SIG, 0, 0, \
-		((jvalue*)(((uint8*)obj) + FIELD_OFFSET(*fld)))->TYPE)
+		((jvalue*)(((uint8*)obj) + FIELD_BOFFSET(*fld)))->TYPE)
 
 	for (i = 0; i < len; i++, fld++) {
 		if (FIELD_ISREF(*fld)) {
@@ -220,7 +260,7 @@ kaffe_io_ObjectStreamClassImpl_outputClassFieldInfo(struct Hkaffe_io_ObjectStrea
 		type = FIELD_TYPE(*fld);
 		if (CLASS_IS_PRIMITIVE(type)) {
 			do_execute_java_method(out, "writeByte", "(I)V", 0, 0, CLASS_PRIM_SIG(type));
-			do_execute_java_method(out, "writeUTF", "(Ljava/lang/String;)V", 0, 0, utf8Const2Java((*fld)->name));
+			do_execute_java_method(out, "writeUTF", "(Ljava/lang/String;)V", 0, 0, checkPtr(utf8Const2Java((*fld)->name)));
 /* We dont output the modifiers 
 			do_execute_java_method(out, "writeShort", "(I)V", 0, 0, (*fld)->accflags);
 */
@@ -241,11 +281,11 @@ kaffe_io_ObjectStreamClassImpl_outputClassFieldInfo(struct Hkaffe_io_ObjectStrea
 				strcat(buf, ";");
 			}
 			do_execute_java_method(out, "writeByte", "(I)V", 0, 0, buf[0]);
-			do_execute_java_method(out, "writeUTF", "(Ljava/lang/String;)V", 0, 0, utf8Const2Java((*fld)->name));
+			do_execute_java_method(out, "writeUTF", "(Ljava/lang/String;)V", 0, 0, checkPtr(utf8Const2Java((*fld)->name)));
 /* We dont output the modifiers 
 			do_execute_java_method(out, "writeShort", "(I)V", 0, 0, (*fld)->accflags);
 */
-			do_execute_java_method(out, "writeObject", "(Ljava/lang/Object;)V", 0, 0, stringC2Java(buf));
+			do_execute_java_method(out, "writeObject", "(Ljava/lang/Object;)V", 0, 0, checkPtr(stringC2Java(buf)));
 		}
 	}
 }
@@ -263,6 +303,11 @@ kaffe_io_ObjectStreamClassImpl_invokeObjectReader0(struct Hkaffe_io_ObjectStream
 	unhand(in)->currentStreamClass = (struct Hjava_io_ObjectStreamClass*)cls;
 
 	meth = findMethodLocal(unhand(cls)->clazz, readObjectName, ObjectInputStreamSig);
+	if (meth == 0) {
+		errorInfo info;
+		postExceptionMessage(&info, JAVA_LANG(NoSuchMethodError), readObjectName->data);
+		throwError(&info);
+	}
 	do_execute_java_method(obj, 0, 0, meth, 0, in);
 
 	unhand(in)->currentObject = oldObj;
@@ -351,12 +396,14 @@ addToSHA(SHA1_CTX* c, uidItem* base, int len)
  * a newly allocated one in dot form pkg.subpkg.name.
  * Caller must free using KFREE.
  */
-static char*
+char*
 pathname2ClassnameCopy(const char *orig)
 {
 	char* str;
 	str = KMALLOC(strlen(orig) + 1);
-	pathname2classname(orig, str);
+	if (str != 0) {
+		pathname2classname(orig, str);
+	}
 	return (str);
 }
 
@@ -375,7 +422,7 @@ getFieldDesc(Field* fld)
 		/* This is like so: Ljava/lang/String; */
 		orig = ((Utf8Const*)(void*)fld->type)->data;
 		str = KMALLOC(strlen(orig) + 1);
-		return (strcpy(str, orig));
+		return (str ? strcpy(str, orig) : 0);
 	}
 	else if (!CLASS_IS_PRIMITIVE(FIELD_TYPE(fld))) {
 		/* This is like so: java.lang.String */
@@ -383,10 +430,11 @@ getFieldDesc(Field* fld)
 		if (orig[0] == '[') {
 			/* arrays should be fine */
 			str = KMALLOC(strlen(orig) + 1);
-			classname2pathname(orig, str);
+			if (str) classname2pathname(orig, str);
 			return (str);
 		} else {
 			str = KMALLOC(strlen(orig) + 3);
+			if (!str) return 0;
 			strcpy(str, "L");
 			strcat(str, orig);
 			strcat(str, ";");
@@ -397,7 +445,7 @@ getFieldDesc(Field* fld)
 	else {
 		orig = CLASS_PRIM_NAME(FIELD_TYPE(fld))->data;
 		str = KMALLOC(strlen(orig) + 1);
-		return (strcpy(str, orig));
+		return (str ? strcpy(str, orig) : 0);
 	}
 }
 
@@ -405,7 +453,7 @@ static
 char*
 getMethodDesc(Method* mth)
 {
-	return (pathname2ClassnameCopy(mth->signature->data));
+	return (pathname2ClassnameCopy(METHOD_SIGD(mth)));
 }
 
 /*
@@ -433,8 +481,16 @@ kaffe_io_ObjectStreamClassImpl_getSerialVersionUID0(Hjava_lang_Class* cls)
 	jshort tmp;
 	char *classname;
 
+	/* For GCJ compatibility, we cannot hardcode this in java/lang/Class 
+	 * This number won't change as long as the universe exists, so it's
+	 * safe to keep it here.
+	 */
+	if (cls == ClassClass) {
+		return (3206093459760846163LL);
+	}
 	fld = lookupClassField(cls, serialVersionUIDName, true, &einfo);
-	if (fld != 0) {
+
+	if (fld != 0 && (fld->accflags & (ACC_STATIC|ACC_FINAL)) == (ACC_STATIC|ACC_FINAL)) {
 		return (*(jlong*)FIELD_ADDRESS((Field*)fld));
 	} else {
 		discardErrorInfo(&einfo);
@@ -452,17 +508,26 @@ kaffe_io_ObjectStreamClassImpl_getSerialVersionUID0(Hjava_lang_Class* cls)
 	}
 	if (len > 0) {
 		base = KMALLOC(sizeof(uidItem) * len);
+		if (!base) {
+			postOutOfMemory(&einfo);
+			throwError(&einfo);
+		}
 	}
 
 	SHA1Init(&c);
 
 	/* Class -> name(UTF), modifier(INT) */
-	tmp = htons(strlen(cls->name->data));
-	SHA1Update(&c, (char*)&tmp, sizeof(tmp));
 	/* we store the classname with slashes as path names, 
 	 * but here we must use the dotted form
 	 */
 	classname = getClassName(cls);
+	if (!classname) {
+		KFREE(base);
+		postOutOfMemory(&einfo);
+		throwError(&einfo);
+	}
+	tmp = htons(strlen(classname));
+	SHA1Update(&c, (char*)&tmp, sizeof(tmp));
 	SHA1Update(&c, classname, strlen(classname));
 	KFREE(classname);
 	mod = htonl((int)cls->accflags & (ACC_ABSTRACT|ACC_FINAL|ACC_INTERFACE|ACC_PUBLIC));
@@ -479,13 +544,22 @@ kaffe_io_ObjectStreamClassImpl_getSerialVersionUID0(Hjava_lang_Class* cls)
 			base[i].name = getClassName(cls->interfaces[i]);
 			base[i].modifier = -1;
 			base[i].desc = 0;
+			if (!base[i].name) {
+				int j = cls->interface_len;
+
+				for (j--; j > i; j--)
+					KFREE(base[j].name);
+				KFREE(base);
+				postOutOfMemory(&einfo);
+				throwError(&einfo);
+			}
 		}
 		addToSHA(&c, base, cls->interface_len);
 
 		/* Free all the interface name strings */
 		i = cls->interface_len;
 		for (i--; i >= 0; i--) {
-			KFREE((char*)base[i].name);
+			KFREE(base[i].name);
 		}
 	}
 
@@ -503,6 +577,15 @@ kaffe_io_ObjectStreamClassImpl_getSerialVersionUID0(Hjava_lang_Class* cls)
 				base[i].name = fld->name->data;
 				base[i].modifier = (int)fld->accflags & ACC_MASK;
 				base[i].desc = getFieldDesc(fld);
+				if (!base[i].desc) {
+					int j = CLASS_NFIELDS(cls);
+
+					for (j--; j > i; j--)
+						KFREE(base[j].desc);
+					KFREE(base);
+					postOutOfMemory(&einfo);
+					throwError(&einfo);
+				}
 			}
 		}
 		addToSHA(&c, base, CLASS_NFIELDS(cls));
@@ -510,7 +593,7 @@ kaffe_io_ObjectStreamClassImpl_getSerialVersionUID0(Hjava_lang_Class* cls)
 		/* free descriptors */
 		i = CLASS_NFIELDS(cls);
 		for (i--; i >= 0; i--) {
-			KFREE((char*)base[i].desc);
+			KFREE(base[i].desc);
 		}
 	}
 
@@ -538,13 +621,22 @@ kaffe_io_ObjectStreamClassImpl_getSerialVersionUID0(Hjava_lang_Class* cls)
 			}
 			base[i].modifier = (int)mth->accflags & ACC_MASK;
 			base[i].desc = getMethodDesc(mth);
+			if (!base[i].desc) {
+				int j = CLASS_NMETHODS(cls);
+
+				for (j--; j > i; j--)
+					KFREE(base[j].desc);
+				KFREE(base);
+				postOutOfMemory(&einfo);
+				throwError(&einfo);
+			}
 		}
 		addToSHA(&c, base, CLASS_NMETHODS(cls));
 
 		/* Free all the descriptor strings */
 		i = CLASS_NMETHODS(cls);
 		for (i--; i >= 0; i--) {
-			KFREE((char*)base[i].desc);
+			KFREE(base[i].desc);
 		}
 
 		/* NB: we can't reuse the base array here because
@@ -559,7 +651,7 @@ kaffe_io_ObjectStreamClassImpl_getSerialVersionUID0(Hjava_lang_Class* cls)
 			/* skip private methods, constructors, and
 			 * <clinit> again.  Do not include <clinit> twice 
 			 */
-			if (((mth->accflags & (ACC_CONSTRUCTOR|ACC_PRIVATE)) != 0) || (utf8ConstEqual(mth->name, init_name) && utf8ConstEqual(mth->signature, void_signature))) { 
+			if (((mth->accflags & (ACC_CONSTRUCTOR|ACC_PRIVATE)) != 0) || (utf8ConstEqual(mth->name, init_name) && utf8ConstEqual(METHOD_SIG(mth), void_signature))) { 
 				base[i].name = 0;
 			}
 			else {
@@ -567,13 +659,22 @@ kaffe_io_ObjectStreamClassImpl_getSerialVersionUID0(Hjava_lang_Class* cls)
 			}
 			base[i].modifier = (int)mth->accflags & ACC_MASK;
 			base[i].desc = getMethodDesc(mth);
+			if (!base[i].desc) {
+				int j = CLASS_NMETHODS(cls);
+
+				for (j--; j > i; j--)
+					KFREE(base[j].desc);
+				KFREE(base);
+				postOutOfMemory(&einfo);
+				throwError(&einfo);
+			}
 		}
 		addToSHA(&c, base, CLASS_NMETHODS(cls));
 
 		/* Free all the descriptor strings */
 		i = CLASS_NMETHODS(cls);
 		for (i--; i >= 0; i--) {
-			KFREE((char*)base[i].desc);
+			KFREE(base[i].desc);
 		}
 	}
 	
@@ -594,13 +695,12 @@ kaffe_io_ObjectStreamClassImpl_getSerialVersionUID0(Hjava_lang_Class* cls)
 jbool
 kaffe_io_ObjectStreamClassImpl_hasWriteObject(struct Hjava_lang_Class* clazz)
 {
-	for (; clazz != 0; clazz = clazz->superclass) {
-		Method* mptr = findMethodLocal(clazz, writeObjectName, ObjectOutputStreamSig);
-		if (mptr != 0) {
-			return (true);
-		}
+	if (findMethodLocal(clazz, writeObjectName, ObjectOutputStreamSig)) {
+		return (true);
+	}
+	else {
+		return (false);
 	}
-	return (false);
 }
 
 static
@@ -612,11 +712,15 @@ findDefaultSerialization(Hjava_lang_Class* clazz)
 	errorInfo einfo;
 
 	name = KMALLOC(strlen(clazz->name->data) + 22);
+	if (!name) {
+		postOutOfMemory(&einfo);
+		throwError(&einfo);
+	}
 	strcpy(name, clazz->name->data);
 	strcat(name, "$DefaultSerialization");
 
 	/* Use the JNI because it handles errors */
-	nclazz = lookupClass(name, &einfo);
+	nclazz = lookupClass(name, clazz->loader, &einfo);
 	if (nclazz == 0) {
 		discardErrorInfo(&einfo);
 		nclazz = clazz;
@@ -641,9 +745,10 @@ newSerialObject(Hjava_lang_Class* clazz, Hjava_lang_Object* obj)
 
 	n = CLASS_NMETHODS(clazz);
 	for (mptr = CLASS_METHODS(clazz); --n >= 0; ++mptr) {
-		if (strcmp(mptr->name->data, "<init>") == 0 && strcmp(mptr->signature->data, "()V") != 0) {
-			/* XXX !!! */
-			return ((Hjava_lang_Object*)Kaffe_NewObject(0, clazz, (jmethodID)mptr, obj));
+		if (utf8ConstEqual(mptr->name, constructor_name) && !utf8ConstEqual(METHOD_SIG(mptr), void_signature)) {
+			extern JNIEnv Kaffe_JNIEnv;
+			JNIEnv *env = &Kaffe_JNIEnv;
+			return ((Hjava_lang_Object*)(*env)->NewObject(env, clazz, (jmethodID)mptr, obj));
 		}
 	}
 
@@ -673,7 +778,6 @@ HArrayOfObject*
 getFields(struct Hkaffe_io_ObjectStreamClassImpl* cls)
 {
 	Hjava_lang_Class* clazz;
-	int offset;
 	int len;
 	Field* fld;
 	int cnt;
@@ -684,37 +788,35 @@ getFields(struct Hkaffe_io_ObjectStreamClassImpl* cls)
 	if (unhand(cls)->iclazz == 0) {
 		unhand(cls)->iclazz = findDefaultSerialization(unhand(cls)->clazz);
 	}
-	if (unhand(cls)->iclazz == unhand(cls)->clazz) {
-		clazz = unhand(cls)->clazz;
-		offset = 0;
-	}
-	else {
-		/* We skip the first inner class field since this is a
-		 * pointer to the outer class.
-		 */
-		clazz = unhand(cls)->iclazz;
-		offset = 1;
-	}
+	clazz = unhand(cls)->iclazz;
 
 	/* Count the number of fields we need to store */
-	len = CLASS_NIFIELDS(clazz) - offset;
-	fld = CLASS_IFIELDS(clazz) + offset;
+	len = CLASS_NIFIELDS(clazz);
+	fld = CLASS_IFIELDS(clazz);
 	cnt = 0;
 	for (i = 0; i < len; i++, fld++) {
 		if ((fld->accflags & ACC_TRANSIENT) != 0) {
 			continue;
 		}
+		/* Skip innerclass nonsense */
+		if (strncmp(fld->name->data, "this$", 5) == 0) {
+			continue;
+		}
 		cnt++;
 	}
 
 	/* Build an array of those fields */
 	array = (HArrayOfObject*)newArray(ptrType, cnt);
 	cnt = 0;
-	fld = CLASS_IFIELDS(clazz) + offset;
+	fld = CLASS_IFIELDS(clazz);
 	for (i = 0; i < len; i++, fld++) {
 		if ((fld->accflags & ACC_TRANSIENT) != 0) {
 			continue;
 		}
+		/* Skip innerclass nonsense */
+		if (strncmp(fld->name->data, "this$", 5) == 0) {
+			continue;
+		}
 		unhand_array(array)->body[cnt] = (void*)fld;
 		cnt++;
 	}
