@@ -8,8 +8,8 @@
  * of this file. 
  */
 
-#include <stdio.h>
-#include <stdlib.h>
+#include "config.h"
+#include "config-io.h"
 #include <assert.h>
 #include <native.h>
 #include "java_io_FileDescriptor.h"
@@ -19,7 +19,7 @@
 #include "../../../kaffe/kaffevm/support.h"
 
 extern Hjava_lang_Object* buildStackTrace(void*);
-extern void printStackTrace(struct Hjava_lang_Throwable*, struct Hjava_lang_Object*);
+extern void printStackTrace(struct Hjava_lang_Throwable*, struct Hjava_lang_Object*, int);
 
 /*
  * Fill in stack trace information - don't know what thought.
@@ -37,5 +37,5 @@ java_lang_Throwable_fillInStackTrace(struct Hjava_lang_Throwable* o)
 void
 java_lang_Throwable_printStackTrace0(struct Hjava_lang_Throwable* o, struct Hjava_lang_Object* p)
 {
-	printStackTrace(o, p);
+	printStackTrace(o, p, 0);
 }
