@@ -2,7 +2,7 @@
  * GuaraNative.c
  * Utility functions for native implementation of Guarana
  *
- * Copyright 1997,1998,1999 Alexandre Oliva <oliva@dcc.unicamp.br>
+ * Copyright 1997,1998,1999,2000 Alexandre Oliva <oliva@lsd.ic.unicamp.br>
  *
  * This file is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -27,14 +27,10 @@
 
 #include "debug.h"
 
-char * guarana_version = GVER;
+char * guarana_version = GUARANA_VERSION;
 
 /* These are initialized by kaffe_guarana_init_function_pointers, in Class.c */
 
-HArrayOfObject* (*p_makeParameters)(Method *meth);
-
-Hjava_lang_Class* (*p_makeReturn)(Method *meth);
-
 struct Hjava_lang_reflect_Constructor* (*p_makeConstructor)(struct Hjava_lang_Class* clazz, int slot);
 
 struct Hjava_lang_reflect_Method* (*p_makeMethod)(struct Hjava_lang_Class* clazz, int slot);
@@ -49,11 +45,11 @@ static Method *setMetaObjectOf = 0, *newObjConstr = 0,
   *broadcast = 0;
 static jobject *Guarana_blocker = 0;
 
-NativeOperation *guarana_new_native_operation(struct Hjava_lang_Object *object, struct Hjava_lang_Thread *thread, struct HBR_unicamp_Guarana_Operation *replaced) {
+NativeOperation *guarana_new_native_operation(struct Hjava_lang_Object *object, struct HBR_unicamp_Guarana_Operation *replaced) {
   NativeOperation *op;
   if (!operationClass) {
     errorInfo info;
-    operationClass = lookupClass("BR/unicamp/Guarana/Operation", &info);
+    operationClass = lookupClass("BR/unicamp/Guarana/Operation", NULL, &info);
     if (!operationClass)
       throwError(&info);
     CLASS_FSIZE(operationClass) = sizeof(NativeOperation);
@@ -68,22 +64,25 @@ DBG(NEWOBJECT,
     )
 
   op->object = object;
-  op->thread = thread;
   op->replaced = replaced;
-  op->kind = replaced ? op_performed : 0;
+  op->thread = replaced ? ((NativeOperation*)replaced)->thread : 0;
+  op->kind = 0;
   return op;
 }
 
 struct HBR_unicamp_Guarana_Result *guarana_new_result_object(int /* enum result_type */ kind, struct HBR_unicamp_Guarana_Operation *op, jvalue result) {
   NativeResult *res;
+  errorInfo info;
   if (!resultClass) {
-    errorInfo info;
-    resultClass = lookupClass("BR/unicamp/Guarana/Result", &info);
+    resultClass = lookupClass("BR/unicamp/Guarana/Result", NULL, &info);
     if (!resultClass)
       throwError(&info);
     CLASS_FSIZE(resultClass) = sizeof(NativeResult);
   }
-  res = (NativeResult*)newObjectNoMessage(resultClass);
+  res = (NativeResult*)newObjectNoMessage(resultClass, &info);
+  if (!res)
+      throwError(&info);
+  
   res->kind = kind;
   res->op = op;
   res->res = result;
@@ -92,14 +91,17 @@ struct HBR_unicamp_Guarana_Result *guarana_new_result_object(int /* enum result_
 
 OperationFactory *guarana_new_operation_factory(struct Hjava_lang_Object *object, struct HBR_unicamp_Guarana_MetaObject *metaObject) {
   OperationFactory *fact;
+  errorInfo info;
   if (!factoryClass) {
-    errorInfo info;
-    factoryClass = lookupClass("BR/unicamp/Guarana/OperationFactoryInternal", &info);
+    factoryClass = lookupClass("BR/unicamp/Guarana/OperationFactoryInternal",
+			       NULL, &info);
     if (!factoryClass)
       throwError(&info);
     CLASS_FSIZE(factoryClass) = sizeof(OperationFactory);
   }
-  fact = (OperationFactory*)newObjectNoMessage(factoryClass);
+  fact = (OperationFactory*)newObjectNoMessage(factoryClass, &info);
+  if (!fact)
+    throwError(&info);
   fact->obj = object;
   fact->mobj = metaObject;
   return fact;
@@ -116,33 +118,28 @@ struct Hjava_lang_Object *guarana_wrap(struct Hjava_lang_Class *clazz, jvalue va
   switch (CLASS_PRIM_SIG(clazz)) {
   case 'Z':
     *size = 1;
-    return execute_java_constructor(0, javaLangBooleanClass, "(Z)V", val.i);
+    return execute_java_constructor(0, 0, javaLangBooleanClass, "(Z)V", val.i);
   case 'B':
     *size = 1;
-    return execute_java_constructor(0, javaLangByteClass, "(B)V", val.i);
+    return execute_java_constructor(0, 0, javaLangByteClass, "(B)V", val.i);
   case 'S':
     *size = 1;
-    return execute_java_constructor(0, javaLangShortClass, "(S)V", val.i);
+    return execute_java_constructor(0, 0, javaLangShortClass, "(S)V", val.i);
   case 'I':
     *size = 1;
-    return execute_java_constructor(0, javaLangIntegerClass, "(I)V", val.i);
+    return execute_java_constructor(0, 0, javaLangIntegerClass, "(I)V", val.i);
   case 'J':
     *size = 2;
-    return execute_java_constructor(0, javaLangLongClass, "(J)V", val.j);
+    return execute_java_constructor(0, 0, javaLangLongClass, "(J)V", val.j);
   case 'C':
     *size = 1;
-    return execute_java_constructor(0, javaLangCharacterClass, "(C)V", val.i);
+    return execute_java_constructor(0, 0, javaLangCharacterClass, "(C)V", val.i);
   case 'F':
     *size = 1;
-    return execute_java_constructor(0, javaLangFloatClass, "(F)V",
-				    *(int*)&val.f);
-    /* FIXME: a float would be promoted to double because of varargs.
-     * For consistency, kaffe should always pass floats as doubles,
-     * but it doesn't; this breaks native methods that take floats as
-     * arguments or return them.  */
+    return execute_java_constructor(0, 0, javaLangFloatClass, "(F)V", val.f);
   case 'D':
     *size = 2;
-    return execute_java_constructor(0, javaLangDoubleClass, "(D)V", val.d);
+    return execute_java_constructor(0, 0, javaLangDoubleClass, "(D)V", val.d);
   case 'V':
     *size = 0;
     return 0;
@@ -211,7 +208,7 @@ int guarana_unwrap(struct Hjava_lang_Class *clazz, jvalue *val, struct Hjava_lan
 NativeResult *guarana_handle_operation(NativeOperation *op) {
   NativeResult *res;
 
-  res = (NativeResult*)(do_execute_java_class_method("BR/unicamp/Guarana/Guarana", "perform", "(LBR/unicamp/Guarana/Operation;)LBR/unicamp/Guarana/Result;", op).l);
+  res = (NativeResult*)(do_execute_java_class_method("BR/unicamp/Guarana/Guarana", NULL, "perform", "(LBR/unicamp/Guarana/Operation;)LBR/unicamp/Guarana/Result;", op).l);
 
   if (res->kind & res_thrown)
     throwException(res->res.l);
@@ -231,14 +228,14 @@ guarana_slots_reify_method_invocation(struct Hjava_lang_Object *obj, Method *met
     const char *sig;
     int argcount;
 
-    op = guarana_new_native_operation(obj, getCurrentThread(), 0);
+    op = guarana_new_native_operation(obj, 0);
     if (utf8ConstEqual(method->name, init_name) ||
 	utf8ConstEqual(method->name, constructor_name))
       op->kind |= op_constructor_invocation;
     else
       op->kind |= op_method_invocation;
     op->op_id.method = method;
-    sig = op->op_id.method->signature->data;
+    sig = METHOD_SIGD(method);
     argcount = sizeofSig(&sig, true);
     op->op_arg.l = argcount ? ALLOC_ARGVEC_FOR(argcount) : 0;
     if (argcount)
@@ -246,7 +243,7 @@ guarana_slots_reify_method_invocation(struct Hjava_lang_Object *obj, Method *met
 
     res = guarana_handle_operation(op);
 
-    switch(*sig) {
+    switch(*METHOD_RET_TYPE(method)) {
     case 'Z':
     case 'B':
     case 'S':
@@ -280,7 +277,11 @@ guarana_slots_reify_method_invocation(struct Hjava_lang_Object *obj, Method *met
 void
 guarana_slots_resolve_method_invocation(struct Hjava_lang_Object *obj, Method *method, slots *args, slots *retval) {
   if (!METHOD_IS_STATIC(method))
+#ifdef DMETHOD_DESC
       method = DMETHOD_DESC(obj->dtable->method[method->idx]);
+#else
+      method = DMETHOD_ADDR(obj->dtable->method[method->idx]);
+#endif
   guarana_slots_reify_method_invocation(obj, method, args, retval);
 }
 
@@ -416,25 +417,26 @@ guarana_va_list_reify_method_invocation(struct Hjava_lang_Object *obj, Method *m
   NativeOperation *op;
   NativeResult *res;
   const char *sig;
-  int argcount;
+  int argcount, i, len;
   jvalue *argl;
 
   if ((method->accflags & ACC_STATIC) != 0)
     obj = &method->class->head;
-  op = guarana_new_native_operation(obj, getCurrentThread(), 0);
+  op = guarana_new_native_operation(obj, 0);
   if (utf8ConstEqual(method->name, init_name) ||
       utf8ConstEqual(method->name, constructor_name))
     op->kind |= op_constructor_invocation;
   else
     op->kind |= op_method_invocation;
   op->op_id.method = method;
-  sig = method->signature->data;
+  sig = METHOD_SIGD(method);
   argcount = sizeofSig(&sig, true);
   op->op_arg.l = argcount ? ALLOC_ARGVEC_FOR(argcount) : 0;
+  i = 0;
+  len = METHOD_NARGS(method);
   
-  for((sig = method->signature->data + 1), (argl = (jvalue*)op->op_arg.l);
-      *sig != ')'; ++sig, ++argl) {
-    switch (*sig) {
+  for(argl = (jvalue*)op->op_arg.l; i < len; ++i, ++argl) {
+    switch (*METHOD_ARG_TYPE(method, i)) {
     case 'Z':
     case 'B':
     case 'S':
@@ -447,21 +449,14 @@ guarana_va_list_reify_method_invocation(struct Hjava_lang_Object *obj, Method *m
       ++argl;
       break;
     case 'F':
-      argl->f = va_arg(args, jfloat);
+      argl->f = (jfloat)va_arg(args, jdouble);
       break;
     case 'D':
       argl->d = va_arg(args, jdouble);
       ++argl;
       break;
     case '[':
-      while (*sig == '[')
-	++sig;
-      if (*sig == 'L') { /* fall through */
     case 'L':
-	while (*sig != ';') {
-	  sig++;
-	}
-      }
       argl->l = va_arg(args, jref);
       break;
     default:
@@ -473,7 +468,7 @@ guarana_va_list_reify_method_invocation(struct Hjava_lang_Object *obj, Method *m
 
   /* skip to return type */
 
-  switch (*++sig) {
+  switch (*METHOD_RET_TYPE(method)) {
     case 'Z':
     case 'B':
     case 'S':
@@ -504,7 +499,11 @@ guarana_va_list_reify_method_invocation(struct Hjava_lang_Object *obj, Method *m
 void
 guarana_va_list_resolve_method_invocation(struct Hjava_lang_Object *obj, Method *method, va_list args, slots *retval) {
   if (!METHOD_IS_STATIC(method))
+#ifdef DMETHOD_DESC
       method = DMETHOD_DESC(obj->dtable->method[method->idx]);
+#else
+      method = DMETHOD_ADDR(obj->dtable->method[method->idx]);
+#endif
   guarana_va_list_reify_method_invocation(obj, method, args, retval);
 }
 
@@ -513,7 +512,7 @@ guarana_reify_field_load(struct Hjava_lang_Object *o, Hjava_lang_Class *clazz, F
   NativeOperation *op;
   NativeResult *res;
 
-  op = guarana_new_native_operation(o, getCurrentThread(), 0);
+  op = guarana_new_native_operation(o, 0);
   op->kind |= op_field_read;
   op->op_id.field_id.clazz = clazz;
   op->op_id.field_id.field = field;
@@ -590,7 +589,7 @@ void
 guarana_reify_field_store(struct Hjava_lang_Object *o, struct Hjava_lang_Class *clazz, Field *field, jvalue *val) {
   NativeOperation *op;
 
-  op = guarana_new_native_operation(o, getCurrentThread(), 0);
+  op = guarana_new_native_operation(o, 0);
   op->kind |= op_field_write;
   op->op_id.field_id.clazz = clazz;
   op->op_id.field_id.field = field;
@@ -667,7 +666,7 @@ guarana_reify_array_load(struct Hjava_lang_Object *o, int index, jvalue *wrap) {
   NativeOperation *op;
   NativeResult *res;
 
-  op = guarana_new_native_operation(o, getCurrentThread(), 0);
+  op = guarana_new_native_operation(o, 0);
   op->kind |= op_array_read;
   op->op_id.index = index;
 
@@ -743,7 +742,7 @@ void
 guarana_reify_array_store(struct Hjava_lang_Object *o, int index, jvalue *val) {
   NativeOperation *op;
 
-  op = guarana_new_native_operation(o, getCurrentThread(), 0);
+  op = guarana_new_native_operation(o, 0);
   op->kind |= op_array_write;
   op->op_id.index = index;
   op->op_arg = *val;
@@ -824,7 +823,7 @@ guarana_reify_arraylength(struct Hjava_lang_Object *o) {
   if (!mo)
     return obj_length((Array*)o);
 
-  op = guarana_new_native_operation(o, getCurrentThread(), 0);
+  op = guarana_new_native_operation(o, 0);
   op->kind |= op_array_length;
 
   res = guarana_handle_operation(op);
@@ -834,76 +833,58 @@ guarana_reify_arraylength(struct Hjava_lang_Object *o) {
   return wrap.i;
 }
 
-iLock *
+void
 guarana_reify_monitorEnter(struct Hjava_lang_Object *o) {
-  if (o->meta_object) {
-    NativeOperation *op = guarana_new_native_operation(o, getCurrentThread(), 0);
-    op->kind |= op_monitor_enter;
+  NativeOperation *op = guarana_new_native_operation(o, 0);
+  op->kind |= op_monitor_enter;
 
-    guarana_handle_operation(op);
-
-    return getLock(o);
-  } else
-    return lockObjMutexDirect(o);
+  guarana_handle_operation(op);
 }
 
 void
 guarana_reify_monitorExit(struct Hjava_lang_Object *o) {
-  if (o->meta_object) {
-    NativeOperation *op = guarana_new_native_operation(o, getCurrentThread(), 0);
-    op->kind |= op_monitor_exit;
+  NativeOperation *op = guarana_new_native_operation(o, 0);
+  op->kind |= op_monitor_exit;
 
-    guarana_handle_operation(op);
-  } else
-    unlockObjMutexDirect(o);
-}
-
-void
-guarana_reify_monitorExit_known(iLock *l) {
-  Hjava_lang_Object *o = (Hjava_lang_Object *)l->address;
-  if (o->meta_object) {
-    NativeOperation *op = guarana_new_native_operation(o, getCurrentThread(), 0);
-    op->kind |= op_monitor_exit;
-
-    guarana_handle_operation(op);
-  } else
-    unlockObjKnownMutexDirect(l);
+  guarana_handle_operation(op);
 }
 
 struct Hjava_lang_Object*
-guarana_meta_init(struct Hjava_lang_Object *obj, struct Hjava_lang_Class *cc, struct Hjava_lang_Object *creator) {
+guarana_meta_init(struct Hjava_lang_Object *obj, struct Hjava_lang_Class *cc, struct Hjava_lang_Object *creator, errorInfo* einfo) {
   struct HBR_unicamp_Guarana_MetaObject *mo;
 
   if (creator && creator->meta_object) {
     jobject newMetaObject;
 
     if (!guaranaClass) {
-      errorInfo info;
-      guaranaClass = lookupClass("BR/unicamp/Guarana/Guarana", &info);
+      guaranaClass = lookupClass("BR/unicamp/Guarana/Guarana", NULL, einfo);
       if (!guaranaClass)
-	throwError(&info);
+	return 0;
     }
 
     if (!Guarana_blocker) {
-	errorInfo info;
-	Field *blockerField = lookupClassField(guaranaClass, utf8ConstNew("blocker", -1), true, &info);
+	Field *blockerField = lookupClassField(guaranaClass, utf8ConstNew("blocker", -1), true, einfo);
 	if (!blockerField)
-	    throwError(&info);
+	    return 0;
 	Guarana_blocker = *(jobject*)FIELD_ADDRESS(blockerField);
-	if (!Guarana_blocker)
-	    SignalError("java/lang/NullPointerException", "Guarana.blocker must not be null");
+	if (!Guarana_blocker) {
+	  postExceptionMessage(einfo, JAVA_LANG(NullPointerException),
+			       "Guarana.blocker must not be null");
+	  return 0;
+	}
     }
 
     obj->meta_object = (struct HBR_unicamp_Guarana_MetaObject*)Guarana_blocker;
     
+    /* FIXME: einfo? */
     newMetaObject = do_execute_java_method(creator->meta_object, "configure", "(Ljava/lang/Object;Ljava/lang/Object;)LBR/unicamp/Guarana/MetaObject;", 0, 0, obj, creator).l;
 
     if (!setMetaObjectOf) {
-      errorInfo info;
-      setMetaObjectOf = lookupClassMethod(guaranaClass, "setMetaObjectOf", "(Ljava/lang/Object;LBR/unicamp/Guarana/MetaObject;LBR/unicamp/Guarana/MetaObject;)V", &info);
+      setMetaObjectOf = lookupClassMethod(guaranaClass, "setMetaObjectOf", "(Ljava/lang/Object;LBR/unicamp/Guarana/MetaObject;LBR/unicamp/Guarana/MetaObject;)V", einfo);
       if (!setMetaObjectOf)
-	throwError(&info);
+	return 0;
     }
+    /* FIXME: einfo? */
     do_execute_java_method(0, 0, 0, setMetaObjectOf, 1, obj, (jobject)obj->meta_object, newMetaObject);
   }
 
@@ -917,40 +898,41 @@ guarana_meta_init(struct Hjava_lang_Object *obj, struct Hjava_lang_Class *cc, st
     struct Hjava_lang_Object *newObjMsg;
 
     if (!newObjMsgClass) {
-      errorInfo info;
-      newObjMsgClass = lookupClass("BR/unicamp/Guarana/NewObject", &info);
+      newObjMsgClass = lookupClass("BR/unicamp/Guarana/NewObject",
+				   NULL, einfo);
       if (!newObjMsgClass)
-	throwError(&info);
+	return 0;
     }
 
     if (!newObjConstr) {
-      errorInfo info;
       newObjConstr = lookupClassMethod(newObjMsgClass, constructor_name->data,
-				       "(Ljava/lang/Object;)V", &info);
+				       "(Ljava/lang/Object;)V", einfo);
       if (!newObjConstr)
-	throwError(&info);
+	return 0;
     }
 
-    newObjMsg = newObjectNoMessage(newObjMsgClass);
+    newObjMsg = newObjectNoMessage(newObjMsgClass, einfo);
+    if (!newObjMsg)
+      return 0;
+    /* FIXME: einfo? */
     do_execute_java_method(newObjMsg, 0, 0, newObjConstr, 0, obj);
     
     if (!guaranaClass) {
-      errorInfo info;
-      guaranaClass = lookupClass("BR/unicamp/Guarana/Guarana", &info);
+      guaranaClass = lookupClass("BR/unicamp/Guarana/Guarana", NULL, einfo);
       if (!guaranaClass)
-	throwError(&info);
+	return 0;
     }
 
     if (!broadcast) {
-      errorInfo info;
       broadcast = lookupClassMethod(guaranaClass,
 				    "broadcast",
 				    "(LBR/unicamp/Guarana/Message;Ljava/lang/Object;)V",
-				    &info);
+				    einfo);
       if (!broadcast)
-	throwError(&info);
+	return 0;
     }
 
+    /* FIXME: einfo? */
     do_execute_java_method(0, 0, 0, broadcast, 1,
 			   newObjMsg, (jobject)OBJECT_CLASS(obj));
   }
@@ -959,32 +941,38 @@ guarana_meta_init(struct Hjava_lang_Object *obj, struct Hjava_lang_Class *cc, st
 }  
 
 struct Hjava_lang_Object*
-guarana_newObject(struct Hjava_lang_Class *cc, struct Hjava_lang_Object *creator) {
-  return guarana_meta_init(newObjectNoMessage(cc), cc, creator);
+guarana_newObject(struct Hjava_lang_Class *cc, struct Hjava_lang_Object *creator, errorInfo* einfo) {
+  Hjava_lang_Object *obj = newObjectNoMessage(cc, einfo);
+  if (!obj)
+    return 0;
+  return guarana_meta_init(obj, cc, creator, einfo);
 }
 
 struct Hjava_lang_Object*
-newObject(struct Hjava_lang_Class *cc) {
-  return guarana_newObject(cc, 0);
+newObjectChecked(struct Hjava_lang_Class *cc, errorInfo* einfo) {
+  return guarana_newObject(cc, 0, einfo);
 }
 
 struct Hjava_lang_Object*
-guarana_newArray(struct Hjava_lang_Class *cc, int count, struct Hjava_lang_Object *creator) {
-  return guarana_meta_init(newArrayNoMessage(cc, count), cc, creator);
+guarana_newArray(struct Hjava_lang_Class *cc, int count, struct Hjava_lang_Object *creator, errorInfo *einfo) {
+  Hjava_lang_Object *array = newArrayNoMessage(cc, count, einfo);
+  if (!array)
+    return 0;
+  return guarana_meta_init(array, cc, creator, einfo);
 }
 
 Hjava_lang_Object*
-newArray(Hjava_lang_Class* eltype, int count)
+newArrayChecked(Hjava_lang_Class* eltype, int count, errorInfo* einfo)
 {
-  return guarana_newArray(eltype, count, (jobject)0);
+  return guarana_newArray(eltype, count, (jobject)0, einfo);
 }
 
 /* guarana_newMultiArray defined in object.c */
 
 Hjava_lang_Object*
-newMultiArray(Hjava_lang_Class* clazz, int* dims)
+newMultiArrayChecked(Hjava_lang_Class* clazz, int* dims, errorInfo *einfo)
 {
-  return guarana_newMultiArray(clazz, 0, dims);
+  return guarana_newMultiArray(clazz, 0, dims, einfo);
 }
 
 /*
@@ -1037,10 +1025,11 @@ walkOperation(Collector *collector, void* base, uint32 size)
 		    GC_markObject(collector,
 				  JREF_TO_ARGVEC(nop->op_arg.l)->args[i]);
 	    } else {
-		const char *sig = nop->op_id.method->signature->data + 1;
+		Method *meth = nop->op_id.method;
+		int i = 0, len = METHOD_NARGS(meth);
 		jvalue *argl = (jvalue*)nop->op_arg.l;
-		for(; *sig != ')'; ++sig, ++argl) {
-		    switch (*sig) {
+		for(; i < len; ++i, ++argl) {
+		    switch (*METHOD_ARG_TYPE(meth, i)) {
 		    case 'Z':
 		    case 'B':
 		    case 'S':
@@ -1052,14 +1041,7 @@ walkOperation(Collector *collector, void* base, uint32 size)
 			++argl;
 			break;
 		    case '[':
-			while (*sig == '[')
-			    ++sig;
-			if (*sig == 'L') { /* fall through */
-			case 'L':
-			    while (*sig != ';') {
-				sig++;
-			    }
-			}
+		    case 'L':
 			GC_markObject(collector, argl->l);
 			break;
 		    default:
