@@ -34,7 +34,6 @@ import kaffe.util.Ptr;
  */
 class NativeGraphics
   extends Graphics
-  implements ComponentListener
 {
 	Ptr nativeData;
 	int xOffset;
@@ -53,8 +52,7 @@ class NativeGraphics
  * component changes visibility and/or location/size.
  */
 	Component target;
-	int xTarget;
-	int yTarget;
+	GraphicsLink link;
 	NativeGraphics next;
 	static NativeGraphics cache;
 	static Object lock = new Object();
@@ -93,26 +91,6 @@ public void clipRect ( int x, int y, int width, int height ) {
 	Toolkit.graSetClip( nativeData, xClip, yClip, wClip, hClip);
 }
 
-public void componentHidden ( ComponentEvent evt ){
-	Toolkit.graSetVisible( nativeData, false);
-}
-
-public void componentMoved ( ComponentEvent evt ){
-	getClippedGraphics( this, target,
-	                    xOffset - xTarget, yOffset - yTarget,
-	                    xClip, yClip, wClip, hClip, false);
-}
-
-public void componentResized ( ComponentEvent evt ){
-	getClippedGraphics( this, target,
-	                    xOffset - xTarget, yOffset - yTarget,
-	                    xClip, yClip, wClip, hClip, false);
-}
-
-public void componentShown ( ComponentEvent evt ){
-	Toolkit.graSetVisible( nativeData, true);
-}
-
 public void copyArea ( int x, int y, int width, int height, int dx, int dy ){
 	Toolkit.graCopyArea( nativeData, x, y, width, height, dx, dy);
 }
@@ -124,7 +102,11 @@ public Graphics create () {
 									fgClr, bgClr, font, false);
 	if ( xClr != null )
 		g.setXORMode( xClr);
-		
+
+	if ( target != null ){
+		target.linkGraphics( g);
+	}
+
 	return g;
 }
 
@@ -149,7 +131,11 @@ public Graphics create ( int x, int y, int width, int height ) {
 							
 	if ( xClr != null )
 		g.setXORMode( xClr);
-		
+
+	if ( target != null ){
+		target.linkGraphics( g);
+	}
+
 	return g;
 }
 
@@ -164,18 +150,26 @@ public void dispose () {
 		return;
 	}
 
-	synchronized ( lock ) {
+	if ( target != null ) {
+		// make sure we don't get a interspersed updateLinkedGraphics()
+		synchronized ( target ) {
+			xClr = null;
+			font = null;
+			fgClr = null;
+			bgClr = null;
+
+			target.unlinkGraphics( this);
+			target = null;
+		}
+	}
+	else {
 		xClr = null;
 		font = null;
 		fgClr = null;
 		bgClr = null;
+	}
 
-		if ( target != null ){
-			for ( Component c=target; c != null; c = c.parent )
-				c.removeComponentListener( this);
-			target = null;
-		}
-
+	synchronized ( lock ) {
 		next = cache;
 		cache = this;
 	}
@@ -202,7 +196,7 @@ public void drawChars ( char data[], int offset, int length, int x, int y ){
 
 public boolean drawImage (Image img, int x, int y, Color bgcolor, ImageObserver observer) {
 	// if the image isn't loaded yet, start production and return false
-	if ( img.loadImage(-1, -1, observer) == false ) {
+	if ( Image.loadImage( img, -1, -1, observer) == false ) {
 		return (false);
 	}
 	else {
@@ -212,7 +206,7 @@ public boolean drawImage (Image img, int x, int y, Color bgcolor, ImageObserver
 }
 
 public boolean drawImage ( Image img, int x, int y, ImageObserver observer) {
-	if ( img.loadImage(-1, -1, observer) == false ) {
+	if ( Image.loadImage( img, -1, -1, observer) == false ) {
 		return (false);
 	}
 	else {
@@ -225,7 +219,7 @@ public boolean drawImage ( Image img, int x, int y, int width, int height, Color
 
 	// Load image if it's not loaded - we don't scale because we
 	// can do this while drawing.
-	if (img.loadImage(-1, -1, observer) == false) {
+	if ( Image.loadImage( img, -1, -1, observer) == false) {
 		return (false);
 	}
 
@@ -263,7 +257,7 @@ public boolean drawImage ( Image img,
 
 	// Load image if it's not loaded - we don't scale because we
 	// can do this while drawing.
-	if (img.loadImage(-1, -1, observer) == false) {
+	if (Image.loadImage( img, -1, -1, observer) == false) {
 		return (false);
 	}
 
@@ -272,8 +266,15 @@ public boolean drawImage ( Image img,
 		return (false);
 	}
 
-	// We don't create a scaled Image instance since we can draw scaled
-	drawImgScaled( img, dx0, dy0, dx1, dy1, sx0, sy0, sx1, sy1, bgColor);
+	if ( ((sx1 - sx0) == (dx1 - dx0)) && ((sy1 - sy0) == (dy1 - dy0)) ) {
+		// bozo. don't you know about the costs of image scaling?
+		drawImg( img, dx0, dy0, sx0, sy0, (sx1 - sx0), (sy1 - sy0), bgColor);
+	}
+	else {
+		// We don't create a scaled Image instance since we can draw scaled
+		drawImgScaled( img, dx0, dy0, dx1, dy1, sx0, sy0, sx1, sy1, bgColor);
+	}
+	
 	return true;
 }
 
@@ -284,10 +285,14 @@ public boolean drawImage ( Image img,
 	return drawImage(img, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, null, observer);
 }
 
-void drawImg ( Image img,	int x, int y, int sx, int sy,	int width, int height, Color background ){
+void drawImg ( Image img,	int x, int y, int sx, int sy,	int width, int height, Color background ){	
 	if ( (img.flags & Image.BLOCK_FRAMELOADER) != 0 ){
 		img.activateFrameLoader();
 	}
+	else if ( (img.flags & Image.IS_ANIMATION) != 0 ) {
+		Toolkit.imgSetFrame( img.nativeData, 0);
+	}
+	
 	Toolkit.graDrawImage( nativeData, img.nativeData,
 	                      sx, sy, x, y, width, height,
 		                    (background == null) ? -1 : background.nativeValue);
@@ -546,6 +551,8 @@ static NativeGraphics getGraphics ( Object target, Ptr tgtData, int tgtType,
 	g.fgClr    = fg;
 	g.bgClr    = bg;
 
+if ( fg == null ){ Thread.currentThread().dumpStack();}
+
 	g.nativeData = Toolkit.graInitGraphics( g.nativeData, tgtData, tgtType,
 	                                        xOffset, yOffset,
 	                                        xClip, yClip, wClip, hClip,
@@ -592,7 +599,7 @@ void paintChild ( Component c, boolean isUpdate ) {
 		// since some apps might even call update/paint explicitly (again, bad!). But
 		// we want to support at least those who call super.paint()
 		// The PAINT vs. UPDATE problem also shows up in Container.emitRepaints
-		Toolkit.eventQueue.repaint( PaintEvt.PAINT, c, clx, cly, clw, clh);
+		Toolkit.eventQueue.postPaintEvent( PaintEvt.PAINT, c, clx, cly, clw, clh);
 	}
 	else {
 		NativeGraphics g = getGraphics( this, nativeData, TGT_TYPE_GRAPHICS,
@@ -626,6 +633,16 @@ public void setClip ( Shape clip ){
 }
 
 public void setClip ( int x, int y, int width, int height ) {
+	if ( target != null ) {
+		// be paranoid, native widgets automatically clip to their boundaries
+		if ( x < 0 ) x = 0;
+		if ( y < 0 ) y = 0;
+		if ( (x + width) > target.width )
+			width = target.width - x;
+		if ( (y + height) > target.height ) 
+			height = target.height - y;
+	}
+
 	xClip      = x;
 	yClip      = y;
 	wClip  = width;
@@ -685,26 +702,6 @@ public void setPaintMode() {
 	}
 }
 
-void setTarget ( Component tgt ) {
-	target = tgt;
-
-	// We have to monitor not only reshapes of our target, but also of all of its
-	// parents (since this would cause a change in xOffset/yOffset, too). Note that we
-	// cannot simply stop at the last non-native, since some windowing systems 
-	// use screen coordinates for drawing (PEG)
-	for ( Component c = tgt; c != null; c = c.parent )
-		c.addComponentListener( this);
-	
-	// We need this to update the relative offset after a target/parent move.
-	// Not safe in case this isn't directly following a Component.getGraphics
-	// (then again, it is just called by us)
-	xTarget = xOffset;
-	yTarget = yOffset;
-	
-	if ( (target.flags & Component.IS_VISIBLE) == 0 )
-		Toolkit.graSetVisible( nativeData, false);
-}
-
 public void setXORMode ( Color newXorClr ) {
 	if ( newXorClr != xClr ) {
 		xClr = newXorClr;
@@ -713,8 +710,10 @@ public void setXORMode ( Color newXorClr ) {
 }
 
 public String toString() {
-	return getClass().getName() + " [" + xOffset + ',' + yOffset +
-	                     " clip:" + xClip+','+yClip+' '+wClip+','+hClip + ']';
+	return  getClass().getName() +
+	           ' ' + hashCode() + 
+	           " [" + xOffset + ',' + yOffset +
+	           " clip:" + xClip+','+yClip+' '+wClip+','+hClip + ']';
 }
 
 public void translate ( int x, int y ) {
