@@ -1,4 +1,4 @@
-// $Id: ModifyOperation.java,v 1.2 1998/07/24 09:30:53 oliva Exp $
+// $Id: ModifyOperation.java,v 1.3 1999/05/29 21:51:14 oliva Exp $
 
 /* Copyright 1998 Alexandre Oliva <oliva@dcc.unicamp.br>
  *
@@ -29,19 +29,17 @@ import BR.unicamp.Guarana.*;
  * MetaObject can have access to it using its OperationFactory.
  *
  * @author Alexandre Oliva
- * @version $Revision: 1.2 $
+ * @version $Revision: 1.3 $
  */
 public class ModifyOperation extends MetaObject {
-    static final java.lang.reflect.Method toString;
-    static final java.lang.reflect.Field callSuper;
-    static {
-	try { toString =
-		  Object.class.getDeclaredMethod("toString", new Class[0]);
-	} catch (NoSuchMethodException e) { toString = null; }
-	try { callSuper =
-		  ModifyOperationBase.class.getDeclaredField("callSuper");
-	} catch (NoSuchFieldException e) { callSuper = null; } 
-    }
+    static final java.lang.reflect.Method toString = toStringInit();
+    private static java.lang.reflect.Method toStringInit() { try {
+	return Object.class.getDeclaredMethod("toString", new Class[0]);
+    } catch (NoSuchMethodException e) {} return null; }
+    static final java.lang.reflect.Field callSuper = callSuperInit();
+    private static java.lang.reflect.Field callSuperInit() { try {
+	return ModifyOperationBase.class.getDeclaredField("callSuper");
+    } catch (NoSuchFieldException e) {} return null; }
     OperationFactory opf = null;
     public void initialize(final OperationFactory opf, final Object o)
     { this.opf = opf; }
