@@ -1,6 +1,6 @@
-// $Id: Reconfigure.java,v 1.2 1998/09/10 20:16:57 oliva Exp $
+// $Id: Reconfigure.java,v 1.4 1999/10/11 17:59:39 oliva Exp $
 
-/* Copyright 1998 Alexandre Oliva <oliva@dcc.unicamp.br>
+/* Copyright 1998,1999 Alexandre Oliva <oliva@lsd.ic.unicamp.br>
  *
  * This file is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@ class ReconfigureBase {}
  * meta-configurations, meta-object replacement and removal.
  *
  * @author Alexandre Oliva
- * @version $Revision: 1.2 $
+ * @version $Revision: 1.4 $
  */
 public class Reconfigure extends ReconfigureBase {
     public static void main(String[ ] argv) {
@@ -44,7 +44,7 @@ public class Reconfigure extends ReconfigureBase {
 	Guarana.reconfigure(o, null, b);    // a replaced with b
 	Guarana.reconfigure(o, b, a);       // b replaced with a
 	Guarana.reconfigure(o, b, a);       // ignored by a
-	Guarana.reconfigure(o, null, null); // b replaced with null
+	Guarana.reconfigure(o, null, null); // a replaced with null
 	Guarana.reconfigure(o, a, b);       // ignored by the kernel
     }
 }
